//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef HISTEXPLORERVIEW_H
#define HISTEXPLORERVIEW_H

#include <QSharedPointer>
#include <QList>
#include <QMap>

#include "ui_HistExplorerViewUi.h"
#include "hist_manager/histmanager.h"
#include "histdialog.h"


typedef QSharedPointer<HistDialog> HistDialogPtr;

typedef QList<HistDialog *> HistDialogList;
class HistTypeMap: public QMap<int, hist_type_t> {};

class HistExplorerView : public QWidget, public Ui::HistExplorerViewUi
{
    Q_OBJECT
public:
    explicit HistExplorerView(QWidget *parent = nullptr);
    void recreateHistograms (const HistList &hl);
    QString currentHistName() const;

public Q_SLOTS:
    void updateView();
    void onHistogramsChanged(const HistList &hl);

signals:
    void clearAllHists();
    void clearHist(const hist_index_t &index);
    void saveAllHists();
    void saveHist(const hist_index_t &index);

protected:
//    void showhist(int id);
//    HistDialog *findhist(int id);
//    const HistDialog *const_findhist(int id) const;
//    HistDialog *current_hist();
//    const HistDialog *const_current_hist() const;
    void update_mainhist_controls();
//    void copyHistData(const HistPlot *hp);
    void update_current_main_hist();
    void update_checkBox_autoRange();
    void update_checkBox_autoFit();
//    void setStatus(const QString &s);
    void update_current_main_hist_bh(bool enable_controls);
//    void clear_current_main_hist();

protected Q_SLOTS:
    void showEvent(QShowEvent *event) override;

private Q_SLOTS:
    void setCurrentHistType(int);
    void setCurrentChannel(int);
    void setAutoRange(bool state);
    void setAutoFit(bool state);
    void clearCurrentHist();
    void save();
    void saveAll();

private:
    th1 currentHist() const;
    hist_type_t currentHistType() const;
    int currentHistChannel() const;
    hist_index_t currentHistIndex() const;
    int findIndexByType(const hist_type_t &t) const;
    void updateComboBox(const HistTypeList &list);
    HistDialogList hists;
    HistList histList;
};

#endif // HISTEXPLORERVIEW_H
