
#include <QVector>
#include <qwt_math.h>
#include <qwt_scale_engine.h>
#include <qwt_plot_curve.h>
#include <qwt_plot_grid.h>
#include <qwt_plot_marker.h>

#include "HistPlotBase.h"
#include "histplot_p.h"

HistPlotBase::HistPlotBase (QWidget * p)
    : QwtPlot (p), d(new HistPlotPrivate)
{
    //  setAutoLegend (FALSE);

    // grid
    d->grid.attach(this);
    d->grid.setMajorPen(QPen(Qt::darkGray, 0, Qt::DotLine));

    d->curv.attach(this);
    d->curv.setTitle("Data");
    d->curv.setPen (QPen (Qt::black, 2));
    d->curv.setStyle (QwtPlotCurve::Steps);
    d->curv.setBrush (Qt::blue);
    //  curv->setSymbol(QwtSymbol(QwtSymbol::XCross, black, black, QSize(2,2)));


    // fit function
    d->curv_f.attach(this);
    d->curv_f.setTitle("Fit");
    d->curv_f.setPen (QPen (Qt::black, 1));
    d->curv_f.setStyle (QwtPlotCurve::Lines);
    //	curv_f->setCurveFitter(QwtSplineCurveFitter);

    //  Insert zero line at y = 0
    d->mark_y0.attach(this);
    d->mark_y0.setYValue(0.0);
    d->mark_y0.setLineStyle(QwtPlotMarker::HLine);
    d->mark_y0.setLabelAlignment(Qt::AlignRight | Qt::AlignBottom);
    d->mark_y0.setLinePen(QPen(Qt::green, 0, Qt::DashDotLine));

    //  long mY = insertLineMarker ("", QwtPlot::yLeft);
    //  setMarkerYPos (mY, 0.0);

    // Set axis titles
    //    setAxisTitle(QwtPlot::xBottom, "Time/seconds");
    //    setAxisTitle(QwtPlot::yLeft, "Values");

    //    setAxisOptions(QwtPlot::yLeft, QwtAutoScale::Floating | QwtAutoScale::Logarithmic);

}

HistPlotBase::~HistPlotBase()
{
    d->grid.detach();
    d->curv.detach();
    d->curv_f.detach();
    d->mark_y0.detach();
    delete d;
}

constexpr size_t MAX_HIST_BINS = 1000;

static std::vector<double> merge_bins(const std::vector<double> &d)
{
    if (d.size() < MAX_HIST_BINS)
        return d;
    size_t factor = d.size() / MAX_HIST_BINS;
    size_t new_size = (d.size() + factor - 1) / factor;
    std::vector<double> v;
    v.resize(new_size);
    for (size_t i=0; i<d.size(); i++)
        v[i / factor] += d[i];
    return v;
}

void HistPlotBase::setpoints (const std::vector<double> &inx, const std::vector<double> &iny)
{
    const size_t len = std::min(inx.size(), iny.size());
    d->px = QVector<double>::fromStdVector(inx);
    d->py = QVector<double>::fromStdVector(iny);
    if (len == 0)
        d->curv.hide();
#if QWT_VERSION >= 0x060000
    d->curv.setSamples(d->px, d->py); // qwt 6
#else
    d->curv.setData(d->px, d->py); // qwt 5
#endif
    if (len > 0)
        d->curv.show();
}

QwtPlotCurve *HistPlotBase::getHistCurve()
{
    return &d->curv;
}

QwtPlotCurve *HistPlotBase::getFitCurve()
{
    return &d->curv_f;
}

void HistPlotBase::set_fit_points (const std::vector<double> &inx, const std::vector<double> &iny)
{
    const size_t len = std::min(inx.size(), iny.size());
    d->fpx = QVector<double>::fromStdVector(inx);
    d->fpy = QVector<double>::fromStdVector(iny);
    if (len == 0) {
        d->curv_f.hide();
    }
#if QWT_VERSION >= 0x060000
    d->curv_f.setSamples (d->fpx, d->fpy); // qwt 6
#else
    d->curv_f.setData (d->fpx, d->fpy); // qwt 5
#endif
    if (len > 0)
        d->curv_f.show();
}

//std::vector<double> HistPlotBase::getpointsx() const
//{
//    return d->px.toStdVector();
//}

//std::vector<double> HistPlotBase::getpointsy() const
//{
//    return d->py.toStdVector();
//}

//std::vector<double> HistPlotBase::getfitpointsx() const
//{
//    return d->fpx.toStdVector();
//}

//std::vector<double> HistPlotBase::getfitpointsy() const{
//    return d->fpy.toStdVector();
//}

/*
void HistPlotBase::autorange() {

  if (!px||!py||(len<=0)) return;

  double maxvalue = 0;
  for (int i = 0; i < len; i++) {
    if (py[i] > maxvalue) maxvalue = py[i];
  }

  double threshold = maxvalue * 0.025;
  if (threshold < 2) threshold = 2;

    double pxmin = px[0], pxmax = px[len-1];
    for (int i = 1; i < len; i++)
      if (py[i] > threshold) {
        pxmin = px[i-1];
        break;
      }
    for (int i = len - 3; i > 0; i--)
      if (py[i] > threshold) {
        pxmax = px[i+2];
        break;
      }
    if (pxmin<pxmax) {
          ar_xmin = pxmin;
          ar_xmax = pxmax;
      setAxisScale (QwtPlot::xBottom, pxmin, pxmax, 0);
        }
}
*/

void HistPlotBase::set_x_range(double xmin, double xmax)
{
    setAxisScale (QwtPlot::xBottom, xmin, xmax, 0);
}

void HistPlotBase::get_x_range(double *xmin, double *xmax) const
{
    auto d = axisScaleDiv(QwtPlot::xBottom);
        *xmin = d.lowerBound();
        *xmax = d.upperBound();
}
/*
void HistPlotBase::autorange() {
  double ar_min=0, ar_max=0;
  h->autorange();
  h->getrange(&ar_min, &ar_max);
  set_x_range(ar_min, ar_max);
}
*/
