/*
**    Copyright 2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#ifndef WAVEFORMCONTROLLER_H
#define WAVEFORMCONTROLLER_H

#include <QObject>
#include <QMap>

#include "qxw/WaveformData.h"
#include "qxw/WaveformFilter.h"
#include "dsplib/fir.h"
#include "global_types.h"

class WaveformModel_ADC;
class WaveformViewAdvanced;
class AdcmEvent;
class WaveformAverager;

class WaveformController : public QObject
{
    Q_OBJECT
public:
    explicit WaveformController(WaveformModel_ADC *m, QObject *parent = nullptr);
    ~WaveformController();
    void setView(WaveformViewAdvanced *v);
    void setFilterData(int channel, const fir_t<decoder_fir_coef_t> &fir);
    void setFullScale(int channel, double value);

Q_SIGNALS:

public Q_SLOTS:
    void setOffsetComp(bool);
    void setFiltering(bool);
    void setAveraging(bool);
    void setFFT(bool);
    void actionClear();
    void actionExport();
    void updateView();
    void updateViewControls();
    void modelChanged();

protected:
    WaveformDataAll makeWaveform(const WaveformDataAll &wda) const;
    WaveformDataAll getPlotData() const;
    WaveformDataAll linear_to_dBFS(const WaveformDataAll &wda) const;

private:
    WaveformModel_ADC * const model;
    WaveformViewAdvanced * view = nullptr;
    WaveformAverager * const averager;
    QMap<int, fir_t<decoder_fir_coef_t> > filter;
    QMap<int, double> fullScale;
    WaveformFilterFFT waveformFilterFFT;
    WaveformFilterFIR waveformFilterFIR;
    bool offsetComp = false;
    bool selectFilter = false;
    bool enableAverager = false;
    bool enableFFT = false;
};

#endif // WAVEFORMCONTROLLER_H
