
#include <QMap>
#include <QString>
#include <QList>
#include <QStringList>

#include "adcmlib/adcm_decstat.h"
#include "reports.h"
namespace  {
int runMs=0;
}

QString make_line(const QString &name, quint64 v1, quint64 v2, quint64 v3, bool is_error = false)
{
    if (is_error && v1==0 && v2==0) return QString(); // skip empty
    QString s;
    s +="<tr><td>";
    s += name;
    s += "</td><td>";
    bool red1 = is_error && v1>0;
    if (red1) s+= "<font color=\"red\">";
    s += QString::number(v1);
    if (red1) s+= "</font>";

    s += "</td><td>";
    bool red2 = is_error && v2>0;
    if (red2) s+= "<font color=\"red\">";
    s += QString::number(v2);
    if (red2) s+= "</font>";

    s += "</td><td>";
    if (red2) s+= "<font color=\"red\">";
    if(runMs)
        s += QString::number((double)v3*1000/runMs, 'f', 2);

    if (red2) s+= "</font>";

    s +="</td></tr>";
    return s;
}

QString make_decst_report(const adcm_decstat_t &st, const adcm_decstat_t &stsum, const adcm_decstat_t &stDiff, int runMSec)
{
    runMs = runMSec;
    QString text;

    //  typedef QMap<QString, QString> map_t;
    typedef QStringList map_t;
    QList<QStringList> allTable;
    QList<QString> headers;
    //  typedef QValueList<QString> line_t;
    //  QValueList<line_t> x;
    headers.push_back("RAW Data Summary");
    headers.push_back("RAW Block Summary");
    headers.push_back("ADC Data Errors");
    headers.push_back("Signal Processor Summary");
    headers.push_back("Event Manager Summary");

    //  line_t ll = line_t(QString(""), QString(""));
    //  x.push_back(ll);

    QStringList section;

    section.clear();
    section += make_line("Data size, KB",   st.nw / 256, stsum.nw / 256, stDiff.nw / 256);
    section += make_line("Buffer overruns",   st.overruns,  stsum.overruns, stDiff.overruns, true);
    section += make_line("Raw stream resync",   st.n_resync,  stsum.n_resync, stDiff.n_resync, true);
    //  m += make_line("Frames",      st.nf, stsum.nf);
    section += make_line("ADC data frames", st.nf_dadc, stsum.nf_dadc, stDiff.nf_dadc);
    section += make_line("Ctrl req frames", st.nf_creq, stsum.nf_creq, stDiff.nf_creq);
    section += make_line("Ctrl ack frames", st.nf_cack, stsum.nf_cack, stDiff.nf_cack);
    section += make_line("Status  frames", st.nf_stat, stsum.nf_stat, stDiff.nf_stat);
    section += make_line("Unknown frames",  st.nf_unkn, stsum.nf_unkn, stDiff.nf_unkn, true);
    section += make_line("Frames with CRC error",   st.nf_crc,  stsum.nf_crc,  stDiff.nf_crc, true);
    section += make_line("Frames with sequence error",   st.nf_seq,  stsum.nf_seq,  stDiff.nf_seq, true);
    allTable.push_back(section);

    section.clear();
    //  m += make_line(" Total blocks",      st.nblk_all,      stsum.nblk_all);
    section += make_line("ADC data blocks (type 0)",             st.nblk_adc_0,      stsum.nblk_adc_0,      stDiff.nblk_adc_0);
    section += make_line("ADC data blocks (type 1)",             st.nblk_adc_1,      stsum.nblk_adc_1,      stDiff.nblk_adc_1);
    section += make_line("ADC data blocks (type 2)",             st.nblk_adc_2,      stsum.nblk_adc_2,      stDiff.nblk_adc_2);
    //  m += make_line("Stat data blocks",            st.nblk_stat,     stsum.nblk_stat);
    section += make_line("Unknown data blocks",         st.nblk_unkn,     stsum.nblk_unkn,     stDiff.nblk_unkn, true);
    section += make_line("Bad (corrupt) data blocks",st.nblk_bad,      stsum.nblk_bad,      stDiff.nblk_bad, true);
    allTable.push_back(section);

    section.clear();
    section += make_line("Length errors",      st.nblk_adc_len,  stsum.nblk_adc_len,  stDiff.nblk_adc_len, true);
    section += make_line("Roller errors",      st.nblk_adc_roll, stsum.nblk_adc_roll, stDiff.nblk_adc_roll, true);
    section += make_line("Bad event number",   st.nblk_adc_evnum, stsum.nblk_adc_evnum, stDiff.nblk_adc_evnum, true);
    section += make_line("Duplicate channel errors", st.nblk_adc_dup,  stsum.nblk_adc_dup,  stDiff.nblk_adc_dup, true);
    allTable.push_back(section);

    section.clear();
    bool b = false; // dsp error severity
    section += make_line("Total waveforms",      st.dsp_good_waveform,     stsum.dsp_good_waveform,     stDiff.dsp_good_waveform);
    section += make_line("Total pulses",         st.dsp_total,     stsum.dsp_total,     stDiff.dsp_total);
    section += make_line("Bad waveform length",  st.dsp_bad_len,   stsum.dsp_bad_len,   stDiff.dsp_bad_len,   true);
    section += make_line("Threshold not found",  st.dsp_thr_nf,    stsum.dsp_thr_nf,    stDiff.dsp_thr_nf,    true);
    section += make_line("Baseline insuff. data",st.dsp_bl_range,  stsum.dsp_bl_range,  stDiff.dsp_bl_range,  b);
    section += make_line("Integral insuff. data",st.dsp_int_range, stsum.dsp_int_range, stDiff.dsp_int_range, b);
    section += make_line("Positive clipping",    st.dsp_clip_pos,  stsum.dsp_clip_pos,  stDiff.dsp_clip_pos,  b);
    section += make_line("Negative clipping",    st.dsp_clip_neg,  stsum.dsp_clip_neg,  stDiff.dsp_clip_neg,  b);
    section += make_line("Pksrch iinsuff. data", st.dsp_pk_range,  stsum.dsp_pk_range,  stDiff.dsp_pk_range,  b);
    section += make_line("Fit failed",           st.dsp_fit_fail,  stsum.dsp_fit_fail,  stDiff.dsp_fit_fail,  true);
    section += make_line("Fit bad",              st.dsp_fit_bad,   stsum.dsp_fit_bad,   stDiff.dsp_fit_bad,   b);
    section += make_line("Fit ok",               st.dsp_fit_ok,    stsum.dsp_fit_ok,    stDiff.dsp_fit_ok);
    allTable.push_back(section);

    section.clear();
    section += make_line("Total events",         st.ev_total,    stsum.ev_total,    stDiff.ev_total);
    section += make_line("Dropped by dT cut",    st.ev_dt_cut,    stsum.ev_dt_cut,    stDiff.ev_dt_cut);
    allTable.push_back(section);

    text += "<table width=90%>";
    text += "<tr><td/><td>Current</td><td>Summed</td><td>Rate,Hz</td></tr>";
    for(int n=0;n<allTable.count();n++) {
        QString tmp_text;
        tmp_text += "<tr><td colspan=4>";
        tmp_text += "<b>";
        tmp_text += headers[n];
        tmp_text += "</b>";
        tmp_text += "</td></tr>\n";
        QStringList l;
        for (auto &s : allTable[n]) {
            if (!s.isEmpty()) l += s;
        }
        tmp_text += l.join("\n");
        if (!l.isEmpty()) text += tmp_text;
    }
    text += "</table>\n";
    return text;
}
