
#include <cstddef>
#include <ctime>
#ifdef __USE_POSIX
#include <sys/param.h>
#endif

#include <iostream>
#include <QFile>
#include <QStringList>
#include <QTextStream>
#include <QDebug>

#include "sysmon_impl.h"

const int cache_time = 1; // seconds

SystemMonitorImpl::SystemMonitorImpl()
{
}

void SystemMonitorImpl::check_update() const
{
    QDateTime now = QDateTime::currentDateTime();
    if (last_updated.secsTo(now) >= cache_time) read();
}

QString SystemMonitorImpl::get(const QString & name) const
{
    check_update();
    return data[name];
}

data_t SystemMonitorImpl::get_all() const
{
    check_update();
    return data;
}

void SystemMonitorImpl::read_meminfo() const
{
    QMap<QString,QString> memstats;

    QFile file( "/proc/meminfo" );
    if ( file.open( QIODevice::ReadOnly ) ) {
        QTextStream stream( &file );
        QString line;
        while ( !stream.atEnd() ) {
            line = stream.readLine(); // line of text excluding '\n'
            QString name  = line.section(":",0,0).trimmed().toLower();
            QString rhs = line.section(":",1,1).trimmed();
            QString value = rhs.section(" ",0,0);
            //            QString units = rhs.section(" ",1,1);
            if (!value.isEmpty())
                memstats[name]=value;

        }
        file.close();
    }

    data["memtotal"] = memstats["memtotal"];
    data["memfree"]  = memstats["memfree"];
    /*
  size_t memtotal = memstats["memtotal"].toULong();
  size_t memfree = memstats["memfree"].toULong();
  if (memtotal && memfree)
        data["MemoryUsage"] = QString::number((memtotal-memfree)/memtotal);
*/
}

void SystemMonitorImpl::read_stat() const
{
#ifdef __USE_POSIX
    sysstats_t sysstats;
    QFile file( "/proc/stat" );
    if ( file.open( QIODevice::ReadOnly ) ) {
        QTextStream stream(&file);
        while (true) {
            QString line = stream.readLine(); // line of text excluding '\n'
            if (line.isNull())
                break;
            const QStringList all = line.split(" ");
            if (all.count() > 1) {
                const QString name = all[0].toLower();
                //	std::cout <<"1: "<<name<<" = "<<all.join(",")<<std::endl;

                //            all.pop_front();
                sysstats[name]=all;
            }
        }
        file.close();
    }
//    Q_FOREACH(QString key, sysstats.keys()) {
//        qDebug() << QString("sysstats[%1] = ").arg(key) << sysstats.value(key);
//    }

    QMap<int, QString> descr;
    descr[0] = "item";
    descr[1] = "user";
    descr[2] = "nice";
    descr[3] = "sys";
    descr[4] = "idle";
    descr[5] = "iowait";
    descr[6] = "irq";
    descr[7] = "softirq";
    QString item = "cpu";
//    sysstats_t current;
    const QStringList &current_values = sysstats[item];
    const QStringList &last_values = last_sysstats[item];
    int maxCount = std::min(current_values.count(), last_values.count());
    for(int i=0; i<maxCount; i++) {
        bool ok;
        unsigned long last = last_values[i].toULong(&ok);
        if (!ok) continue;
        unsigned long curr = current_values[i].toULong(&ok);
        if (!ok) continue;
        unsigned long value;
        if (last<=curr) value = curr-last;
        else value = ULONG_MAX-last-curr;
        const QString name = item+"_"+descr[i];
        //        double pcnt = 100.*value / HZ; // BUG: HZ not defined correctly
        data[name] = QString::number(value);
        //	std::cout <<name<<" = "<<value<<std::endl;
    }
//    qDebug() << data;

    last_sysstats[item] = sysstats[item];

    /*
        left_popup->changeItem(left_popup->idAt(1), i18n("User\t: ") + QString::number(user * 100 / total) + "%");
        left_popup->changeItem(left_popup->idAt(3), i18n("Sys\t: ") + QString::number(sys  * 100 / total) + "%");
        left_popup->changeItem(left_popup->idAt(2), i18n("Nice\t: ") + QString::number(nice * 100 / total) + "%");
        left_popup->changeItem(left_popup->idAt(4), i18n("Idle\t: ") + QString::number(idle * 100 / total) + "%");
        left_popup->changeItem(left_popup->idAt(6), i18n("Ram\t: ") + QString::number(mem_total - mem_free)   + " / " + QString::number(mem_total)  + i18n(" MB"));
        left_popup->changeItem(left_popup->idAt(7), i18n("Swap\t: ") + QString::number(swap_total - swap_free) + " / " + QString::number(swap_total) + i18n(" MB"));
*/
#endif
}

void SystemMonitorImpl::read() const
{
    mutex.lock();
    read_meminfo();
    read_stat();
    last_updated = QDateTime::currentDateTime();
    mutex.unlock();
}
