
#include <cassert>

#include "adcm16.h"
#include "adcmlib/adcm_ev.h"
#include "adcmlib/AdcmChbuf.h"
#include "waveformmodeladc.h"
#include "config/allconfig.h"
#include "adcmraw/adcm_mem.h"

//! return time scale point for channel x, step i
double waveform_x(double step, int latency, int i)
{
    return (i-latency) * step;
}

void WaveformModel_ADC::addEvent (const AdcmEvent &ev, const conf_mem_t &conf_mem)
{
//    // channel offsets
//    std::vector<double> offs;
//    for (int c = 0; c < cfg.dec.calzero.nent.size(); c++) {
//        offs.push_back(cfg.dec.calzero.get(c));
//    }
    //  bool use_offset = isDisplayBipolar();
    //  bool use_offset = true;

    //  WaveformDataAll &wd(cache);
//    WaveformDataAll wda;
    //  wd.resize(ev->ncb);
    //  adcm_print_event(ev.get());
    wda.clear();
    wda_conv.clear();
    for (auto c : ev.channel.keys()) {
        const adcm_chbuf_t &cb = ev.channel[c];
//        WaveformData wd;
//        wd.c = c;
//        wd.baseline = cb.w_calzero;
        const double adc_step = ev.format.step;
        {
            const auto &waveform_raw = cb.w;
            WaveformData wd_raw(QString("raw %1").arg(c), c);
            wd_raw.baseline = cb.w_calzero;
            wd_raw.p.reserve(waveform_raw.size());
            for (int i=0; i<waveform_raw.size(); i++) {
                point_t p;
                p.x = (i - cb.w_latency) * adc_step * 1e6;
                p.y = waveform_raw[i];
                wd_raw.p.push_back(p);
            }
            wda[c] = wd_raw;
        }
        {
            const auto &waveform_conv = cb.conv.w;
            WaveformData wd_conv(QString("conv %1").arg(c), c);
            // wd_conv.baseline = cb.w_calzero;
            wd_conv.p.reserve(waveform_conv.size());
            for (int i=0; i<waveform_conv.size(); i++) {
                point_t p;
                p.x = (i - cb.w_latency) * adc_step * 1e6;
                p.y = waveform_conv[i];
                wd_conv.p.push_back(p);
            }
            for (auto i : cb.conv.peaks) {
                double ptx = (i.pt - cb.w_latency) * adc_step * 1e6; // convert to microseconds
                wd_conv.t.push_back(ptx);
            }

            wda_conv[c] = wd_conv;
        }
    }
    /*
  // TODO: move to WaveformModelAdcm
// Add match window marks
  WaveformData marks;
  marks.c = -1; // negative id will be painted with black
  const double step = cfg->adc.step[0] * 0.001; // ns->us
  const bool is_co_trigger = cfg->adc.trigbits & ADCM_TRIGBIT_INT;
  if (is_co_trigger) {
        const double w1 = (- cfg->adc.prewin)*step;
        const double w2 = (- cfg->adc.prewin + cfg->adc.matchwin) * step;
        marks.t.push_back(w1 - 0.5*step);
    marks.t.push_back(w2 + 0.5*step);
  }
  wd[marks.c] = marks;
*/
//    set(wda);
    emit dataChanged();
}
