/*
**    Copyright 2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ADCMAPP_H
#define ADCMAPP_H

#include <QObject>

class ConfigManager;
class AdcInstance;
class DecodeManager;
class EventListFiller;
class RawDataWriter;
class RemoteControlServer;
class HistMerger;
class HistList;
class QThread;
struct hist_index_t;

class AdcmApp : public QObject
{
    Q_OBJECT
public:
    explicit AdcmApp(QObject *parent = nullptr);
    ~AdcmApp() final;

Q_SIGNALS:
    void histogramsChanged(const HistList &r);
    void clearAllHists();
    void clearHist(const hist_index_t &index);

public Q_SLOTS:

public:
    ConfigManager * const configManager;
    AdcInstance * const adcInstance;
    DecodeManager * const decodeManager;
    EventListFiller * const filler;
    RawDataWriter * const writeInstance;
    RemoteControlServer * const remoteControlServer;
    HistMerger * const histMerger;

    // threads
    QThread * const adcThread;
    QThread * const decodethread;
    QThread * const fillerThread;
    QThread * const writethread;
    QThread * const remoteControlThread;

private Q_SLOTS:
    void startThreads();
    void onHistogramsChanged(const HistList &r);
private:
    void stopThreads();

};

#endif // ADCMAPP_H
