#ifndef DECODEWORKER_H
#define DECODEWORKER_H

#include <QObject>

class AdcmEvent;
//class adcm_el_t;
#include "adcmlib/adcm_el.h"
#include "adcmlib/adcm_dec.h"

class adcm_decstat_t;

class HistManager;
class storage;

class FrameList;
class HistList;
class AdcmDataBlockDecoder;
class WaveformDecoder;
struct adcm_counters_t;
class conf_mem_t;
class AdcConfig;
class DecoderConfig;
class StorageConfig;
class CalZeroConfig;
class CalAvgConfig;
class QTimer;
class QMutex;
class QSemaphore;
struct hist_index_t;

class DecodeWorker : public QObject
{
    Q_OBJECT

public:

    explicit DecodeWorker (int workerIndex,
                      QSemaphore * _idleWorkers,
                      storage * _stor,
                      DecoderConfig * decConf,
                      adcm_el_t *,
                      bool storEnabled);
    ~DecodeWorker () override;

    void setStorageConfig(bool);
    void clear(); // Clears stats & hist
    void reset();
    unsigned int getWorkerIndex() const { return workerIndex; }
    bool isIdle();
    void lockWorker();
    void unlockWorker();
    HistManager *getHistManager() { return hmgr; }
    void setHistEnabled(bool);
//    void pause(); // not used yet
//    void resume(); // not used yet

public Q_SLOTS:
    void setAdcConfig(const AdcConfig &r);
    void setAdcConfMem(const conf_mem_t &value);
    void sendStatistics();
    void decodeEventList (EventListIt, EventListIt);
    void setDecoderConfig(const DecoderConfig &decConf);

Q_SIGNALS:
    void monitorEvent(const AdcmEvent &); // we should share monitorEventTime and include mutex
    void histogramsChanged(const HistList &);
    void statisticsUpdated(const adcm_decstat_t &, int index);
    void decodedCalZero(const CalZeroConfig &);
    void decodedCalAvg(const CalAvgConfig &);

    void adcConfMemChanged(const conf_mem_t &value);
    void adcConfigModified(const AdcConfig &);
    void decoderConfigModified(const DecoderConfig &decConf);
//    void clearAllHists();
//    void clearHist(const hist_index_t &index);

private:
    bool getHistEnable(int index) const;
    int processEvent(AdcmEvent *ev);
    bool writeEvent(const AdcmEvent &ev);

private Q_SLOTS:
    void sendCalResult();

private:
    const unsigned int workerIndex = 0;
    QSemaphore * const idleWorkers;
    QMutex * const stateMutex; // Locked when a job assigned; unlocked when it is idle
    storage * const stor; // Unique created at DecodeManager
    HistManager * const hmgr; // Separated
    bool histEnabled = false;
    adcm_el_t * const eventList;
    adcm_decstat_t * const decst; // Can be independent and be joined in DM with adcm_decstat_t::add

    WaveformDecoder * const decoder; // Separated
    QTime * const monitorEventTime;

    QTimer * const statisticsTimer; // Call updateStatistics()
    QTimer * const calResultTimer;
    decoder_mode_t decodeMode;
    bool storageEnabled = false;
    CalAvgConfig * const newCalAvgConfig;  // CalAvgConfig/CalZeroConfig should store separated and collected in DecodeManager
    CalZeroConfig * const newCalZeroConfig;// CalAvgConfig/CalZeroConfig should store separated and collected in DecodeManager
    quint64 prevEvTs = 0;
};

#endif /* DECODEWORKER_H */
