/*
**    Copyright 2014 Ivan Filippov
**    Copyright 2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef EVENTLISTFILLER_H
#define EVENTLISTFILLER_H

#include <QObject>

class QTime;
class QTimer;

class adcm_el_t;
class adcm_decstat_t;
class AdcmDataBlockDecoder;
class DecodeManager;
class conf_mem_t;
class FrameList;

class EventListFiller : public QObject
{
    Q_OBJECT
public:
    explicit EventListFiller(adcm_el_t *evList, DecodeManager * decodeManager);
    ~EventListFiller() override;

signals:
    void completed(quint64);
    void progressChanged(double);
    void statisticsUpdated(const adcm_decstat_t &);
    void eventsProceedUpdated(quint64);
//    void requestWorkerStatistic();

public slots:
    void appendFrameList(const FrameList &r);
    void setAdcConfMem(const conf_mem_t &value);
    void clear();
    void reset();
    void sendStatistics();

private:
    void appendFrame(const QByteArray &buf);
    int decodeRawFrame(const unsigned int *frame, size_t flen);
    void updateProgress(bool force=false);

    adcm_el_t * const eventList;
    AdcmDataBlockDecoder * const adcmDataBlockDecoder;
    adcm_decstat_t * const fillerDecStat;
    DecodeManager * const manager;

    uint frameListSize = 0; // for progress
    uint framesDecoded = 0; // for progress
    QTime * const lastProgressTime;
    QTime * const workTime;
    QTimer * const statisticTimer;
};

#endif // EVENTLISTFILLER_H
