#ifndef REMOTECONTROLSERVER_H
#define REMOTECONTROLSERVER_H

#include <QObject>

class QTcpServer;
class QTcpSocket;
class QTimer;

class RemoteControlServer : public QObject
{
    Q_OBJECT
public:
    explicit RemoteControlServer(QObject *parent = nullptr);
    virtual ~RemoteControlServer();
    int getTcpPort() const { return tcpPort; }
    bool isFree() const { return !portInUse; }
    bool isListening() const { return listening; }

public slots:
    void startListen(bool state);
    void runStateUpdated(const QString &runState);

signals:
    void startRequest();
    void stopRequest();
    void connectionStatusUpdated(int tcpPort, bool listening, bool portInUse);

private slots:
    void updateConnections();
    void readCommandRequest();
    void onDisconnected();

private:
    void init();
    void sendDescription();
    bool isRunControlConnected() const;
    void sendString(const QString &mess);
    void updateStatus();

    bool listening;
    quint16 tcpPort;
    bool portInUse;
    QTcpServer * const qTcpServer;
    QTcpSocket *tcpSocket;
    QString lastRunState;
};

#endif // REMOTECONTROLSERVER_H
