/*
**    Copyright 2006-2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "adcm_decstat.h"

#include <cstdio> // FIXME: printf
#include <cstring>
#include <cassert>

#include <QDebug>
#include <QString>
#include <QTextStream>

#include "adcm_ev.h"
#include "AdcmChbuf.h"
//#include "timer.h"

//QDebug operator<<(QDebug s, const decst_prof_t &st)
//{
//    QString str;
//    QTextStream stream(&str);
//    stream << st;
//    s.noquote() << str;
//    return s;
//};

//QDebug operator<<(QDebug s, const adcm_decstat_t &st)
//{
//    QString str;
//    QTextStream stream(&str);
//    stream << st;
//    s.noquote() << str;
//    return s;
//};

void adcm_decstat_t::clear()
{
    *this = adcm_decstat_t();
}

const char *decst_prof_t::label(decst_items_t n)
{
    switch(n) {
    case decst_n_dummy:
        return ("dummy");
    case decst_n_vld:
        return ("vld");
#ifdef FIT_SINE
    case decst_n_sine:
        return ("sine");
#endif
    case decst_n_thr:
        return ("thr");
    case decst_n_rms:
        return ("rms");
    case decst_n_bl:
        return ("bl");
    case decst_n_int:
        return ("int");
    case decst_n_clip:
        return ("clip");
    case decst_n_mwb:
        return ("mwb");
    case decst_n_mwbso:
        return ("mwbso");
    case decst_n_conv:
        return ("conv");
    case decst_n_pksrch:
        return ("pksrch");
    case decst_n_pu:
        return ("pu_rej");
    case decst_n_tfast:
        return ("tfast");
    case decst_n_dtcut:
        return ("dtcut");
    case decst_n_fit:
        return ("fit");
    case decst_n_rawdec:
        return ("rawdec");
    case decst_n_hist:
        return ("hist");
    case decst_n_all:
        return ("all");
    case decst_n_items:
        return ("-");
    }
    return ("unknown");
}

quint64 adcm_decstat_t::n_frame_err() const
{
    return n_resync + nf_crc + nf_seq;
}
quint64 adcm_decstat_t::n_data_err() const
{
    return nblk_bad + nblk_unkn + nblk_adc_len + nblk_adc_roll +
           nblk_adc_evnum + nblk_adc_dup;
}
quint64 adcm_decstat_t::n_dsp_err() const
{
    return dsp_bad_len + dsp_thr_nf + dsp_bl_range + dsp_int_range + dsp_clip_pos +
           dsp_clip_neg + dsp_pk_range + dsp_fit_fail + dsp_fit_bad;
}

//    double t_thr, t_rms, t_bl, t_int, t_clip, t_mwb, t_conv, t_pu, t_tfast, t_fit, t_all;
//    double tm_thr, tm_rms, tm_bl, tm_int, tm_clip, tm_mwb, tm_conv, tm_pu, tm_tfast, tm_fit, tm_all;

//void decst_print_prof (const struct decst_prof_t &decstProf)
//{
//    qInfo() << decstProf.toString();
//}

QTextStream &operator<<(QTextStream &s, const decst_prof_t &prof)
{
//    QString str;
//    QTextStream stream(&str);
//    QString ts;
    int i;
    QVector<double> da(decst_n_items);
    QVector<double> dm(decst_n_items);
    const int w = 20; // bar graph width
    double vmin=1e10, vmax=-1e10;

    if (prof.n == 0) {
        return s;
    }

    for(i=0;i<decst_n_items;i++) {
        da[i] = prof.ta[i]*1e6 / prof.n;
        dm[i] = prof.tm[i]*1e6;
    }

    for(i=0;i<decst_n_items-1;i++) {
        double v = da[i];
        if (v>vmax) vmax=v;
        if (v<vmin) vmin=v;
    }

    s << "---- Decode time (us) -----\n";
    s << "   item     avg       min\n";
    s << "---------------------------\n";
    double sumavg=0, summin=0;
    for(i=0;i<decst_n_items;i++) {
        double tavg = da[i];
        double tmin = dm[i];// - 1e6*timer_read_cost_min;
        sumavg += tavg;
        summin += tmin;
        char s1[w+1];
        {
            double v = tavg;
            int ss;
            for (ss = 0; ss < w; ss++) {
                double val = (v-vmin)*(w-1)/(vmax-vmin);
                if (val >= ss)
                    s1[ss] = '*';
                else
                    s1[ss] = ' ';
            }
            s1[w] = 0;
        }
        //        printf("%-7s %8.2f  %8.2f  %s\n",
        //               decst_prof_label((decst_items_t)i),
        //               tavg,
        //               tmin,
        //               ((i+1)==decst_n_items)?"":s1);
        s << QString("%1 %2  %3  %4\n")
                  .arg(prof.label((decst_items_t)i), 7)
                  .arg(tavg, 8, 'f', 2)
                  .arg(tmin, 8, 'f', 2)
                  .arg(((i+1)==decst_n_items)?"":s1);
        if (i+2 == decst_n_items) {
            s << "---------------------------\n";
            //            printf("%-7s %8.2f  %8.2f\n", "sum", sumavg, summin);
            s << QString("%1 %2  %3\n")
                      .arg("sum", 7)
                      .arg(sumavg, 8, 'f', 2)
                      .arg(summin, 8, 'f', 2);
        }
        if (i+1 == decst_n_items)
            s << "===========================\n";
    }
    return s;
}

decst_prof_t::decst_prof_t()
{
    for(int i=0;i<decst_n_items;i++)
        tm[i] = 999999;
}

void decst_prof_t::clear()
{
    *this = decst_prof_t();
}

decst_prof_t &decst_prof_t::operator+=(const struct decst_prof_t &p)
{
    for(int i=0; i<decst_n_items; i++) {
        ta[i] += p.ta[i];
        tl[i] = p.ta[i];
    }
    n += p.n;
    if (n > 0 && p.n > 0) {
        for(int i=0; i<decst_n_items; i++) {
            if (tm[i] > p.tm[i] || tm[i] == 0)
                tm[i] = p.tm[i];
            //  	  printf("%2d %e, %e\n", i, prof.tm[i], p->tm[i]);
        }
    }
    //  printf("decst_add_prof: added %u entries\n", p->n);
    return *this;
}

adcm_chstat_t &adcm_chstat_t::operator+=(const adcm_chstat_t &p)
{
    size_t n = std::min(all.size(), p.all.size());
    for(unsigned int c=0; c<n; c++) {
        all[c] += p.all[c];
        blnd[c] += p.blnd[c];
        intnd[c] += p.intnd[c];
        blout[c] += p.blout[c];
        clip[c] += p.clip[c];
        pu[c] += p.pu[c];
    }
    return *this;
}

adcm_decstat_t &adcm_decstat_t::operator+=(const adcm_decstat_t &incr)
{
    nw += incr.nw;
    overruns += incr.overruns;
    lastpos = incr.lastpos; // non-additive value
    n_resync += incr.n_resync;
    nf_crc += incr.nf_crc;
    nf_seq += incr.nf_seq;
    nf += incr.nf;
    nf_echo += incr.nf_echo;
    nf_dadc += incr.nf_dadc;
    nf_creq += incr.nf_creq;
    nf_cack += incr.nf_cack;
    nf_stat += incr.nf_stat;
    nf_unkn += incr.nf_unkn;
    praw_err = incr.praw_err; // non-additive value
    pf_err = incr.pf_err; // non-additive value

    nblk_all += incr.nblk_all;
    nblk_adc_0 += incr.nblk_adc_0;
    nblk_adc_1 += incr.nblk_adc_1;
    nblk_adc_2 += incr.nblk_adc_2;
    //  nblk_stat += incr.nblk_stat;
    nblk_bad += incr.nblk_bad;
    nblk_unkn += incr.nblk_unkn;
    nblk_crc += incr.nblk_crc;
    pblk_err = incr.pblk_err; // non-additive value

    nblk_adc_len += incr.nblk_adc_len;
    nblk_adc_roll += incr.nblk_adc_roll;
    nblk_adc_dup += incr.nblk_adc_dup;
    nblk_adc_evnum += incr.nblk_adc_evnum;
    nblk_adc_ok += incr.nblk_adc_ok;

    dsp_good_waveform += incr.dsp_good_waveform;
    dsp_total += incr.dsp_total;
    dsp_bad_len += incr.dsp_bad_len;
    dsp_thr_nf += incr.dsp_thr_nf;
    dsp_bl_range += incr.dsp_bl_range;
    dsp_int_range += incr.dsp_int_range;
    dsp_clip_pos += incr.dsp_clip_pos;
    dsp_clip_neg += incr.dsp_clip_neg;
    dsp_pk_range += incr.dsp_pk_range;
    dsp_fit_fail += incr.dsp_fit_fail;
    dsp_fit_bad += incr.dsp_fit_bad;
    dsp_fit_ok += incr.dsp_fit_ok;
    dsp_sine_ok += incr.dsp_sine_ok;

    ev_total += incr.ev_total;
    ev_dt_cut += incr.ev_dt_cut;

    frame_errors_suppressed += incr.frame_errors_suppressed;
    data_errors_suppressed += incr.data_errors_suppressed;
    dsp_errors_suppressed += incr.dsp_errors_suppressed;

    prof += incr.prof;
    cs += incr.cs;
    return *this;
}

void decst_add_ev_chstat (adcm_decstat_t * decst, const AdcmEvent &ev)
{
    assert(decst);
    struct adcm_chstat_t *cs = &decst->cs;
    for (auto c : ev.channel.keys()) {
        const adcm_chbuf_t &cb = ev.channel[c];
        const auto &ferr = cb.error_flags;
        const auto &f = cb.flags;
        if (! f.bit.has_data) continue;
        cs->all[c]++;
        if (ferr.bit.bl_nodata) cs->blnd[c]++;
        if (ferr.bit.int_nodata) cs->intnd[c]++;
        if (ferr.bit.bl_out) cs->blout[c]++;
        if (f.bit.has_clip) cs->clip[c]++;
        if (f.bit.has_pu) cs->pu[c]++;
    }
}

QTextStream &operator<<(QTextStream &s, const adcm_decstat_t &st)
{
    s << "---------- Decoder statistics --------------------------------" << endl;
    s << "* Data size:   " << 1.*st.nw / 256./1024. << " MB, " << st.nw << " words\n";
    s << "  Resync:          " << st.n_resync << endl;
    s << "  Overruns:        " << st.overruns << endl;
    s << "* Frames:          " << st.nf << endl;
    s << "    adc data:        " << st.nf_dadc;
    s << "    ctrl req:        " << st.nf_creq << endl;
    s << "    ctrl ack:        " << st.nf_cack << endl;
    s << "    status:          " << st.nf_stat << endl;
    s << "    unknown:         " << st.nf_unkn << endl;
    s << "  Bad CRC:         " << st.nf_crc << endl;
    s << "  Bad sequence:    " << st.nf_seq << endl;
    s << "* Data blocks           " << st.nblk_all << endl;
    s << "    adc0:                 " << st.nblk_adc_0 << endl;
    s << "    adc1:                 " << st.nblk_adc_1 << endl;
    s << "    adc2:                 " << st.nblk_adc_2 << endl;
    s << "    unknown:              " << st.nblk_unkn << endl;
    s << "  bad (corrupt data):   " << st.nblk_bad << endl;
    s << "* ADC data blocks\n" << endl;
    s << "    length error:         " << st.nblk_adc_len << endl;
    s << "    bad event number:     " << st.nblk_adc_evnum << endl;
    s << "    roller error:         " << st.nblk_adc_roll << endl;
    s << "    duplicate channel:    " << st.nblk_adc_dup << endl;
    s << "* DSP statistics\n" << endl;
    s << "    waveforms decoded:      " << st.dsp_good_waveform << endl;
    s << "    pulses decoded:         " << st.dsp_total << endl;
    s << "    bad waveform length:    " << st.dsp_bad_len << endl;
    s << "    threshold not found:    " << st.dsp_thr_nf << endl;
    s << "    baseline insuff. data:  " << st.dsp_bl_range << endl;
    s << "    integral insuff. data:  " << st.dsp_int_range << endl;
    s << "    positive clipping:      " << st.dsp_clip_pos << endl;
    s << "    negative clipping:      " << st.dsp_clip_neg << endl;
    s << "    pksrch insuff. data:    " << st.dsp_pk_range << endl;
    s << "    fit failed:             " << st.dsp_fit_fail << endl;
    s << "    fit bad:                " << st.dsp_fit_bad << endl;
    s << "    fit ok:                 " << st.dsp_fit_ok << endl;
    s << "    sine ok:                " << st.dsp_sine_ok << endl;
    s << endl << endl;
    s << "Channel decode statistics\n" << endl;
    s << " ch |    ALL   |   BL_ND  |  INT_ND  |  BL_OUT  |   CLIP   |  PILEUP  " << endl;

    const struct adcm_chstat_t &cs = st.cs;
    for (size_t c=0; c<st.cs.all.size(); c++) {
        if (cs.all[c] == 0) continue;
        s << c << cs.all[c] << cs.blnd[c] << cs.intnd[c] << cs.blout[c] << cs.clip[c] << cs.pu[c] << endl;
    }
    s << "--------------------------------------------------------------\n";
    return s;
}
