#ifndef _ADCM_EV_H
#define _ADCM_EV_H 1

#include <vector>
#include <QMetaType>
#include <QMap>

#include "AdcmChbuf.h"
#include "adcm16.h"

class QTextStream;

// Event decode flags
// (grep ADCM_EV_E adcm_dec.c|awk '{print $3}'|sed -e 's/.$//'|sort|uniq)
// Fatal error - do not use this event in any way
enum { ADCM_EV_INVAL = 0x1000000 };/* event data is invalid, decode fatal error */

// Some error while decoding occured, however data may be useful
enum { ADCM_EV_ECH    = 0x0000001 };
enum { ADCM_EV_EDLEN  = 0x0000002 };/* Data length error */
enum { ADCM_EV_EDUPCH = 0x0000004 };
enum { ADCM_EV_EID    = 0x0000008 };
enum { ADCM_EV_ENS    = 0x0000010 };
enum { ADCM_EV_EROLL  = 0x0000020 };/* Sample counter error, sample data may be invalid */
enum { ADCM_EV_ESPW   = 0x0000040 };
enum { ADCM_EV_EBIT   = 0x0000080 };
enum { ADCM_EV_EUNEXP = 0x0000100 };
enum { ADCM_EV_BADBLK = 0x0000200 };/* bad data block found */

//enum { ADCM_EV_INIT   = 0x8000000 };/* event is initialized */

enum { ADCM_EV_EANY   = 0x00000FF };/* mask - any of above */

// slide flags

//enum { ADCM_CH_FINISHED   = 0x00000001 }; /* waveform is complete */
//enum { ADCM_CH_HAS_DATA   = 0x00000002 };
//enum { ADCM_CH_HAS_THR    = 0x00000004 }; /* has signal over threshold */
//enum { ADCM_CH_HAS_CLIP   = 0x00000008 }; /* clipping detected */
//enum { ADCM_CH_HAS_PU     = 0x00000010 }; /* pile-up detected */
//enum { ADCM_CH_DT_CUT     = 0x00000020 }; /* channel discarded by dT cut */
////
//enum { ADCM_CH_EVAL_SINE  = 0x00000080 }; /* find sine wave parameters */
//enum { ADCM_CH_EVAL_THR   = 0x00000100 }; /* threshold position evaluated */
//enum { ADCM_CH_EVAL_BL    = 0x00000200 }; /* baseline evaluated */
//enum { ADCM_CH_EVAL_INT   = 0x00000400 }; /* integral evaluated */
//enum { ADCM_CH_EVAL_CLIP  = 0x00000800 }; /* clipping evaluated */
//enum { ADCM_CH_EVAL_PU    = 0x00001000 }; /* pile-up evaluated */
//enum { ADCM_CH_EVAL_TFINE = 0x00002000 }; /* time evaluated */
//enum { ADCM_CH_EVAL_RMS   = 0x00004000 }; /* DC, RMS evaluated */
//enum { ADCM_CH_EVAL_TFAST = 0x00008000 }; /* fast (approx.) time evaluated */

//// errors
//enum { ADCM_CH_BL_NODATA  = 0x00010000 }; /* insufficient data for baseline calculation */
//enum { ADCM_CH_INT_NODATA = 0x00020000 }; /* insufficient data for integral calculation */
//enum { ADCM_CH_BL_OUT     = 0x00040000 }; /* baseline out of specified limits */
//enum { ADCM_CH_PKS_NODATA = 0x00080000 }; /* insufficient data for peak search */
//enum { ADCM_CH_TFIT_ERR   = 0x00100000 }; /* time reconstruction error */
//enum { ADCM_CH_TFIT_OFF   = 0x00200000 }; /* reconstructed time is too far from peak */
//enum { ADCM_CH_BAD_LEN    = 0x00400000 }; /* waveform length bad */
//enum { ADCM_CH_ANY_ERR    = 0x00FF0000 }; /* mask */

typedef enum {
    OFFSET_BINARY_ENCODING,
    SIGNED_ENCODING
} SampleEncoding;

typedef struct AdcmSampleFormat {
    double step = 1; /// nanoseconds
    int bits = 2;
    SampleEncoding encoding = OFFSET_BINARY_ENCODING;
} AdcmSampleFormat;

class AdcmEvent {
public:
    enum { ADCM_EV_DT_CUT = 0x0000400 }; /* event should be dropped by dT cut */

    void clear();
    //      int write_event () const;
    void print_event() const;
    void print_flags () const;

    // get, set
    unsigned int getFlags() const { return flags; }
    //      void setFlags(unsigned int n) { flags = n; }
    bool getFlag(unsigned int mask) const { return (flags & mask) != 0; }
    void setFlag(unsigned int mask) { flags |= mask; }
    void clearFlag(unsigned int mask) { flags &= ~mask; }
    unsigned int getBurstNumber() const { return burstNum; }
    void setBurstNumber(unsigned int n) { burstNum = n; }
    unsigned int getEventNumber() const { return evNum; }
    void setEventNumber(unsigned int n) { evNum = n; }
    quint64 getEventTimestamp() const { return ts; }
    void setEventTimestamp(quint64 n) { ts = n; }

    // helpers
    int getAdcHalfRange() const { return 1<<(format.bits-1); }

public:
    AdcmSampleFormat format;
    QMap<int, adcm_chbuf_t> channel;

private:
    unsigned int burstNum = 0; // Burst Number
    unsigned int evNum = 0; // Event Number
    unsigned int flags = 0;
    quint64 ts = 0; // Event TimeStamp

    friend QTextStream &operator<<(QTextStream &s, const AdcmEvent &ev);
};

Q_DECLARE_METATYPE(AdcmEvent)

//int adcm_get_chbuf_flags(const adcm_chbuf_t &cb) { return cb.flags; } // channel flags
//double adcm_get_chbuf_pt(const adcm_chbuf_t &cb) { return cb.pt; } // fine time

//  void adcm_ev_ch_flag_set(struct adcm_ev_t *ev, unsigned int ch, unsigned int mask, unsigned int value);
//  int adcm_ev_ch_flag_cmp(const struct adcm_ev_t *ev, unsigned int ch, unsigned int mask, unsigned int value);
//int adcm_ev_ch_flag_getbit(const AdcmEvent &ev, unsigned int ch, unsigned int mask);
//void adcm_ev_ch_flag_setbit(AdcmEvent *ev, unsigned int ch, unsigned int mask);
//void adcm_ev_ch_flag_clrbit(AdcmEvent *ev, unsigned int ch, unsigned int mask);

/*
  struct adcm_chstat_t; // accumulated channel statistics
  struct adcm_chstat_t *adcm_alloc_chstat();
  void adcm_free_chstat(struct adcm_chstat_t *cs);
  void adcm_clear_chstat(struct adcm_chstat_t *cs);
  void adcm_add_chstat(struct adcm_chstat_t *cs, const struct adcm_ev_t *ev);
  void adcm_print_chstat(const struct adcm_chstat_t *cs);
*/

#endif /* _ADCM_EV_H */
