
#include <cstddef>
#include <cassert>

#include <QDir>
#include <QFileInfo>
#include <QDateTime>

#include "util.h"

#include <sys/types.h>

#if defined(Q_OS_WIN32)
#include <windows.h>
#elif defined(Q_OS_DARWIN)
#include <sys/stat.h>
#include <sys/statvfs.h>
#include <sys/param.h>
#include <sys/mount.h>
#elif defined(Q_OS_LINUX)
#include <sys/stat.h>
#include <sys/vfs.h>
#include <sys/statfs.h>
#endif

bool get_disk_space(const QString &fn, qint64 *bytes_total, qint64 *bytes_avail)
{
    QFileInfo fi(fn);
    QString dirPath = fi.dir().path();
#ifdef _WIN32
    QString sCurDir = QDir::current().absPath();
    QDir::setCurrent( dirPath );

    ULARGE_INTEGER free,total;
    bool bRes = ::GetDiskFreeSpaceExA( 0 , &free , &total , nullptr );
    if ( !bRes ) return false;

    QDir::setCurrent( sCurDir );

    if (bytes_avail)
        *bytes_avail = static_cast<__int64>(free.QuadPart);
    if (bytes_total)
        *bytes_total = static_cast<__int64>(total.QuadPart);

#else // linux
    struct stat stst = {};
    struct statfs stfs = {};

    if (::stat(dirPath.toLocal8Bit(), &stst) == -1) return false;
    if (::statfs(dirPath.toLocal8Bit(), &stfs) == -1) return false;

    if (bytes_avail)
        *bytes_avail = stfs.f_bavail * stst.st_blksize;
    if (bytes_total)
        *bytes_total = stfs.f_blocks * stst.st_blksize;
    /*
        struct statvfs st;
        qint64 ba = 0, bt=0;

        int code = statvfs(dirc, &st);
//  free(dname);
  free(dirc);
        if (code==0) {
          ba = (qint64)st.f_bsize * (double)st.f_bavail;
          bt = (qint64)st.f_frsize * (double)st.f_blocks;
        } else
          return (-1);
  if (bytes_total) *bytes_total = bt;
  if (bytes_avail) *bytes_avail = ba;
//  printf("Stat %s: %lld, %lld\n", dname, bt, ba);
  return true;
*/
#endif
    return true;
}
