//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "adcm_ids.h"

#include <QString>

QString adc_board_name_str(AdcmId id)
{
    switch (id) {
    case UNKNOWN_BOARD_ID: return "unknown";
    case ADCM_BOARD_ID: return "ADCM16";
    case ADCMLTC_BOARD_ID: return "ADC16-LTC";
    case ADCM3_BOARD_ID: return "ADCM16-3";
    case ADCMG_BOARD_ID: return "ADCM8";
    case ADC64_BOARD_ID: return "ADC64";
    case ADC64S2_BOARD_ID: return "ADC64S2 v1/v2/v3";
    case ADC64S2V4_BOARD_ID: return "ADC64S2 v4/v5";
    case ADC64VE_BOARD_ID: return "ADC64VE";
    case ADC64ECAL_BOARD_ID: return "ADC64ECAL";
    case ADC64VEV2_BOARD_ID: return "ADC64VE v2";
    case TQDC16VS_BOARD_ID: return "TQDC16VS";
    case BPM4100_BOARD_ID: return "BPM4100";
    }
    return "unknown";
}

int maxSlaveCount(AdcmId id)
{
    switch (id) {
    case UNKNOWN_BOARD_ID: return 0;
    case ADCM_BOARD_ID: return 1;
    case ADCMG_BOARD_ID: return 0;
    case ADCMLTC_BOARD_ID: return 1;
    case ADCM3_BOARD_ID: return 0;
    case BPM4100_BOARD_ID: return 0;
    case ADC64_BOARD_ID: return 3;
    case ADC64S2_BOARD_ID: return 3;
    case ADC64S2V4_BOARD_ID: return 3;
    case ADC64VE_BOARD_ID: return 3;
    case ADC64VEV2_BOARD_ID: return 3;
    case ADC64ECAL_BOARD_ID: return 3;
    case TQDC16VS_BOARD_ID: return 0;
    }
    return 0;
}

bool isDeviceTypeSupported(AdcmId id)
{
    switch (id) {
    case UNKNOWN_BOARD_ID: return false;
    case ADCM_BOARD_ID: return true;
    case ADCMG_BOARD_ID: return true;
    case ADCMLTC_BOARD_ID: return true;
    case ADCM3_BOARD_ID: return true;
    case BPM4100_BOARD_ID: return true;
    case ADC64_BOARD_ID: return true;
    case ADC64S2_BOARD_ID: return true;
    case ADC64S2V4_BOARD_ID: return true;
    case ADC64VE_BOARD_ID: return true;
    case ADC64VEV2_BOARD_ID: return true;
    case ADC64ECAL_BOARD_ID: return true;
    case TQDC16VS_BOARD_ID: return true;
    }
    return false;
}

int boardChannelCount(AdcmId id)
{
    switch (id) {
    case UNKNOWN_BOARD_ID: return 16.;
    case ADCM_BOARD_ID: return 16;
    case ADCMG_BOARD_ID: return 8;
    case ADCMLTC_BOARD_ID: return 16;
    case ADCM3_BOARD_ID: return 16;
    case BPM4100_BOARD_ID: return 4;
    case ADC64_BOARD_ID: return 16;
    case ADC64S2_BOARD_ID: return 16;
    case ADC64S2V4_BOARD_ID: return 16;
    case ADC64VE_BOARD_ID: return 16;
    case ADC64VEV2_BOARD_ID: return 16;
    case ADC64ECAL_BOARD_ID: return 16;
    case TQDC16VS_BOARD_ID: return 16;
    }
    return 16;
}

int boardAdcBits(AdcmId id)
{
    switch (id) {
    case UNKNOWN_BOARD_ID: return 10.;
    case ADCM_BOARD_ID: return 10;
    case ADCMG_BOARD_ID: return 10;
    case ADCMLTC_BOARD_ID: return 14;
    case ADCM3_BOARD_ID: return 14;
    case BPM4100_BOARD_ID: return 14;
    case ADC64_BOARD_ID: return 12;
    case ADC64S2_BOARD_ID: return 12;
    case ADC64S2V4_BOARD_ID: return 14;
    case ADC64VE_BOARD_ID: return 12;
    case ADC64VEV2_BOARD_ID: return 14;
    case ADC64ECAL_BOARD_ID: return 14;
    case TQDC16VS_BOARD_ID: return 14;
    }
    return 14;
}

double boardAdcStepNs(AdcmId id)
{
    switch (id) {
    case UNKNOWN_BOARD_ID: return 10.;
    case ADCM_BOARD_ID: return 10.;
    case ADCMG_BOARD_ID: return 10.;
    case ADCMLTC_BOARD_ID: return 10.;
    case ADCM3_BOARD_ID: return 20.;
    case BPM4100_BOARD_ID: return 10.;
    case ADC64_BOARD_ID: return 20.;
    case ADC64S2_BOARD_ID: return 16.;
    case ADC64S2V4_BOARD_ID: return 16.;
    case ADC64VEV2_BOARD_ID: return 16.;
    case ADC64VE_BOARD_ID: return 16.;
    case ADC64ECAL_BOARD_ID: return 16.;
    case TQDC16VS_BOARD_ID: return 8.;
    }
    return 10.;
}

bool boardVirtualSlave(AdcmId id)
{
    switch (id) {
    case ADCM_BOARD_ID: return false;
    case ADCMLTC_BOARD_ID: return false;
    default: return true;
    }
}
