#ifndef ADCM_MEM_H
#define ADCM_MEM_H

#include "adc_hardware_types.h"
#include "adcm_ids.h"

#include <QMetaType>
#include <QVector>
#include <QMap>
#include <QStringList>

class QTextStream;

QTextStream &operator<<(QTextStream &s, const adcm_prop_reg_t &b);

// Hardware configuration, read-only register data
class AdcmHardwareConfig {
public:
    bool operator==(const AdcmHardwareConfig &rhs) const;
    bool operator!=(const AdcmHardwareConfig &rhs) const { return !operator==(rhs); }
    bool validateAny(int i);
    bool validateSlave(int i, const AdcmHardwareConfig &master);
    QString firmwareStr() const;
    QString getSerial() const;

    quint16 fw_ver = 0;
    quint16 fw_rev = 0;
    quint32 serial_id = 0;
    quint16 ch_dpm_ks = 0;
    quint16 flen_max = 0;
    quint16 fcnt_max = 0;
    adcm_prop_reg_t prop_reg = {};

    friend QTextStream &operator<<(QTextStream &s, const AdcmHardwareConfig &b);
};

class conf_mem_board_t {
public:
    conf_mem_board_t() = default;
    conf_mem_board_t(AdcmId typeId, const AdcmHardwareConfig &adcmHardwareConfig);
    bool operator==(const conf_mem_board_t &rhs) const;
    bool operator!=(const conf_mem_board_t &rhs) const { return !operator==(rhs); }
    QString descriptionStr() const;
    QString fwPropStr() const;
    int getAdcHalfRange() const;
    int getAdcMaxValue() const;
    int getAdcMidValue() const;
    int getAdcMinValue() const;
    unsigned int getAdcBits() const;

    AdcmId board_type = UNKNOWN_BOARD_ID;
    int nch = 0;
//    unsigned int adcbits;
    double adcstep_ns = 1;
    AdcmHardwareConfig hw;

//    friend class conf_mem_t;
    friend QTextStream &operator<<(QTextStream &s, const conf_mem_board_t &b);
};

class conf_mem_t {
public:
    conf_mem_t(const QMap<int, conf_mem_board_t> &boards = QMap<int, conf_mem_board_t>());
    bool operator==(const conf_mem_t &rhs) const;
    bool operator!=(const conf_mem_t &rhs) const { return !operator==(rhs); }
    conf_mem_board_t master() const { return b[0]; }
    [[deprecated]] AdcmId master_board_id() const;
    [[deprecated]] adcm_prop_reg_t master_firmware_prop() const;
    int getAdcHalfRange(int ch) const;
    int getAdcMaxValue(int ch) const;
    int getAdcMidValue(int ch) const;
    int getAdcMinValue(int ch) const;
    bool hasHyst(int ch) const;
    bool hasExtMatchWin() const;
    bool hasMultiFragment() const;
    bool hasTrigMultiplicity() const;

    void clear();
    bool hasGainControl() const;
    bool hasTriggerControl() const;
    int adc_get_prewin_min() const;
    int adc_get_prewin_max() const;
    int adc_get_matchwin_min() const;
    int adc_get_matchwin_max() const;
    int adc_get_latency_min() const;
    int adc_get_latency_max() const;
    int adc_halfrange() const;

    double getMinGainDb() const;
    double getMaxGainDb() const;
    double getGainStepDb() const;

    // FIXME: assumed slave has same hardware limitations (see AdcmHardwareConfig::validateSlave)
    [[deprecated]] int getMasterAdcBits() const;
    [[deprecated]] double getMasterAdcStepNs() const;
    [[deprecated]] int getMasterMaxFlen() const; //!< maximum fragment length, samples, payload only
    [[deprecated]] int getMasterMaxFcnt() const; //!< maximum number of fragments
    [[deprecated]] int getMasterChannelMemorySizeKs() const; //!< channel memory size in samples (header, payload, trailer)

    QStringList getDeviceIdStrList() const;
    [[deprecated]] QString getMasterSerial() const;
    [[deprecated]] QString getMasterTypeStr() const;
    const QMap<int, conf_mem_board_t> &boards() const { return b; }
    void setBoard(int index, const conf_mem_board_t &board) {b[index] = board; nch = getNch();}

//    QVector<double> temperature;
    friend QTextStream &operator<<(QTextStream &s, const conf_mem_t &conf_mem);

private:
    QMap<int, conf_mem_board_t> b;
    int getNch() const;
    bool map_ch(int ch, int *bn, int *bc) const;

public:
    unsigned int nch = 0; // cached for speed
};

Q_DECLARE_METATYPE(conf_mem_t)

#endif /* ADCM_MEM_H */
