/*
**    Copyright 2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#ifndef CONFIGMANAGER_H
#define CONFIGMANAGER_H

#include <QObject>
#include <QDebug>
#include <QSettings>
#include <QSharedPointer>

#include "adcmraw/adcm_mem.h"
#include "allconfig.h"

class ConfigStatus;

class ConfigManager : public QObject
{
    Q_OBJECT
public:
    Q_DISABLE_COPY(ConfigManager)
    explicit ConfigManager(QObject *parent = nullptr);
    ~ConfigManager() final;

Q_SIGNALS:
    bool configModified(bool);
    bool fileNameChanged(const QString &);
    void adcConfMemChanged(const conf_mem_t &);
    void adcConfigChanged(const AdcConfig &);
    void decoderConfigChanged(const DecoderConfig &);
    void calAvgConfigChanged(const CalAvgConfig &);
    void calZeroConfigChanged(const CalZeroConfig &);
    void fitProfileChanged(const FitProfile &);
    void histConfigChanged(const HistConfig &); // Unused (there is no slots for this signal)
    void runConfigChanged(const RunConfig &); // Unused (there is no slots for this signal)
    void rawStorageConfigChanged(const StorageConfig &); // sends to TWrite
    void evStorageConfigChanged(const StorageConfig &); // sends to DecodeManager

public Q_SLOTS:
    void addCalZero(const CalZeroConfig &cal);
    void addCalAvg(const CalAvgConfig &cal);
    void newConfig();
    ConfigStatus loadConfig(const QString &fileName, bool merge = true);
    bool saveConfig(const QString &fileName);
    void setAllConfig(const AllConfig &r);
    void setAdcConfMem(const conf_mem_t &r);
    void setAdcConfig(const AdcConfig &r);
    void setDecoderConfig(const DecoderConfig &r);
    void setCalAvgConfig(const CalAvgConfig &r);
    void setCalZeroConfig(const CalZeroConfig &r);
    void setFitProfile(const FitProfile &r);
    void setRunConfig(const RunConfig &r);
    void setRawStorageConfig(const StorageConfig &r);
    void setEvStorageConfig(const StorageConfig &r);

private Q_SLOTS:
    void autoLoad();
    void autoSave();

private:
//        conf_mem_t getAdcConfMem() { return adcConfMem; }
//        AllConfig getAllConfig() const { return allConfig; }
    AdcConfig read_adc_setup_memory() const;
    QSharedPointer<QSettings> readSettingsFile(const QString &fileName, ConfigStatus *status = nullptr);
    void setModified(bool state);
    void setCurrentFileName(const QString &s);

    conf_mem_t adcConfMem;
    AllConfig allConfig;
    QSharedPointer<const AllConfig> savedcfg;
    QSharedPointer<const AllConfig> lastAutoSaveConfig;
    QString currentFileName;
    bool modified = false;
};

#endif // CONFIGMANAGER_H
