//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "calavgconfig.h"

#include <QDebug>
#include <QTextStream>

#include <cstdio>
#include <cmath>

#include "adcm16.h"
#include "dsplib/dsplib.h"
#include "adcconfig.h"
#include "util/AsciiGraphics.h"

static const double eps = 1e-15;

CalAvdChannelData::CalAvdChannelData(const QVector<int> &r)
{
    for (auto i : r)
        push_back(i);
}

bool CalAvgChannelConfig::operator==(const CalAvgChannelConfig &other) const
{
    return true
            && nent == other.nent
            && data == other.data
            && abs(step - other.step) < eps
            ;
}

bool CalAvgConfig::operator==(const CalAvgConfig &other) const
{
    return true
            && channels == other.channels
            ;
}

CalAvgChannelConfig &CalAvgChannelConfig::operator+=(const CalAvgChannelConfig &r)
{
    if (r.isEmpty())
        return *this;
    if (r.data.size() != data.size()) {
        clear();
    }
    if (abs(r.step - step) > eps) {
        clear();
    }
    step = r.step;
    nent += r.nent;
    if (data.size() < r.data.size())
        data.resize(r.data.size());
    for(int i=0; i<r.data.size(); i++)
        data[i] += r.data[i]; // FIXME: optimize
    return *this;
}

void CalAvgChannelConfig::clear()
{
    nent = 0;
    data.resize(0);
}

CalAvdChannelData CalAvgChannelConfig::get() const
{
    if (isEmpty())
        return CalAvdChannelData();
    CalAvdChannelData tmp(data);
    for (int i = 0; i < data.size(); i++) {
        tmp[i] /= nent;
    }
    return tmp;
}

CalAvgConfig &CalAvgConfig::operator+=(const CalAvgConfig &rhs)
{
    for (auto c : rhs.channels.keys()) {
        channels[c] += rhs.channels[c];
    }
    return *this;
}

void CalAvgConfig::clear()
{
    channels.clear();
}

void CalAvgConfig::clear_enabled(const AdcConfig &adc)
{
    for (auto c : channels.keys()) {
        if (!adc.ch_enabled_shape_calibration (c))
            continue;
        channels.remove(c);
    }
}

bool CalAvgChannelConfig::isEmpty() const
{
    return (nent == 0) || data.isEmpty();
}

QTextStream &operator<<(QTextStream &s, const CalAvgConfig &a)
{
    s << "# --- CalAvgConfig ---" << endl;
    if (a.channels.isEmpty()) {
        s << "# empty" << endl;
    } else {
        s << "# Number of channels: " << a.channels.size() << endl;
        for (int c : a.channels.keys()) {
            const CalAvgChannelConfig channel = a.channels[c];
            if (channel.isEmpty())
                continue;
            s << "calavg[" << c << "]: " << (channel.isEmpty() ? "empty" : "")
              << ", step=" << channel.step << endl;
            AsciiGraphics::drawWaveform(s, channel.data, QString("calavg[%1]").arg(c));
        }
    }
    return s;
}
