//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CALAVGCONFIG_H
#define CALAVGCONFIG_H

#include <QVector>
#include <QMap>
#include <QMetaType>

class QTextStream;
class AdcConfig;

class CalAvdChannelData : public QVector<double>
{
public:
    CalAvdChannelData() = default;
    ~CalAvdChannelData() = default;
    CalAvdChannelData(const CalAvdChannelData &) = default;
    CalAvdChannelData &operator=(const CalAvdChannelData &) = default;
    CalAvdChannelData(const QVector<int> &);
};

//Q_DECLARE_METATYPE(CalAvdChannelData)
class CalAvgConfig;

class CalAvgChannelConfig
{
public:
    CalAvgChannelConfig() = default;
    CalAvgChannelConfig(const CalAvdChannelData &_data, double _step = 1.0)
        : step(_step), data(_data), nent(1) {}
    CalAvgChannelConfig &operator+=(const CalAvgChannelConfig &r);
    bool operator==(const CalAvgChannelConfig &other) const;
    bool operator!=(const CalAvgChannelConfig &other) const {return !operator==(other);}
    bool isEmpty() const;
    void clear();
    CalAvdChannelData get() const;
    int get_nent() const {return nent;}

    friend QTextStream &operator<<(QTextStream &s, const CalAvgConfig &r);

    double step = 1.0;
private:
    CalAvdChannelData data; //!< unnormalized average waveform (divide by nent when use it)
    int nent = 0;           //!< number of averaged waveforms, must be < 2M (overflow warning)
};

Q_DECLARE_METATYPE(CalAvgChannelConfig)

class CalAvgConfig
{
public:
    CalAvgChannelConfig get(int ch) const { return channels[ch]; }
    void print() const;
    void clear();
    void clear_enabled(const AdcConfig &adc);

    CalAvgConfig &operator+=(const CalAvgConfig &rhs);
    bool operator==(const CalAvgConfig &other) const;
    bool operator!=(const CalAvgConfig &other) const {return !operator==(other);}

    friend CalAvgConfig operator+(CalAvgConfig lhs, const CalAvgConfig &rhs)
    { lhs += rhs; return lhs; }

    QMap<int, CalAvgChannelConfig> channels;

    friend QTextStream &operator<<(QTextStream &s, const CalAvgConfig &r);
};

Q_DECLARE_METATYPE(CalAvgConfig)

#endif // CALAVGCONFIG_H
