//
//    Copyright 2013-2019 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "calzeroconfig.h"

#include <QDebug>
#include <QTextStream>
#include <cmath>

#include "adcm16.h"
#include "adcconfig.h"

const double eps = 1e-15;

bool CalZeroChannelConfig::operator==(const CalZeroChannelConfig &other) const
{
    return true
            && nent == other.nent
            && abs(base - other.base) < eps
            && abs(rms - other.rms) < eps
            ;
}

CalZeroChannelConfig &CalZeroChannelConfig::operator+=(const CalZeroChannelConfig &rhs)
{
    nent += rhs.nent;
    base += rhs.base;
    rms += rhs.rms;
    return *this;
}

bool CalZeroConfig::operator==(const CalZeroConfig &other) const
{
    return channels == other.channels;
}

void CalZeroConfig::clear_calzero_ch(int ch)
{
    channels[ch].clear();
}

CalZeroConfig &CalZeroConfig::operator+=(const CalZeroConfig &rhs)
{
    for (auto c : rhs.channels.keys()) {
        channels[c] += rhs.channels[c];
    }
    return *this;
}

double CalZeroChannelConfig::get_base() const
{
    if (nent > 0)
        return base / nent;
    else
        return 0;
}

double CalZeroChannelConfig::get_rms() const
{
    if (nent > 0)
        return rms / nent;
    else
        return 0;
}


double CalZeroChannelConfig::getcalnoiserms() const
{
    if (nent > 0)
        return rms / nent;
    else
        return 1.0;
}

double CalZeroConfig::get(int ch) const
{
    return channels[ch].get_base();
}

double CalZeroConfig::getcalnoiserms (int ch) const
{
    return channels[ch].getcalnoiserms();
}

void CalZeroConfig::clear_enabled(const AdcConfig &adc)
{
    for (auto c : channels.keys()) {
        if (!adc.ch_enabled_offset_calibration (c))
            continue;
        clear_calzero_ch(c);
    }
}

QTextStream &operator<<(QTextStream &s, const CalZeroChannelConfig &channel)
{
    if (channel.isEmpty())
        s << "# empty";
    else
        s << "dc=" << channel.get_base() << ", rms=" << channel.get_rms();
    return s;
}

QTextStream &operator<<(QTextStream &s, const CalZeroConfig &v)
{
    s << "# --- CalAvgConfig ---" << endl;
    if (v.channels.isEmpty()) {
        s << "# empty" << endl;
    } else {
        s << "# Number of channels: " << v.channels.size() << endl;
        for (auto c : v.channels.keys()) {
            s << "calzero[" << c << "]: " << v.channels[c];
        }
    }
    return s;
}
