//
//    Copyright 2013-2019 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CALZEROCONFIG_H
#define CALZEROCONFIG_H

#include <QVector>
#include <QMap>
#include <QMetaType>

class QTextStream;
class AdcConfig;

// zero calibration structure
class CalZeroChannelConfig
{
public:
    CalZeroChannelConfig() = default;
    CalZeroChannelConfig(double set_base, double set_rms)
        : nent(1), base(set_base), rms(set_rms) {}

    CalZeroChannelConfig &operator+=(const CalZeroChannelConfig &rhs);
    bool operator==(const CalZeroChannelConfig &other) const;
    bool operator!=(const CalZeroChannelConfig &other) const {return !operator==(other);}

    double get_base() const;
    double get_rms() const;
    double getcalnoiserms() const;
    void clear() {*this = CalZeroChannelConfig();}
    bool isEmpty() const { return nent == 0; }

    friend CalZeroChannelConfig operator+(CalZeroChannelConfig lhs, const CalZeroChannelConfig &rhs)
    { lhs += rhs; return lhs; }
    friend QTextStream &operator<<(QTextStream &s, const CalZeroChannelConfig &channel);

    int nent = 0;          // number of measured waveforms, must be < 2M (overflow warning)
    double base = 0;
    double rms = 0;
};

Q_DECLARE_METATYPE(CalZeroChannelConfig)

class CalZeroConfig
{
public:
    void clear() {channels.clear();}
    void clear_calzero_ch(int ch);
    void clear_enabled(const AdcConfig &adc);
    double get(int ch) const;
    double getcalnoiserms (int ch) const;
    bool isEmpty() const { return channels.isEmpty(); }

    CalZeroConfig &operator+=(const CalZeroConfig &rhs);
    bool operator==(const CalZeroConfig &other) const;
    bool operator!=(const CalZeroConfig &other) const {return !operator==(other);}

    friend CalZeroConfig operator+(CalZeroConfig lhs, const CalZeroConfig &rhs)
    { lhs += rhs; return lhs; }
    friend QTextStream &operator<<(QTextStream &s, const CalZeroConfig &v);

    QMap<int, CalZeroChannelConfig> channels;
};

Q_DECLARE_METATYPE(CalZeroConfig)

#endif // CALZEROCONFIG_H
