//
//    Copyright 2010-2019 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "AdcConfigDialog.h"

#include <iostream>

#include <QRadioButton>

#include "adcm16.h"
#include "adcmlib/ch_label.h"


AdcConfigDialog::AdcConfigDialog(QWidget *parent) :
    QDialog(parent)
{
    setupUi(this);

    textLabel_m->setText(ustr_ch_label_M);
    textLabel_s->setText(ustr_ch_label_S);

    spinBox_trig_min->setMinimum(1);
    spinBox_trig_max->setMinimum(1);
    spinBox_trig_min->setMaximum(ADC_NCH);
    spinBox_trig_max->setMaximum(ADC_NCH);

    spinBox_fragm_len->setMaximum(ADCM_MAX_N_SAMP);
    spinBox_fragm_len->setMinimum(ADCM_MIN_N_SAMP);
    spinBox_fragm_len->setSingleStep(2);

    spinBox_fragments_m->setMaximum(ADCM_MAX_N_FRAGM);
    spinBox_fragments_m->setMinimum(ADCM_MIN_N_FRAGM);

    spinBox_fragments_s->setMaximum(ADCM_MAX_N_FRAGM);
    spinBox_fragments_s->setMinimum(ADCM_MIN_N_FRAGM);

    update();
}

void AdcConfigDialog::setAdcConfig(const AdcConfig &r)
{
    if (r == adcConfig)
        return;
    adcConfig = r;
    update();
}

void AdcConfigDialog::update()
{
    radioButton_int_simple->setChecked(adcConfig.trigbits.bit.int_simple);
    radioButton_int->setChecked(adcConfig.trigbits.bit.coinc_logic);
    radioButton_ext->setChecked(adcConfig.trigbits.bit.ext1);
    radioButton_int_simple->setEnabled(false);

//    spinBox_trig_min->setMaximum(conf_mem.nch);
//    spinBox_trig_max->setMaximum(conf_mem.nch);
    spinBox_trig_min->setValue(adcConfig.mult_min);
    spinBox_trig_max->setValue(adcConfig.mult_max);

    spinBox_fragm_len->setValue(adcConfig.fragm_len);
    spinBox_fragments_m->setValue(adcConfig.fragm_count_m);
    spinBox_fragments_s->setValue(adcConfig.fragm_count_s);

    radioButtonTimestampRelative->setChecked(!adcConfig.timestamp_is_utc);
    radioButtonTimestampUTC->setChecked(adcConfig.timestamp_is_utc);
}

void AdcConfigDialog::on_buttonBox_clicked(QAbstractButton* button)
{
    switch(buttonBox->standardButton(button)) {
    case QDialogButtonBox::RestoreDefaults: {
        const auto tmp = AdcConfig();
        adcConfig.trigbits = tmp.trigbits;
        adcConfig.fragm_len = tmp.fragm_len;
        adcConfig.fragm_count_m = tmp.fragm_count_m;
        adcConfig.fragm_count_s = tmp.fragm_count_s;
        adcConfig.mult_min = tmp.mult_min;
        adcConfig.mult_max = tmp.mult_max;
        adcConfig.timestamp_is_utc = tmp.timestamp_is_utc;
        update();
        break;
    }
    case QDialogButtonBox::Apply:
    case QDialogButtonBox::Ok:
        emit adcConfigChanged(adcConfig);
        break;
    default:;
    }
}

void AdcConfigDialog::on_radioButton_int_simple_toggled(bool checked)
{
    adcConfig.trigbits.bit.int_simple = checked;
}

void AdcConfigDialog::on_radioButton_int_toggled(bool checked)
{
    adcConfig.trigbits.bit.coinc_logic = checked;
}

void AdcConfigDialog::on_radioButton_ext_toggled(bool checked)
{
    adcConfig.trigbits.bit.ext1 = checked;
}

void AdcConfigDialog::on_spinBox_fragm_len_valueChanged( int val )
{
    adcConfig.fragm_len = val;
}

void AdcConfigDialog::on_spinBox_fragments_m_valueChanged( int val )
{
    adcConfig.fragm_count_m = val;
}

void AdcConfigDialog::on_spinBox_fragments_s_valueChanged( int val )
{
    adcConfig.fragm_count_s = val;
}

void AdcConfigDialog::on_spinBox_trig_min_valueChanged( int val )
{
    adcConfig.mult_min = val;
}

void AdcConfigDialog::on_spinBox_trig_max_valueChanged( int val )
{
    adcConfig.mult_max = val;
}

void AdcConfigDialog::on_radioButtonTimestampUTC_toggled( bool )
{
    adcConfig.timestamp_is_utc = radioButtonTimestampUTC->isChecked();
}

