//
//    Copyright 2010-2019 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef DECONVCONFIGDIALOG_H
#define DECONVCONFIGDIALOG_H

#include <vector>
#include <QVector>
#include "ui_DeconvConfigDialog.h"

#include "config/adcconfig.h"
#include "config/decoderconfig.h"

class RFFTEngine;

class gauss_param_t;

class DeconvConfigDialog : public QDialog, public Ui::DeconvConfigDialog
{
    Q_OBJECT
public:
    explicit DeconvConfigDialog(QWidget* parent = nullptr);
    DecoderConfig getDecoderConfig() const {return newDecoderConfig; }
     ~DeconvConfigDialog() override;

signals:
    void decoderConfigChanged(const DecoderConfig &);

public Q_SLOTS:
    void setAdcConfig(const AdcConfig &r);
    void setDecoderConfig(const DecoderConfig &r);

private slots:
    void on_comboBoxWindowingFunction_activated( int );
    void on_sbfirnp_valueChanged( int );
    void on_sbsnf_valueChanged( double ); // Noise floor of source
    void on_sbrnf_valueChanged(double ); // Noise floor of target
    void on_sbput_valueChanged( int );
    void on_sbspumd_valueChanged( int  );
    void on_sbi1_valueChanged( int ); // Start of signal's interval
    void on_sbi2_valueChanged( int ); // End of signal's interval
    void on_sbblr1_valueChanged( int ); // Start of baseline's interval
    void on_sbblr2_valueChanged( int ); // End of baseline's interval
    void on_sbpos_valueChanged(double);
    void on_sbwid_valueChanged(double);
    void on_spinBoxChannel_valueChanged ( int inch);
    void on_sbexp_valueChanged( double );
    void on_sbblrmd_valueChanged( int );
    void on_pushButtonLoadShape_clicked();
    void on_pushButtonSaveShape_clicked();
    void on_pushButtonClearFir_clicked();
    void on_pushButtonGenerateFir_clicked();
    void on_pushButtonSaveFir_clicked();
    void on_pushButtonLoadFir_clicked();
    void on_buttonBox_clicked(QAbstractButton* button);

protected:
    void setButtons ();
    bool modified() const;
    void apply();
    void restoreDefaults();
    void replotINP();
    void replotINPFFT();
    void replotFIR();
    void replotFIRFFT();
    void replotTEST();
    void Restore();
    void Redraw();
    double reqf( int i ) const;
    double reqfunscaled( int i ) const;
//    double ca( unsigned int ch, int i ) const;
    double getTimeStepInNs() const;
    int data_latency() const;
    QVector<double> getCalAvgY(int channel) const;
    std::vector<double> getCalAvgXInNs(int channel) const;
    size_t getRefSize() const;
    std::vector<double> getRefY_n(int channel, double amplitudeScaler, int _n) const;
//    std::vector<double> getRefY(int channel, double amplitudeScaler) const;
    std::vector<double> getRefY(int channel) const;
    std::vector<double> getRefXInNs(int channel) const;
    double getCalAvgAmplitude(int channel) const;
    fwinf_t win_type() const;
    gauss_param_t find_hit(const std::vector<double> &testResult) const;

private:
    DeconvConfigDialog * const ui;
    int ch;
    int restoring;
    int disable_control_slots;
    AdcConfig adcConfig;
    DecoderConfig currentDecoderConfig;
    DecoderConfig newDecoderConfig;
    RFFTEngine * const fft;
};

#endif // DECONVCONFIGDIALOG_H
