//
//    Copyright 2010-2019 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "EvStorageConfigDialog.h"

#include <QLabel>
#include <QSpinBox>
#include <QCheckBox>
#include <QDir>
#include <QFileInfo>
#include <QFileDialog>

EvStorageConfigDialog::EvStorageConfigDialog(QWidget *parent) :
    QDialog(parent)
{
    setupUi(this);
}

void EvStorageConfigDialog::setData(const StorageConfig &r)
{
    if (r == storageConfig)
        return;
    storageConfig = r;
    update();
}

void EvStorageConfigDialog::update()
{
    checkBoxEnable->setChecked(storageConfig.enable);
    lineEdit_FileName->setText(storageConfig.fileName);
    radioButtonRecycle->setChecked(storageConfig.mode == 0);
    radioButtonContinuous->setChecked(storageConfig.mode != 0);
    checkBoxSizeLimit->setChecked(storageConfig.enableSizeLimit);
    spinBoxSizeLimit->setValue((storageConfig.szlimit + 512*1024) / 1024 / 1024);
}

void EvStorageConfigDialog::updateEnableState()
{
    lineEdit_FileName->setEnabled(storageConfig.enable);
    pushButtonSelect->setEnabled(storageConfig.enable);
    groupBoxMode->setEnabled(storageConfig.enable);
    spinBoxSizeLimit->setEnabled(storageConfig.mode == 0);
    checkBoxSizeLimit->setEnabled(storageConfig.mode == 0);
}

void EvStorageConfigDialog::on_buttonBox_clicked(QAbstractButton* button)
{
    switch(buttonBox->standardButton(button)) {
    case QDialogButtonBox::Apply :
    case QDialogButtonBox::Ok :
        emit dataChanged(storageConfig);
        break;
    case QDialogButtonBox::RestoreDefaults :
        storageConfig = StorageConfig();
        update();
        break;
    default:;
    }
}
void EvStorageConfigDialog::on_pushButtonSelect_clicked()
{
    QString s = QFileDialog::getSaveFileName(
        this, tr("Data output"),
        storageConfig.fileName,
        tr("RAW  files (*.raw *.dat)")
            + tr(";;")
            + tr("Any files (*)"));
    if (s.isNull())
        return;
    storageConfig.fileName = s;
}

void EvStorageConfigDialog::on_spinBoxPeriod_valueChanged(int value)
{
    storageConfig.period = value;
}

void EvStorageConfigDialog::on_spinBoxSizeLimit_valueChanged(int value)
{
    storageConfig.szlimit = value * 1024 * 1024;
}

void EvStorageConfigDialog::on_checkBoxSizeLimit_toggled(bool checked)
{
    storageConfig.enableSizeLimit = checked;
}

void EvStorageConfigDialog::on_checkBoxEnable_toggled(bool checked)
{
    storageConfig.enable = checked;
    updateEnableState();
}

void EvStorageConfigDialog::on_radioButtonRecycle_clicked()
{
    storageConfig.mode = 0;
    updateEnableState();
}

void EvStorageConfigDialog::on_radioButtonContinuous_clicked()
{
    storageConfig.mode = 1;
    updateEnableState();
}
