//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "RunConfigDialog.h"

#include <QLabel>
#include <QSpinBox>
#include <QCheckBox>
#include <QSettings>

RunConfigDialog::RunConfigDialog(QWidget *parent) :
    QDialog(parent)
{
    setupUi(this);

    doubleSpinBoxTime->setMinimum(0.01);
    doubleSpinBoxTime->setMaximum(1.0*86400*365);

    spinBoxEventCount->setMinimum(1);
    spinBoxEventCount->setMaximum(1000000000);
}

void RunConfigDialog::setData(const RunConfig &r)
{
    if (r == runConfig)
        return;
    runConfig = r;
    update();
}

//void RunConfigDialog::restoreDefaults()
//{
//    // Upper check-boxes
//    RunControl defaultRunControl;
//    checkBox_EnableToolTips->setChecked(false); // unused
//    checkBox_Recover->setChecked(defaultRunControl.getAutoRecover());
//    checkBox_Autostart->setChecked(defaultRunControl.getAutoRun());
//    checkBox_Autosave->setChecked(false); // there is no default presets, stores just in QSettings

//    // Stop conditions
//    RunConfig defRunConfig = RunConfig();
//    groupBoxStopCondition->setChecked(defRunConfig.stopCondition);
//    checkBoxTime->setChecked(defRunConfig.stopByTimer);
//    doubleSpinBoxTime->setValue(defRunConfig.stopTimerValue);
//    checkBoxEventCount->setChecked(defRunConfig.stopByEventCount);
//    spinBoxEventCount->setValue(defRunConfig.stopEventCountValue);

//}

//void RunConfigDialog::setData( const SettingsData & in )
//{
//    data = in;
//    QSettings s;
//    checkBox_EnableToolTips->setChecked(s.value("tooltips").toBool());
//    checkBox_Recover->setChecked(s.value("autorecover").toBool());
//    checkBox_Autostart->setChecked(s.value("autostart").toBool());
//    checkBox_Autosave->setChecked(s.value("autosave").toBool());
//    checkBox_AutosaveGUI->setChecked(s.value("autosavegui").toBool());

//    updateEnabledStatus();
//}

//void RunConfigDialog::writeSettings()
//{
//    QSettings s;
//    s.setValue("tooltips", checkBox_EnableToolTips->isChecked());
//    s.setValue("autorecover", checkBox_Recover->isChecked());
//    s.setValue("autostart", checkBox_Autostart->isChecked());
//    s.setValue("autosave", checkBox_Autosave->isChecked());
//    s.setValue("autosavegui", checkBox_AutosaveGUI->isChecked());
//    s.sync();
//}

// Stop Condition
void RunConfigDialog::on_doubleSpinBoxTime_valueChanged( double value )
{
    runConfig.stopTimerValue = value;
}

void RunConfigDialog::on_spinBoxEventCount_valueChanged( int value )
{
    runConfig.stopEventCountValue = value;
}

void RunConfigDialog::on_checkBoxTime_toggled( bool value )
{
    runConfig.stopByTimer = value;
}

void RunConfigDialog::on_checkBoxEventCount_toggled( bool value )
{
    runConfig.stopByEventCount = value;
}

void RunConfigDialog::on_groupBoxStopCondition_toggled( bool value )
{
    runConfig.stopCondition = value;
}

void RunConfigDialog::updateControls()
{
    doubleSpinBoxTime->setEnabled(runConfig.stopCondition && runConfig.stopByTimer);
    spinBoxEventCount->setEnabled(runConfig.stopCondition && runConfig.stopByEventCount);
}

void RunConfigDialog::update()
{
    // Stop Condition
    groupBoxStopCondition->setChecked(runConfig.stopCondition);

    checkBoxTime->setChecked(runConfig.stopByTimer);
    doubleSpinBoxTime->setValue(runConfig.stopTimerValue);

    checkBoxEventCount->setChecked(runConfig.stopByEventCount);
    spinBoxEventCount->setValue(runConfig.stopEventCountValue);

}

void RunConfigDialog::on_buttonBox_clicked(QAbstractButton* button)
{
    switch(buttonBox->standardButton(button)) {
    case QDialogButtonBox::Apply :
    case QDialogButtonBox::Ok :
        emit dataChanged(runConfig);
        break;
    case QDialogButtonBox::RestoreDefaults :
        runConfig = RunConfig();
        update();
        break;
    default:;
    }
}
