/*
**    Copyright 2006-2019 Ilja Slepnev
**    Copyright 2014 Ivan Filippov
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ADCMDATABLOCKDECODER_H
#define ADCMDATABLOCKDECODER_H

#include <QtGlobal>

class adcm_decstat_t;
class adcm_el_t;
class conf_mem_t;

//
// Decode frame data. Input is frame data block without header and trailer.
//
class AdcmDataBlockDecoder
{
public:
    AdcmDataBlockDecoder(int index=0) :
        decoderIndex(index),
        prev_ah_ec(index),
        prev_ec(index)
    { }
    static void unpack_samples_offbin(uint32_t bits, const uint32_t iptr[], uint32_t datablock_ns, int16_t optr[]);
    static void unpack_samples_twoscomp(uint32_t bits, const uint32_t iptr[], uint32_t datablock_ns, int16_t optr[], bool fast = true);
    int adcm_decode_data_blocks (const unsigned int *f, size_t len, adcm_el_t * el, adcm_decstat_t * st);
    void reset();
    void setAdcConfMem(const conf_mem_t &conf_mem);

protected:
    void print_decoder_data_err (adcm_decstat_t * decst, const char *fmt, ...);

private:
    const int decoderIndex;
    unsigned int prev_ah_ec;
    unsigned long prev_ec;
//    quint64 prevEvTs = 0; // timestamp of previous event
//    qint64 prevEvInterval = 0; // interval b/w previous and pre-previous events
//    quint32 badEc = 0;
//    quint64 lastPrintTs = 0; // ts of last printing
//    quint64 sumOfBadIntervals = 0;
//    quint64 sumOfGoodIntervals = 0;
    unsigned int ec_offset = 0;
    bool useLowerTimeStampBits = false;
    unsigned int raw_bits = 16;
    bool raw_signed = false;
    double raw_step = 1;
};

#endif // ADCMDATABLOCKDECODER_H
