/*
**    Copyright 2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "AdcmEventDecoder.h"
#include "config/calzeroconfig.h"
#include "config/calavgconfig.h"
#include "adcmlib/AdcmChbuf.h"
#include "adcmlib/adcm_ev.h"

CalZeroConfig AdcmEventDecoder::makeCalZero(const AdcmEvent &ev)
{
    CalZeroConfig cal;
    for (auto ch : ev.channel.keys()) {
        const adcm_chbuf_t &cbuf = ev.channel[ch];
        if (! cbuf.flags.bit.eval_rms)
            continue;
        if (! cbuf.flags.bit.has_data)
            continue;
        CalZeroChannelConfig channel(cbuf.dc, cbuf.rms);
        cal.channels[ch] += channel;
    }
    return cal;
}


CalAvgConfig AdcmEventDecoder::makeCalAvg(const AdcmEvent &ev)
{
    CalAvgConfig cal;
    for (auto ch : ev.channel.keys()) {
        const adcm_chbuf_t &cbuf = ev.channel[ch];

        if (! cbuf.flags.bit.has_thr) {
//                  qWarning (QString("avg: not test and no threshold on ch %1, flags: 0x%2").arg(c).arg(cb.flags, 8, 16));
            continue;
        }
        if (cbuf.flags.bit.has_clip)
            continue;
        if (cbuf.flags.bit.has_pu)
            continue;

        CalAvdChannelData d;
        for (auto i : cbuf.w)
            d.push_back(i - cbuf.w_calzero);
        CalAvgChannelConfig channel(d, ev.format.step);
        cal.channels[ch] += channel;
    }
    return cal;
}
