//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "waveformdecoder.h"

#include <cmath>
#include <QTextStream>
#include <QDebug>

#include "adcmlib/adcm_ev.h"
#include "adcmlib/adcm_decstat.h"
#include "adcmlib/adcm_dec.h"
#include "adcmlib/timer.h"
#include "adcm_ws_t.h"
#include "adcmraw/adcm_mem.h"
#include "config/adcconfig.h"
#include "config/allconfig.h"
#include "config/calzeroconfig.h"
#include "config/decoderconfig.h"
#include "decoder_profiling_macros.h"
#include "qlog/qlog.h"
#include "util/DebugTemplates.h"

static const int ADC_CLIP_WARNING = 5;

#ifndef PRINT_N_DSP_ERR
static const int PRINT_N_DSP_ERR = 10;
#endif

WaveformDecoder::WaveformDecoder(QObject *parent)
    : GenericDecoder(parent)
//    :   ws(new adcm_ws_t)
{
    //  adcm_dec_msgbuf = new char[ADCM_DEC_MSGBUF_SIZE];
    clearmsg();
    stages.all = 0;
    stages.bit.eval_tfine = 1;
}

WaveformDecoder::~WaveformDecoder ()
{
//    delete ws;
    //  delete [] adcm_dec_msgbuf;
    //  adcm_dec_msgbuf = 0;
}
/*
void WaveformDecoder::printmsg ()
{
  if (adcm_dec_msgbuf)
    printf ("%s", adcm_dec_msgbuf);
}

char *WaveformDecoder::msg () const
{
  return (adcm_dec_msgbuf);
}
*/

void WaveformDecoder::clearmsg ()
{
    //  if (adcm_dec_msgbuf)
    //    adcm_dec_msgbuf[0] = 0;
}
#if 0
int WaveformDecoder::makewb (const AdcmEvent &ev)
{
    ws->wb.clear();
    //  memset (&ws->wb, 0, sizeof (wb_t));

    // No need to zero ALL the channels in convolution buffer.
    // Used channels will be zeroed by convolve()
    //  memset (&ws->cb, 0, sizeof (cb_t));

    // Zero fitter buffer. - Not necessary. see adcm_ev_find_fasttime().
    //  memset (&ws->fb, 0, sizeof (fb_t));

    //  for (unsigned int c = 0; c < ADC_NCH; c++)
    //        ws->wb[c].len=0;

    for (auto n : ev.channel.keys()) {
        //  for (c = 0; c < ADC_NCH; c++) {
        //    const adcm_chbuf_t *evc = adcm_get_ev_chbuf_const(ev, c);
        //	if (!evc) continue;
        const adcm_chbuf_t &evc = ev.channel[n];
        int c = n; // evc.ch;

        wbc_t *cws = &ws->wb[c];
        //	if (!(evc->flags & ADCM_CH_HAS_THR))
        if (!(evc.flags & ADCM_CH_HAS_DATA))
        {
            cws->len = 0;
            continue;
        } else {
            // no gain from memcpy - 'for' loop is optimized
            //	  memcpy(cws->d, evc->w, evc->len*sizeof(wbc_sample_t));
            const AdcmCbufSample *wptr = &evc.w[0];
            for (unsigned int i = 0; i < evc.len(); i++)
                cws->d[i] = *wptr++;
            cws->len = evc.len();
        }
    }

#if 0
    // complete with last value
    for (int ch = 0; ch < ADC_NCH; ch++)
        if (sn[ch] > 0)
            for (int i = sn[ch]; i < cfg.adc.recwin[ch]; i++)
                ws->wb[ch].d[i] = ws->wb[ch].d[sn[ch] - 1];
#endif
    return (0);
}
#endif

// Subtract baseline for time measurements
void WaveformDecoder::subtract_calzero(AdcmEvent *ev)
{
    for (auto c : ev->channel.keys()) {
        const double cz = dec->calzero.get(c);
        for (auto &sample : ev->channel[c].w)
            sample -= cz;
    }
}

#if 0
// Subtract baseline for time measurements
int WaveformDecoder::makewb_suboffs()
{
    // not usefult when small number of channels with len>0
    //  float czall[ADC_NCH];
    //  getcalzero_all (cfg, ADC_NCH, czall);

    for (auto c : ws->wb.keys()) {
        int cz = round(dec->calzero.get(c));
        wbc_t &wbc = ws->wb[c];
        wbc_data_t &d = wbc.d;
        int len = wbc.len;
        for (int i=0; i<len; i++) {
            d[i] -= cz;
        }
        if (len<=0) continue;
        {
            wbc_data_t &s = wbc.d;
            wbc_sample_t *iptr = &s[0];
            wbc_sample_t *iptrEnd = iptr + len;
            while(iptr < iptrEnd)
                *iptr++ -= cz;
        }
    }
    return 0;
}
#endif

adcm_eval_flags_t WaveformDecoder::resolve_decoder_stages_dependencies(const adcm_eval_flags_t &f)
{
    adcm_eval_flags_t d(f);
    if (d.bit.eval_bl)
        d.bit.eval_thr = 1;
    if (d.bit.eval_int)
        d.bit.eval_thr = 1;
    if (d.bit.eval_tfine)
        d.bit.eval_tfast = 1;
    if (d.bit.eval_tfast)
        d.bit.eval_thr = 1;
    return d;
}

adcm_eval_flags_t WaveformDecoder::decoder_stages_for_mode (decoder_mode_t mode, int fitalgo)
{
    adcm_eval_flags_t job_flags;
    job_flags.all = 0;

    if (mode == DM_ZERO)
        job_flags.bit.eval_rms = 1;
    if (mode == DM_AVG) {
        job_flags.bit.eval_thr = 1;
        job_flags.bit.eval_bl = 1;
        job_flags.bit.eval_int = 1;
        job_flags.bit.eval_clip= 1;
        job_flags.bit.eval_pu = 1;
    }
    if (mode == DM_REC) {
#ifdef FIT_SINE
        job_flags.bit.eval_sine = 1;
        job_flags.bit.eval_clip = 1;
#else
        job_flags.bit.eval_bl = 1;
        job_flags.bit.eval_int = 1;
        job_flags.bit.eval_clip= 1;
        job_flags.bit.eval_pu = 1;
#endif
        if (fitalgo == FIT_ALGO_ACCURATE)
            job_flags.bit.eval_tfine = 1;
        else
            job_flags.bit.eval_tfast = 1;
    }
    job_flags = resolve_decoder_stages_dependencies (job_flags);
    return (job_flags);
}

void WaveformDecoder::decode (AdcmEvent * ev, adcm_decstat_t * decst)
{
    adcm_eval_flags_t in_stages = decoder_stages_for_mode(dec->mode, dec->fitprof.fitalgo);
    Q_ASSERT(ev);
    Q_ASSERT(decst);

    decst_t_t pt;
    ftimer_t t;
#if DECODER_PROFILING
    ftimer_t ta;
    memset (&pt, 0, sizeof (decst_t_t));
#endif

    decode_prepare(*ev, in_stages);

#if DECODER_PROFILING
    PROF_TMR_START(ta);
#endif
    PROF_TMR_START(t);

    decode_work(ev, decst, pt, t);

#if DECODER_PROFILING
    pt[decst_n_all] = ta.read();
    if (decst) {
        struct decst_prof_t prof;
        int i;

        prof.clear();
        prof.n = 1;
        for(i=0;i<decst_n_items;i++) {
            prof.ta[i] = pt[i];
            prof.tm[i] = prof.ta[i];
        }
        decst->prof += prof;
    }
#endif /* DECODER_PROFILING */

    decode_finished(*ev, decst);
}

void WaveformDecoder::print_decoder_dsp_err (adcm_decstat_t * decst, const char *fmt, ...)
{
#ifdef DEBUG
    char buf[1024] = { '\0' };
    va_list args;

    if (decst) {
        if (decst->dsp_errors_suppressed)
            return;
        if (decst->n_dsp_err() > PRINT_N_DSP_ERR) {
            qWarning("dsp error: more errors, suppressed...");
            decst->dsp_errors_suppressed = 1;
            return;
        }
    }

    va_start (args, fmt);
    vsnprintf (buf, sizeof (buf), fmt, args);
    va_end (args);

    buf[sizeof (buf) - 1] = '\0';
    //  size_t len = strlen (buf);
    //  if ((len > 0) && (buf[len - 1] != '\n'))
    //    strncat (buf, "\n", sizeof (buf)-strlen(buf));

//    for(int i=0; i<1000; ++i){
//        QString s = QString("dsp error: %1").arg(buf);
//        qWarning(s);
//        qWarning()<<s;
//    }
//    for(int i=0; i<1000; ++i)
    qWarning() << QString("dsp error: %1").arg(buf);
#endif
}

void WaveformDecoder::adcm_decoder_clear_stat (adcm_decstat_t * st)
{
    Q_ASSERT(st);
    st->clear();
    st->prof.clear();
}

void WaveformDecoder::decode_prepare(const AdcmEvent &ev, const adcm_eval_flags_t &in_stages)
{
    // find dependencies
    stages = resolve_decoder_stages_dependencies (in_stages);
    /*
  if (ev->ec<10) {
    print_ev_flags(ev);
    adcm_print_event(ev);
  }
*/
    //  log_debug(1, "ev %08u", ev->ec);
    //  log_debug(4, "adcm_ev_decode: job code %08X", stages);
}

void WaveformDecoder::decode_finished(const AdcmEvent &ev, adcm_decstat_t * decst)
{
    if (decst) {
        decst->ev_total++;
        decst_add_ev_chstat (decst, ev);
    }
    //  print_ev_flags(ev);

#ifdef DEBUG
    if (logger.getDebugLevel() >= 1 && decst && !decst->dsp_errors_suppressed) {
        adcm_cbuf_error_flags_t errors;
        adcm_cbuf_error_flags_t mask;
        mask.all = 0xFFFFFFFFU;
        mask.bit.bl_out = 0;
        for (auto c : ev.channel.keys()) {
            const adcm_chbuf_t &cb = ev.channel[c];
            errors.all |= cb.error_flags.all & mask.all;
        }
        if (errors.all)
        {
            // FIXME: use QStringSteram and logger
            printf("#---------------------------------------------%08X\n", errors.all);
            ev.print_event();
        }
    }

    if (logger.getDebugLevel() >= 2 && ev.getEventNumber() < 10) {
        // FIXME: use QStringStream
        ev.print_flags();
        ev.print_event();
    }
#endif
}

void WaveformDecoder::decode_work (AdcmEvent * ev, adcm_decstat_t * decst,
                                   decst_t_t &pt, ftimer_t &t)
{
    PROF_TMR_MARK(pt, t, decst_n_dummy);

    // fill in flags, calzero, latency
    for (auto n : ev->channel.keys()) {
        ev->channel[n].trig = adc->channels[n].flags;
        ev->channel[n].w_latency = adc->latency;
        ev->channel[n].w_calzero = dec->calzero.get(n);
    }

    adcm_ev_validate_ch(ev, decst);
    PROF_TMR_MARK(pt, t, decst_n_vld);

    if (stages.bit.eval_thr)
        adcm_ev_find_thrpos (ev, decst);
    PROF_TMR_MARK(pt, t, decst_n_thr);

    if (stages.bit.eval_rms)
        adcm_ev_find_rms (ev, decst);
    PROF_TMR_MARK(pt, t, decst_n_rms);

    if (stages.bit.eval_bl)
        adcm_ev_find_baseline (ev, decst);
    PROF_TMR_MARK(pt, t, decst_n_bl);

    if (stages.bit.eval_int)
        adcm_ev_find_integral (ev, decst);
    PROF_TMR_MARK(pt, t, decst_n_int);

    if (stages.bit.eval_clip)
        adcm_ev_find_clip (ev, decst);
    PROF_TMR_MARK(pt, t, decst_n_clip);

    if (stages.bit.eval_pu || stages.bit.eval_tfast || stages.bit.eval_tfine || stages.bit.eval_sine) {
        Q_ASSERT(ev);
//        makewb (*ev);
        PROF_TMR_MARK(pt, t, decst_n_mwb);
//        makewb_suboffs();
        subtract_calzero(ev);
        PROF_TMR_MARK(pt, t, decst_n_mwbso);
    }

    //  if (stages & ADCM_CH_EVAL_PU) adcm_ev_find_pileup (ev, adcm_dec_ws, decst);
    //    PROF_TMR_MARK(pt, t, decst_n_pu);

}


//void WaveformDecoder::adcm_ev_subtract_offset(AdcmEvent *ev)
//{
//    for (auto c : ev->channel.keys()) {
//        adcm_chbuf_t *evc = &ev->channel[c];
//        if (!(evc->flags & ADCM_CH_HAS_DATA))
//            continue;
//        int offset = ev->getAdcHalfRange();
//        for (unsigned int i=0; i<evc->len(); i++)
//            evc->w[i] -= offset;
//    }
//}

int WaveformDecoder::adcm_ev_find_thrpos (AdcmEvent * ev, adcm_decstat_t * decst)
{
    // Find threshold position
    Q_ASSERT(ev);

    const bool calibration_mode = adc->is_testmode();

    for (auto n : ev->channel.keys()) {
        adcm_chbuf_t *evc = &ev->channel[n];
        if (!evc->flags.bit.has_data)
            continue;
        int c = evc->ch;
        const adcm_channel_flags_t &flags = adc->channels[c].flags;
        const bool zero_suppression = !calibration_mode && flags.bit.zero;

        // do not search pulses when zero suppression disabled
        // FIXME - it is not working. commented out.
        //    if (!zero_suppression)
        //      continue;

        // Count pulse here
        if (decst)
            decst->dsp_total++;

        bool triggerM = false;
        if (!(flags.bit.en)) {
            print_decoder_dsp_err (decst, "ch %d: unexpected data on disabled channel (no En)", c);
            continue;
        }
        triggerM = flags.bit.m;
        /*
    if (!tt) {
      print_decoder_dsp_err (decst, "ch %d: unexpected data on disabled channel (no M or S)", c);
//      printf("***\n");
      continue;
    }
*/

        {
            const int len = evc->len();
            int start = adc->latency;
            int end = start+1;
            if (!triggerM) {
                start -= adc->prewin;
                end = start + adc->matchwin + 1;
            }
            //printf("adc_latency (cfg)=%f\n", adc_latency (cfg));
            //start--;
            //end++;

            //printf("*** find thrpos: len=%d, start=%d, end=%d\n", len, start, end);
            bool selftrig = (flags.bit.m && flags.bit.s);
            if (!zero_suppression || selftrig) { start = 1; end = len; }

            if (start < 1)
                start = 1;
            if (end > len)
                end = len;

            evc->flags.bit.has_thr = 0;
            {
                int i;
                int thr_mv = adc->channels[c].thr;
                int cur_thr = AllConfig::thr_mv_to_value (dec->calzero, *adc, c, thr_mv);
                //    log_debug (3, "ch %d: threshold search, thr:%d", c, cur_thr);
                //      if (ev->ec<20) printf("ch %d: threshold search %d-%d, thr:%d\n", c, start, end, cur_thr);
                for(i=start; i< end; i++) {
                    // THRESHOLD RULE HERE //
                    if (evc->w[i-1] < cur_thr && evc->w[i] >= cur_thr) {
                        evc->tp = i;
                        evc->flags.bit.has_thr = 1;
                        //          if (ev->ec<20) printf("ch %d: threshold found at %d\n", c, i);
                        //        log_debug (3, "ch %d: Found threshold position at %d: %d >= %d", c, i, evc->w[i], cur_thr);
                        break;
                    }
                }
                // threshold not found in expected range. raise arror and try search full range
                if (zero_suppression && !adc->trigbits.bit.ext1 && !evc->flags.bit.has_thr) {
                    if (decst)
                        decst->dsp_thr_nf++;
                    bool fb_thr_found = false;
                    for(i=1; i< len; i++) {
                        // THRESHOLD RULE HERE //
                        if (evc->w[i-1] < cur_thr && evc->w[i] >= cur_thr) {
                            evc->tp = i;
                            evc->flags.bit.has_thr = 1;
                            fb_thr_found = true;
                            break;
                        }
                    }
                    if (fb_thr_found) {
                        print_decoder_dsp_err (decst, "Ev %d; ch %d: threshold at %d is outside expected range %d-%d", ev->getEventNumber(), c, evc->tp, start, end-1);
                    } else {
                        print_decoder_dsp_err (decst, "Ev %d; ch %d: threshold not found in range %d-%d", ev->getEventNumber(), c, start, end);
                    }
                }
            }

        }
        evc->flags.bit.has_thr = 1;
    }
    return (0);
}

bool WaveformDecoder::adc_find_integration_range (AdcmEvent * ev, unsigned int ch, int r1, int r2, int *fi1, int *fi2)
{
    Q_ASSERT(ev);

    int tp, i1, i2;
    adcm_chbuf_t *evc = &ev->channel[ch];

    if (!evc->flags.bit.has_thr) {
        qWarning() << QString("ch %1: requested find_int_range but no threshold found").arg(ch);
        return false;
    }
    tp = evc->tp;
    i1 = tp + r1;
    i2 = tp + r2;

    //if (ev->ec<100 && ch==17) printf("*** integrator: tp=%d, i1=%d, i2=%d, n=%d\n", tp, i1, i2, i2-i1);

    if ((i1 < 0) || (i2 > int(evc->len()))) {
        return false;
    }
    if (fi1)
        *fi1 = i1;
    if (fi2)
        *fi2 = i2;
    return true;
}

// Calculate BaseLine
int WaveformDecoder::adcm_ev_find_baseline (AdcmEvent * ev, adcm_decstat_t * decst)
{
    Q_ASSERT(ev);

    for (auto n : ev->channel.keys()) {
        //  for (c = 0; c < ADC_NCH; c++) {
        int i1, i2;
        bool rangeFound;

        adcm_chbuf_t *evc = &ev->channel[n];
        int c = n; // evc->ch;
        //    adcm_chbuf_t *evc = ev->get_ev_chbuf(c);
        //    if (!evc) continue;

        if (!evc->flags.bit.has_thr)
            continue;
        // Find out start/end indexes of baseline interval; blr1,blr2 relative to threshold pos
        rangeFound = adc_find_integration_range (ev, c, dec->channels[c].blr1, dec->channels[c].blr2, &i1, &i2);

        if (!rangeFound) {
            evc->error_flags.bit.bl_nodata = 1;
            if (decst)
                decst->dsp_bl_range++;
            print_decoder_dsp_err (decst, "Ev %d; ch %d: insufficient data for baseline", ev->getEventNumber(), c);
            continue;
        }
        evc->error_flags.bit.bl_nodata = 0;

        {
            int baseLineSum = 0;
            const AdcmCbufSample *wptr = &evc->w[i1];
            for (int i = i1; i < i2; i++)
                //        val += evc->w[i];
                baseLineSum += *wptr++;
            if (i2 != i1)
                //        val /= (double) (i2 - i1);
                evc->b = (double) baseLineSum / (i2 - i1);
            else
                evc->b=0;
        }

        {
            const double bref = dec->calzero.get(c);
            const double brms = dec->calzero.getcalnoiserms (c);
            const double nq = 1. / sqrt (12.);
            double nt;

            // lower limit on brms
            nt = std::max(brms, nq);
            if (fabs(evc->b - bref) > dec->channels[c].baserc * nt)
            {
                evc->error_flags.bit.bl_out = 1;
                //        ev->flags |= F_BLRERR;
                //        print_decoder_dsp_err(decst, "ch=%d, Baseline out of limits: %.2f, calibrated %.2f (rms %.2f), maxdev %.2f)\n", c, ev->ch[c].b, bref, brms, dec->baserc[c]*nt);
            }
        }
        evc->flags.bit.eval_bl = 1;
        //    if (ev->ec<100 && c==17)
        //	  printf("baseline: ev %u, i1=%i, i2=%i, bl=%f\n", ev->ec, i1, i2, evc->b);
    }
    return (0);
}

// Calculate Integral
int WaveformDecoder::adcm_ev_find_integral (AdcmEvent * ev, adcm_decstat_t * decst)
{
    Q_ASSERT(ev);

    for (auto n : ev->channel.keys()) {
        //  for (c = 0; c < ADC_NCH; c++) {
        //    adcm_chbuf_t *evc = ev->get_ev_chbuf(c);
        //	if (!evc) continue;
        adcm_chbuf_t *evc = &ev->channel[n];
        int c = n; // evc->ch;

        int i1=0, i2=0;
        bool rangeFound;

        if (!evc->flags.bit.has_thr)
            continue;
        // Find out start/end indexes of integral interval; int1,int2 relative to threshold pos
        rangeFound = adc_find_integration_range (ev, c, dec->channels[c].int1, dec->channels[c].int2, &i1, &i2);
        if (!rangeFound) {
            evc->error_flags.bit.int_nodata = 1;
            if (decst)
                decst->dsp_int_range++;
//            print_decoder_dsp_err (decst, "Ev %d; ch %d: insufficient data for integral: i1=%d, i2=%d, tp=%d", ev->getEventNumber(), c, i1, i2, evc->tp);
            continue;
        }

        //    evc->flags &= ~ADCM_CH_INT_NODATA;
        //    print_decoder_dsp_err (decst, "Ev %d; ch %d: integral range: i1=%d, i2=%d, tp=%d", c, i1, i2, ev->ch[c].tp);

        int integralSum = 0;
        const AdcmCbufSample *wptr = &evc->w[i1];
        for (int i = i1; i < i2; i++)
            integralSum += *wptr++;

        if (i2 != i1)
            //      val /= (double) (i2 - i1);
            evc->i = (double)integralSum / (i2 - i1);
        else
            evc->i = 0;
        evc->flags.bit.eval_int = 1;
        /*
    if (ev->ec<100 && c==17) {
          printf("integrator: ev %u, i1=%i, i2=%i, inte=%f\n", ev->ec, i1, i2, evc->i);
    int val = 0;
        int i;
    const unsigned short int *wptr = &evc->w[i1];
        printf("DATA: ");
    for (i = i1; i < i2; i++)
      printf("%d ", *wptr++);
        printf("\n");
        }
*/
    }
    return (0);
}

int WaveformDecoder::adcm_ev_find_clip (AdcmEvent * ev, adcm_decstat_t * decst)
{
    Q_ASSERT(ev);

    for (auto n : ev->channel.keys()) {
        //  for (c = 0; c < ADC_NCH; c++) {
        //    adcm_chbuf_t *evc = ev->get_ev_chbuf(c);
        //	if (!evc) continue;
        adcm_chbuf_t *evc = &ev->channel[n];
        int c = n; // evc->ch;

        if (!evc->flags.bit.has_data)
            continue;

        {
            const AdcmCbufSample *wptr = &evc->w[0];
            int adcmin = -(1 << (evc->w_bits-1)) + ADC_CLIP_WARNING;
            int adcmax = (1 << (evc->w_bits-1)) - 1 - ADC_CLIP_WARNING;
            for (size_t i = 0; i < evc->len(); i++) {
                if (*wptr >= adcmax) {
                    evc->flags.bit.has_clip = 1;
                    //        ev->flags |= F_CLIP;
                    if (decst)
                        decst->dsp_clip_pos++;
                    print_decoder_dsp_err (decst, "ch %d: positive CLIPING at sample %d (%d >= %d)", c, i, evc->w[i], adcmax);
                    break;
                }
                if (*wptr <= adcmin) {
                    evc->flags.bit.has_clip = 1;
                    //        ev->flags |= F_CLIP;
                    if (decst)
                        decst->dsp_clip_neg++;
                    print_decoder_dsp_err (decst, "ch %d: negative CLIPING at sample %d (%d <= %d)", c, i, evc->w[i], adcmin);
                    break;
                }
                wptr++;
            }
            evc->flags.bit.eval_clip = 1;
        }
    }
    return (0);
}

int WaveformDecoder::adcm_ev_find_rms (AdcmEvent * ev, adcm_decstat_t * decst)
{
    Q_ASSERT(ev);

    for (auto n : ev->channel.keys()) {
        //  for (c = 0; c < ADC_NCH; c++) {
        //    adcm_chbuf_t *evc = ev->get_ev_chbuf(c);
        //	if (!evc) continue;
        adcm_chbuf_t *evc = &ev->channel[n];
        //    int c = evc->ch;

        const AdcmCbufWaveform &d = evc->w;

        if (!evc->flags.bit.has_data)
            continue;
        const size_t len = evc->len();
        if (len == 0)
            continue;

        qint64 dc = 0;
        for (size_t i = 0; i < len; i++)
            dc += d[i];
        double dcnorm = 1.0 * dc / len;
        double rms = 0;
        for (size_t i = 0; i < len; i++) {
            double tmp = d[i] - dcnorm;
            rms += tmp * tmp;
        }
        evc->dc = dcnorm;
        evc->rms = sqrt ((float)rms/len);
        evc->flags.bit.eval_rms = 1;
        //    printf ("ch %d: dc %.2f, rms %.2f\n", c, evc->dc, evc->rms);
        //    log_debug (4, "ch %d: dc %.2f, rms %.2f", c, dc, rms);
    }
    return (0);
}

int WaveformDecoder::adcm_ev_validate_ch (AdcmEvent * ev, adcm_decstat_t * decst)
{
    Q_ASSERT(ev);
    for (auto n : ev->channel.keys()) {
        adcm_chbuf_t *evc = &ev->channel[n];
        if (!evc->flags.bit.has_data)
            continue;

        int c = evc->ch;
        if (!evc->flags.bit.finished) {
            if (decst)
                decst->dsp_bad_len++;
            print_decoder_dsp_err (decst, "ch %d: incomplete waveform, length %d", c, evc->len());
            evc->error_flags.bit.bad_len = 1;
        } else {
            if (decst)
                decst->dsp_good_waveform++;
        }

        unsigned int explen = adc->recwin(c);
            if (evc->len() != explen) {
                if (decst)
                    decst->dsp_bad_len++;
                print_decoder_dsp_err (decst, "ev %d ch %d: bad length %d, expected %d",ev->getEventNumber(), c, evc->len(), explen);
                evc->error_flags.bit.bad_len = 1;
            }
        //    for(int i=0;i<evc->len;i++) {
        //  	  if ((evc->w[i] < 512-20) || (evc->w[i] > 512+20)) {
        //        log_debug (0, "ch %d: [%02d] bad data %d", c, i, evc->w[i]);
        //	  }
        //    }
    }
    return (0);
}

inline int WaveformDecoder::cb_peak_rule(int *d, int ip)
{
    return d[ip-1] + d[ip] + d[ip+1];
}

#if 0
int WaveformDecoder::cb_find_pileup (int ch, adcm_chbuf_t *cb, adcm_decstat_t *decst)
{
    Q_ASSERT(cb);
    auto conv = &cb->conv;
    int i1 = conv->i1;
    int i2 = conv->i2;
    int ip = conv->ip;

    int mindist = static_cast<int>(trunc(1.0 * dec->channels[ch].pumindist / adc->step));
    double thr = 0.01 * dec->channels[ch].puthr;
    if (thr >= 0.999)
        return (0);
    // LOG("mindist=%d, thr=%.2f\n", mindist, thr);
    int peakval = cb_peak_rule (&conv->w[0], ip);
    int thrval = static_cast<int>(trunc(peakval * thr));
    int warnval = (int) (0.75 * thrval);
    int maxval = 0;
    //  int mp = 0;
    //  int md = 0;
    for (int i = i1 + 1; i < i2 - 1; i++) {
        int dist = i - ip;
        if (dist < 0)
            dist = (-dist);
        if (dist >= mindist) {
            int a = cb_peak_rule (&conv->w[0], i);
            if (a > maxval) {
                maxval = a;
                //        mp = i;
                //        md = dist;
            }
            if (a >= thrval) {
                /*
        de->m[ch].flags |= F_PILEUP;
        de->flags |= F_PILEUP;
//          LOG("signal PILEUP at sample %d: ampl %d. peak at %d, ampl %d. dist %d\n",           i, a, ip, peakval, dist);
*/
                //        print_decoder_dsp_err (decst, "ch=%2d PILEUP threshold at %d: %.2f, dist %d. peak %d at %d", ch, i,
                //                   peakval ? (1. * a / peakval) : 0, dist, peakval, ip);
                return (1);
            } else if (a >= warnval) {
                // LOG("PILEUP warning   at %d: %.2f, dist %d\n", i, peakval?(1.*a/peakval):0, dist);
            }
        }
    }
    // LOG("minor  at %d: %.2f, dist %d\n", mp, peakval?(1.*maxval/peakval):0, md);
    return (0);
}
#endif
