//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "waveformdecoderpeaks.h"

#include <cmath>
#include <QDebug>

#include "dsplib/dsplib.h"
#include "dsplib/fitter.h"
#include "adcm_ws_t.h"
#include "config/allconfig.h"
#include "adcmraw/adcm_mem.h"
#include "dsplib/fir.h"
#include "adcmlib/adcm_decstat.h"
#include "adcmlib/AdcmChbuf.h"
#include "adcmlib/adcm_ev.h"
#include "adcmlib/adcm_dec.h"
#include "decoder_profiling_macros.h"
#include "qlog/qlog.h"

#ifndef DEC_DEBUG
static const int DEC_DEBUG = 0;
#endif

WaveformDecoderPeaks::WaveformDecoderPeaks (QObject *parent)
    : WaveformDecoder(parent),
    fitterGaus(new FitterGaus(MAX_FITGAUS_POINTS))
{
}

WaveformDecoderPeaks::~WaveformDecoderPeaks ()
{
    delete fitterGaus;
}

void WaveformDecoderPeaks::update_peaksearch_range(AdcmEvent * ev, adcm_decstat_t * decst)
{
    const DecoderConfig &decoderConfig = *dec;

    for (auto c : ev->channel.keys()) {
        const auto &fir = decoderConfig.channels[c].deconv.fir;
        int fside = AllConfig::get_dec_fitside(adc->step, decoderConfig, c);

        int search_extend = 4;
        int SEARCH_START = adc->latency - search_extend;
        int SEARCH_END = SEARCH_START + 2 * search_extend;
        const adcm_channel_flags_t &trig = adc->channels[c].flags;
        if (trig.bit.s) {
            SEARCH_START -= adc->prewin;
            SEARCH_END += adc->matchwin;
        }
        const auto &conv = ev->channel[c].conv;

        if (SEARCH_START < 1)
            SEARCH_START = 1;
        if (SEARCH_END >= conv.w.size())
            SEARCH_END = conv.w.size() - 1;

//        printf("peak search: %d-%d\n", SEARCH_START, SEARCH_END);

        // find FIR delay
        int fir_dt = fir.offs;
        int firlen = fir.coef.size();
        ev->channel[c].conv.offs = fir_dt; // - cin1;

        // find interesting convolution region

        // Left margin = 0. Uncomment. FIXME.
        //    cin1 = 0;
//        int cin1 = SEARCH_START - firlen;
//        if (cin1 < 0)
//            cin1 = 0;
//        int cin2 = SEARCH_END + firlen;
//        if (cin2 > conv.w.size())
//            cin2 = conv.w.size();

        // peak search range i1<=t<i2
        //    int icentr = fir_dt;
        int i1 = SEARCH_START + conv.offs;
        int i2 = SEARCH_END + conv.offs;
        if (i1 < fside)
            i1 = fside;
        if (i2 > conv.w.size() - fside)
            i2 = conv.w.size() - fside;

        ev->channel[c].conv.i1 = i1;
        ev->channel[c].conv.i2 = i2;
    }
}

#if 0
int WaveformDecoderPeaks::convolve_ch(adcm_decstat_t * decst, int c)
{
    const DecoderConfig & decoder = *dec;
    const intfir_t &fir = decoder.channels[c].fir;

    const wbc_t *wb = &ws->wb[c];
    cb_t *cb = &ws->cb[c];

    if (wb->len < 3)
        return(0);

    int fside = AllConfig::get_dec_fitside(adc->step, decoder, c);
    // First, convolver is run over signal

    int search_extend = 4;
    int SEARCH_START = adc->latency - search_extend;
    int SEARCH_END = SEARCH_START + 2 * search_extend;
    const adcm_channel_flags_t &trig = adc->channels[c].flags;
    if (trig.bit.s) {
        SEARCH_START -= adc->prewin;
        SEARCH_END += adc->matchwin;
    }

    if (SEARCH_START < 1)
        SEARCH_START = 1;
    if (SEARCH_END >= wb->len)
        SEARCH_END = wb->len - 1;

    //  printf("peak search: %d-%d\n", SEARCH_START, SEARCH_END);

    // find FIR delay
    int fir_dt = fir.offs;
    int firlen = fir.coef.size();

    // find interesting convolution region

    // Left margin = 0. Uncomment. FIXME.
    //    cin1 = 0;
    int cin1 = SEARCH_START - firlen;
    if (cin1 < 0)
        cin1 = 0;
    int cin2 = SEARCH_END + firlen;
    if (cin2 > int(CONVLEN))
        cin2 = CONVLEN;

    // do convolution
    //  LOG("ch %d, FIR: %d taps\n", c, dec->fir[c].klen);
    int inp_len = cin2-cin1;
    int cblen;
    if (inp_len > 0 && firlen > 0) {
        fir_convolve_s (&fir, &wb->d[cin1], inp_len, &cb->d[0], CONVLEN);
        cblen = inp_len + firlen - 1;
    } else {
        for (int i = 0; i < inp_len; i++) { // direct copy
            //        ws->cb[c].len = ws->wb[c].len;
            ws->cb[c].len = inp_len;
            if (i < wb->len)
                cb->d[i] = wb->d[i+cin1];
            else
                cb->d[i] = 0;
        }
        cblen = inp_len;
    }
    cb->len = cblen;
    //printf("fir_dt=%d,  cin1=%d\n", fir_dt, cin1);
    cb->offs = fir_dt - cin1;
    //    cb->offs = cin1;

    // peak search range i1<=t<i2
    //    int icentr = fir_dt;
    int i1 = SEARCH_START + cb->offs;
    int i2 = SEARCH_END + cb->offs;
    if (i1 < fside)
        i1 = fside;
    if (i2 > cblen - fside)
        i2 = cblen - fside;

    cb->i1 = i1;
    cb->i2 = i2;

    return 0;
}
#endif

void WaveformDecoderPeaks::make_convolve(AdcmEvent * ev) const
{
    for (auto c : ev->channel.keys()) {
        if (dec->channels[c].deconv.fir.isEmpty()) {
            ev->channel[c].conv.w.clear();
            for (auto i : ev->channel[c].w)
                ev->channel[c].conv.w.push_back(i);
        } else {
            convolve(dec->channels[c].deconv.fir.coef, ev->channel[c].w, &ev->channel[c].conv.w);
        }
        //    print_cb(adc_latency (cfg), &ws->cb[c]);
    }
}

// find largest peak with 3x moving average filter
// in range i1+1 <= i < i2+1
// return peak position ip, alplitude ap
void WaveformDecoderPeaks::find_peak (adcm_chbuf_t *cb, adcm_decstat_t *decst) const
{
    auto *conv = &cb->conv;
    const auto &d = cb->conv.w;
    conv->peaks.clear();
    if (d.empty())
        return;

    int i1 = conv->i1+1;
    int i2 = conv->i2-1;

    if (i2 - i1 < 1) {
        if (decst)
            decst->dsp_pk_range++;
//        print_decoder_dsp_err (decst, "ch %d, peak search range bad: %d - %d", c, i1, i2);
//        adcm_ev_ch_flag_setbit(ev, c, ADCM_CH_PKS_NODATA);
        cb->error_flags.bit.pks_nodata = 1;
        //      ev->ch[c].flags |= ADCM_CH_PKS_NODATA;
        //      de->flags |= F_TERR;
        return;
    }

    //  print_decoder_dsp_err (decst, "ch %d, peak not found in %d - %d", c, i1, i2);

    int ip = (i2+i1)/2;                      // default
    /*
  ap = cb_peak_rule (d, ip);
  for (int i = i1; i < i2; i++) {
    if (cb_peak_rule (d, i) > ap) {
      ip = i;
      ap = cb_peak_rule (d, i);
    }
  }
*/
    int ap = d[ip];
//    // Fast moving average filter
//    {
//        int tmp;
//        tmp = d[i1-1]+d[i1]+d[i1+1];
//        ap = tmp;
//        for (int i = i1; i < i2; i++) {
//            if (tmp>ap) {
//                ip=i;
//                ap=tmp;
//            }
//            tmp += -d[i-1]+d[i+2];
//        }
//    }
    for (int i=1; i < d.size() - 1; i++) {
        if (d[i] > ap) {
            ip = i;
            ap = d[i];
        }
    }
    adcm_peak_t peak;
    peak.pt_coarse = ip;
    peak.pt = ip;
    peak.pa_coarse = d[ip];
    peak.pa = d[ip];
    peak.pw_coarse = 1;
    peak.pw = 1;
    conv->peaks.push_back(peak);
    //  printf ("ch %d, peak found in %d - %d at %d\n", c, i1, i2, ip);
}

#if 0
int WaveformDecoderPeaks::find_pileup (AdcmEvent *ev, adcm_decstat_t * decst)
{
    Q_ASSERT(ev);

    for (auto c : ev->channel.keys())
    {
        int pu_code;
        pu_code = cb_find_pileup (c, &ev->channel[c].conv, decst);
        if (pu_code == 1)
            ev->channel[c].flags |= ADCM_CH_HAS_PU;
//            adcm_ev_ch_flag_setbit(ev, c, ADCM_CH_HAS_PU);
    }
    return 0;
}
#endif

int WaveformDecoderPeaks::find_fasttime (int method, AdcmEvent * ev, adcm_decstat_t * decst)
{
#if 0
    Q_ASSERT(ev);

    const int lat = adc->latency;

    for (auto c : ev->channel.keys())
    {
        auto channel = &ev->channel[c];
        auto conv = &channel->conv;

        if ( ! (channel->flags & ADCM_CH_HAS_THR))
            continue;

        if ( ! (channel->flags & ADCM_CH_EVAL_TFAST)) {
            //    double chisq=0;
            int fb_offset;
            int fb_len;

            //    find_peak (cfg, cb);

            //    print_cb(lat, &ws->cb[c]);


            // Fill fitbuffer - fb is durty in enter.
            {
                const int fside = AllConfig::get_dec_fitside(adc->step, *dec, c);
                int fi1 = conv->ip - fside;
                if (fi1 < 0) fi1 = 0;
                int fi2 = fi1 + 2*fside + 1;
                if (fi2 > conv->w.size())
                    fi2 = conv->w.size() - 1;
                for(int i=fi1; i<fi2; i++)
                    fit->w.push_back(conv->w[i]);
                fit->offs = -fi1;
                fb_offset = fi1;
                fb_len = fit->w.size();
            }

            //    print_fb(lat + cb->offs, fb);
            double t=0, a=0, w=0;

            {
                double par[3];
                // initial conditions
                //    par[1] = (fside + fside) / 2;
                //    par[2] = DEFAULT_SIGMA;

                //      FindGausApproxD (fb->d, fb->len, par);
                FindGausApprox (&conv->w[fb_offset], fb_len, par);
                t = par[1] - lat - fit->offs - conv->offs;
                w = par[2];
                a = par[0];
                channel->pchi2 = channel->fastnorm = FindGausDevNorm (&conv->w[fb_offset], fb_len, par) / a / fb_len;
                par[2] = dec->channels[c].deconv.pwid/GausFWHM;
            }

            channel->flags |= ADCM_CH_EVAL_TFAST;
            channel->pa = a;
            channel->fasttime = t;
            channel->pt = t;
            channel->pw = w*GausFWHM;
            //    evc->pchi2 = chisq;
            //    printf("t=%f, ip=%d\n", t, cb->ip);
        } // if ADCM_CH_EVAL_TFAST

    }
#endif
    return (0);
}

int WaveformDecoderPeaks::find_finetime (int method, AdcmEvent * ev, adcm_decstat_t * decst)
{
    Q_ASSERT(ev);

    const int lat = adc->latency;

    // Measure fine time

    for (auto c : ev->channel.keys())
    {
        auto evc = &ev->channel[c];
        auto conv = &evc->conv;
        if (conv->w.empty())
            continue;

//        if ( ! (evc->flags & ADCM_CH_EVAL_TFAST)) continue;
        if (evc->flags.bit.dt_cut) continue;

        for (auto &peak : conv->peaks) {
//            const int fside = dec->fitprof.fitside;
            const int fside = AllConfig::get_dec_fitside(adc->step, *dec, c);
            int ip1 = peak.pt_coarse - fside;
            if (ip1 < 0) ip1 = 0;
            int ip2 = peak.pt_coarse + fside;
            if (ip2 >= conv->w.size())
                ip2 = conv->w.size() - 1;
            const int fit_len = ip2 - ip1 + 1;
            std::vector<double> fit_data; // FIXME: optimize
            fit_data.reserve(fit_len);
            for (int i = ip1; i <= ip2; i++)
                fit_data.push_back(conv->w[i]);
            bool fit_ok = true;
            const double tpk = peak.pt;

            //      if (method == FIT_ALGO_ACCURATE)
            {
                std::array<double, 3> par = {0};
                fitstatus_t fitstatus;
                fitstatus.niter = dec->fitprof.get_dec_fititer();

                int ret = fitterGaus->fit (dec->fitprof.get_dec_fitfixpar(), &fit_data[0], fit_len, &par[0], &fitstatus);

                if (ret != 0) {
                    if (decst)
                        decst->dsp_fit_fail++;
                    print_decoder_dsp_err (decst, "Ev %d; ch %d: fit failed, code=%d", ev->getEventNumber(), c, ret);
                    evc->error_flags.bit.tfit_err = 1;
                    fit_ok = false;
                }
                peak.pa = par[0];
                peak.pt = ip1 + par[1];
                peak.pw = par[2];
                peak.pniter = fitstatus.niter;
                peak.pchi2 = fitstatus.fnorm / peak.pa / fit_len;
                //    	  printf("par %f,  tpk %f  t %f,  cb.offs %d,  fb.offs %d\n", par[1], tpk, t, cb->offs, fb->offs);
            }
            //    log_debug (3, " * ch=%d, ip=%d, a=%g, t=%.2f, w=%.2f\n", c, ip, a, t, w);
            if (abs(peak.pt - peak.pt_coarse) > fside) {
                if (DEC_DEBUG >= 1) {
                    if (decst)
                        decst->dsp_fit_bad++;
                    print_decoder_dsp_err (decst, "Ev %d; ch %d: fit is bad, pt_coarse=%d, pt_fit=%.2f",
                                          ev->getEventNumber(), c, peak.pt_coarse, peak.pt);
                }
                evc->error_flags.bit.tfit_off = 1;
                fit_ok = false;
            }
            if (fit_ok) {
                if (decst)
                    decst->dsp_fit_ok++;
                //      log_debug(3, " + ch %d: t=%.2f", c, t);
            } else {
                evc->error_flags.bit.tfit_err = 1;
            }
//            peak.pt -= (lat - conv->offs); // shift by latency
            peak.pw *= GausFWHM;
            // debug data
            //printf("pp=%d, fasttime=%f, t=%f\n", ev->ch[c].pp, ev->ch[c].fasttime, evc->pt);
            if (fit_ok) evc->flags.bit.eval_tfine = 1;

            /*
// DEBUG DUMP
        printf("ev %6d, ch %2d, thr at %3d -- %s\n", ev->ec, c, evc->tp, tok?"ok":"BAD");
        if (!tok)
        if (adc->trig[c] & ADCM_CCTRL_BIT_S)
        {
          printf("ev %6d, ch %2d, thr at %3d\n", ev->ec, c, evc->tp);
//      print_wb(ev->ch[c].tp, &ws->wb[c]);
      print_cb(lat, cb);
      print_fb(lat + cb->offs, fb);
        }
*/
        }
    }

    return (0);
}

int WaveformDecoderPeaks::find_tfast (AdcmEvent * ev, adcm_decstat_t * decst)
{
    return find_fasttime (FIT_ALGO_COARSE, ev, decst);
}

int WaveformDecoderPeaks::find_tfine (AdcmEvent * ev, adcm_decstat_t * decst)
{
    return find_finetime (dec->fitprof.get_dec_fitalgo(), ev, decst);
}

void WaveformDecoderPeaks::decode_work (AdcmEvent * ev, adcm_decstat_t * decst,
                                        decst_t_t &pt, ftimer_t &t)
{
    WaveformDecoder::decode_work(ev, decst, pt, t);

    if (stages.bit.eval_tfast || stages.bit.eval_tfine) {

        make_convolve(ev);
        PROF_TMR_MARK(pt, t, decst_n_conv);

        update_peaksearch_range(ev, decst);
        peaksearch (ev, decst);
        PROF_TMR_MARK(pt, t, decst_n_pksrch);

//        find_pileup (ev, decst);
        PROF_TMR_MARK(pt, t, decst_n_pu);

        find_tfast (ev, decst);
        PROF_TMR_MARK(pt, t, decst_n_tfast);

        bool pulseFound = find_dt_cut_condition(ev, decst); // TODO
        PROF_TMR_MARK(pt, t, decst_n_dtcut);
        if (!pulseFound) { // no pulses in event - drop event
            ev->setFlag(AdcmEvent::ADCM_EV_DT_CUT);
            if (decst)
                decst->ev_dt_cut++;
        } else {
            if (stages.bit.eval_tfine) {
                find_tfine (ev, decst);
                PROF_TMR_MARK(pt, t, decst_n_fit);
            }
        }
    }
}

int WaveformDecoderPeaks::simulate(AdcmEvent * ev)
{
    Q_ASSERT(ev);

    double a, t, rms, base;

    for (auto c : ev->channel.keys()) {
        adcm_chbuf_t *ch = &ev->channel[c];
        ch->flags.bit.has_data = 0;
        const AdcChannelConfig &channelConfig = adc->channels[c];
        const adcm_channel_flags_t &trig = channelConfig.flags;
//        if (!(trig.bit.m || trig.bit.s))
//            continue;
        //    if (c > 1) break;
//        base = dec->calzero.get(c);
        base = 0; // 1<<(adc->bits-1);
        ch->w_bits = adc->bits;
        if (ch->conv.peaks.empty())
            continue;
        a = ch->conv.peaks[0].pa;
        rms = ch->rms;
        unsigned int len = adc->recwin(c);
        if (len > ADCM_MAX_N_SAMP) len = ADCM_MAX_N_SAMP;
        ch->w.resize(len);
#ifdef FIT_SINE
        t = ch->pt;
        double f = ch->pw;
        for (unsigned int i = 0; i < len; i++)
            ch->w[i] = round(rangaus (rms) + base + a * sin(t + f * i));
#else
        t = ch->conv.peaks[0].pt + adc->latency;
        const double rt = ch->conv.peaks[0].pw;
        const double ft = rt * 10;
        for (unsigned int i = 0; i < len; i++) {
            // double v = detpulse (a, i - t, rt / GausFWHM, ft);
            double v = a * gauss(i - t, rt);
            ch->w[i] = round(rangaus (rms) + base + v);
        }
#endif
        ch->flags.bit.has_data = 1;
        ch->flags.bit.finished = 1;
    }

    /*
{
  FILE *f;
  f = fopen("sim_ev.txt", "w+");
  if (f) {
  for(int c=0;c<ADC_NCH;c++)
        for(int i=0;i<ev->ch[c].len;i++)
          fprintf(f, "%d\t%d\t%d\n", c, i, ev->ch[c].w[i]);
  fclose(f);
  }
}
*/
    return 0;
}

void WaveformDecoderPeaks::peaksearch (AdcmEvent * ev, adcm_decstat_t * decst)
{
    for (auto c : ev->channel.keys()) {
        find_peak (&ev->channel[c], decst);
    }
}

// return true if any useful pulses found
bool WaveformDecoderPeaks::find_dt_cut_condition(AdcmEvent *ev, adcm_decstat_t *decst)
{
    Q_ASSERT(ev);

    bool pulseFound = false;

    if (!dec->dt_cut_en)
        return true;

    double dtMin=dec->dt_cut_l;
    double dtMax=dec->dt_cut_r;
    //printf("dt cut: l=%f, r=%f\n", l, r);
    // clear 'match' flags
    QMap<int, bool> chUsed;
//    for(unsigned int i=0; i<nch; i++)
//        chUsed[i]=false;

    for (auto c : ev->channel.keys()) {
        adcm_chbuf_t *cb = &ev->channel[c];
        cb->flags.bit.dt_cut = 0;
    }

    for (auto mCh : ev->channel.keys()) {
        //	int mc_match = 0;
        const AdcChannelConfig &mChannelConfig = adc->channels[mCh];
        if (!mChannelConfig.flags.bit.m) continue;
        adcm_chbuf_t *mcb = &ev->channel[mCh];
        if (! mcb->flags.bit.eval_tfast) continue;

        for (auto sCh : ev->channel.keys()) {
            //  	  int sc_match = 0;
            const AdcChannelConfig &sChannelConfig = adc->channels[sCh];
            if (!sChannelConfig.flags.bit.s) continue;
            adcm_chbuf_t *scb = &ev->channel[sCh];
            if (scb->conv.peaks.empty() || mcb->conv.peaks.empty())
                continue;
//            if (!(scb->flags & ADCM_CH_EVAL_TFAST)) continue;

            double dt =  scb->conv.peaks[0].pt*adc->step - mcb->conv.peaks[0].pt*adc->step;
            int cond = dtMin <= dt && dt <= dtMax;
            //		printf("m:%2d, s:%2d  dt=%f  -- %s\n", mc, sc, dt, cond?"MATCH":"discard");
            if (cond) {
                pulseFound = true;
                chUsed[mCh] = true;
                chUsed[sCh] = true;
            }
        }
    }
    for (auto c : ev->channel.keys()) {
        adcm_chbuf_t *cb = &ev->channel[c];
        if (cb->flags.bit.eval_tfast && !chUsed[c])
            cb->flags.bit.dt_cut = 1;
    }

    //  for(i=0;i<nch;i++)
    //	printf("ch_use[%2d]: %d\n", i, ch_use[i]);
    //  for(i=0;i<nch;i++)
    //	if (ev->ch[i].flags & ADCM_CH_DT_CUT)
    //	  printf("[%2d]: discarded by dT cut\n", i);
    return pulseFound;
}
