//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef DSPLIB_FFTENGINE_H
#define DSPLIB_FFTENGINE_H

#include <map>
#include <vector>
#include <complex> /* complex must be included before fftw3.h */
#ifdef USE_FFTW
#include <fftw3.h> // NOLINT
#endif


class RFFTEngine {
public:
    virtual ~RFFTEngine();
    std::vector<std::complex<double> > DFT(const std::vector<double> &d); // real to complex
    std::vector<double> IDFT(const std::vector<std::complex<double> > &d); // complex to real
    //        static std::vector<FFTSpectrumPeak> getSpectrumPeaks(const FFTSpectrum &, double threshold = 0.1);

protected:
    size_t getCacheSizeLimit() const;
#ifdef USE_FFTW
    fftw_plan get_r2c(size_t N);
    fftw_plan get_c2r(size_t N);

    std::map<size_t, fftw_plan> r2cCache, c2rCache;
#endif
};

#endif /* DSPLIB_FFTENGINE_H */
