
#include <fcntl.h> // FIXME: O_RDONLY, open()

#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>
#include <vector>

#include "fir.h"
#include "dsplib.h"
#include "fftengine.h"

#include <QDebug>

//static const double FIR_MAX_AMPLIF = 10.;
static const double eps = 1e-100;

//void print_fir (const intfir_t * fir)
//{
//    assert(fir);
//    printf ("FIR: %d bits, %d taps, offset %d: {\n", fir->bits, fir->coef.size(), fir->offs);
//    for (int i = 0; i < fir->coef.size(); i++) {
//        printf (" %d", fir->coef[i]);
//        if (i <= fir->coef.size())
//            printf (",");
//    }
//    printf ("\n}\n");

//}

//void print_dblfir (const dblfir_t * fir)
//{
//    assert(fir);
//    printf ("FIR: %d taps, offset %d: {\n", fir->klen, fir->offs);
//    for (unsigned int i = 0; i < fir->klen; i++) {
//        printf (" %e", fir->coef[i]);
//        if ((i + 1) < fir->klen)
//            printf (",");
//        if (i%5==4) printf("\n");
//    }
//    printf ("\n}\n");

//}

int create_deconvolution_dblFIR (fir_t<decoder_fir_coef_t> *fir, const std::vector<double> &dsrc, const std::vector<double> &dreq)
{
    // Return codes:
    //  -1 - fatal error
    //   0 - success
    //   1,2,... - warning, bad result may occur

    int ret = 0;
    assert(fir);

    // pad with zeroes to make lengths equal
    std::vector<double> src(dsrc);
    std::vector<double> req(dreq);
    auto max_size = std::max(src.size(), req.size());
    src.resize(max_size);
    req.resize(max_size);

    const size_t minCommonSize = std::min(src.size(), req.size());
    const size_t inlen = minCommonSize;
    assert(inlen >= 2);

    assert(fir);
    assert(inlen <= 10000);

    if (!fir || (inlen > 10000)) {
//        logger.err("Bad input");
        return (-1);
    }


    const unsigned int deconvkernlen = inlen;
    const unsigned int deconvkernhalflen = deconvkernlen / 2 + 1;

    std::vector<double> dout(deconvkernlen);
    std::vector<double> powsrc(deconvkernhalflen);
    std::vector<double> powreq(deconvkernhalflen);
    std::vector<double> powout(deconvkernhalflen);
    std::vector<double> sinsrc(deconvkernhalflen);
    std::vector<double> cossrc(deconvkernhalflen);
    std::vector<double> sinreq(deconvkernhalflen);
    std::vector<double> cosreq(deconvkernhalflen);
    std::vector<double> sinout(deconvkernhalflen);
    std::vector<double> cosout(deconvkernhalflen);

    // Calculate signal DC component
    // const double dcsrc = dcdbl (src);
    const double dcreq = dcdbl (req);
    // Calculate signal RMS power
    // const double rmssrc = rmsdbl (src, dcsrc);
    const double rmsreq = rmsdbl (req, dcreq);
    if (rmsreq < eps) {
        return -1;
    }

    RFFTEngine fft;
    const auto &src_dft = fft.DFT(src);
    const auto &req_dft = fft.DFT(req);
    auto imp_dft(src_dft);
    const double N = src.size();
    for (size_t i = 0; i < imp_dft.size(); i++)
        imp_dft[i] = req_dft[i] / src_dft[i] / N;
    const auto &imp = fft.IDFT(imp_dft);

    fir->coef.clear();
    for (unsigned int i = 0; i < deconvkernlen; i++) {
        fir->coef.push_back(imp[(i + deconvkernhalflen) % deconvkernlen]);
    }

    fir->offs = deconvkernhalflen;

    return ret;
}

#if 0
void fir_convolve_vector (const intfir_t &fir, const std::vector<short int> &in, std::vector<int> *out)
{
    if (!out)
        return;
    out->clear();
    if (in.empty())
        return;
    if (!fir.isValid() || fir.isEmpty()) {
        out->resize(in.size());
        const int32_t scaler = ((1 << fir.bits) - 1);
        for (size_t i = 0; i < in.size(); i++)
            (*out)[i] = in[i] * scaler;
        return;
    }
    const size_t out_size = in.size() + fir.coef.size() - 1;
    out->resize(out_size);
    convolve_i_s_i (&fir.coef[0], fir.coef.size(), &in[0], in.size(), &(*out)[0]);
}

void fir_convolve_s (const intfir_t * fir, const short int *in, unsigned int inlen, int *out, unsigned int outlen)
{
    assert(fir);
    assert(in);
    assert(out);
    assert(inlen > 0);
    assert(outlen >= inlen);
    if (!in || !out || (inlen <= 0) || (outlen < inlen)) {
        assert(false);
        // logger.err("fir_convolve(): Bad input parameters");
        return;
    }
    if (fir->isValid() && !fir->isEmpty()) {
        unsigned int workin, workout;
        workin = inlen;
        workout = inlen + fir->coef.size() - 1;
        if (workin > outlen) {
            // logger.err(QString("workin=%1 > outlen=%2").arg(workin).arg(outlen));
            workin = outlen - fir->coef.size();
        }
        if (workout > outlen) {
            // logger.err(QString("workout=%1 > outlen=%2").arg(workout).arg(outlen));
            workout = outlen;
        }
        if ((workin <= 0) || (workout <= 0)) {
            assert(false);
            // logger.err("fir_convolve(): Bad input parameters");
            return;
        }
        /*
                printf("FIR[%d]: ", fir->klen);
                for (int i=0;i<fir->klen;i++) printf("%d, ", fir->coef[i]);
                printf("\n");
                printf("INP[%d]: ", workin);
                for (int i=0;i<workin;i++) printf("%d, ", fir->coef[i]);
                printf("\n");
*/
        if (! fir.isEmpty())
            convolve_i_s_i (&fir->coef[0], fir->coef.size(), in, workin, out);
        else {
            //      log_pri(LOG_CRIT, "NULL FIR - copying data");
            for (unsigned int i = 0; i < workin; i++)
                out[i] = in[i] * ((1 << fir->bits) - 1);
        }
        /*
                printf("OUT[%d]: ", workout);
                for (int i=0;i<workout;i++) printf("%d, ", out[i]);
                printf("\n");
*/
    } else {
        // logger.err("fir_convolve(): Invalid FIR - copying data");
        for (unsigned int i = 0; i < inlen; i++)
            out[i] = in[i] * ((1 << fir->bits) - 1);
        for (unsigned int i = inlen; i < outlen; i++)
            out[i] = 0;
    }
}

/*
// same, with data type: double
void fir_convolve_d (const intfir_t * fir, const double *in, unsigned int inlen, double *out, unsigned int outlen)
{
    assert(fir);
    assert(in);
    assert(out);
    assert(inlen > 0);
    assert(outlen >= inlen);
  if (!in || !out || (inlen <= 0) || (outlen < inlen)) {
    logger.err("fir_convolve(): Bad input parameters");
    return;
  }
  if (fir_valid (fir)) {
    unsigned int workin, workout;
    workin = inlen;
    workout = inlen + fir->klen;
    if (workin > outlen) {
      logger.err(QString("workin=%1 > outlen=%2").arg(workin).arg(outlen));
      workin = outlen - fir->klen;
    }
    if (workout > outlen) {
      logger.err(QString("workout=%1 > outlen=%2").arg(workout).arg(outlen));
      workout = outlen;
    }
    if ((workin <= 0) || (workout <= 0)) {
      logger.err("fir_convolve(): Bad input parameters");
      return;
    }
    if (fir->klen > 0)
      convolve_id (fir->coef, fir->klen, in, workin, out);
    else {
//      log_pri(LOG_CRIT, "NULL FIR - copying data");
      for (unsigned int i = 0; i < workin; i++)
        out[i] = in[i] * ((1 << fir->bits) - 1);
    }
  } else {
    logger.err("fir_convolve_d(): Invalid FIR - copying data");
    for (unsigned int i = 0; i < inlen; i++)
      out[i] = in[i] * ((1 << fir->bits) - 1);
    for (unsigned int i = inlen; i < outlen; i++)
      out[i] = 0;
  }
}
*/
#endif
