#ifndef _FIR_H
#define _FIR_H

#include <cassert>
#include <cstddef>
#include <vector>
#include "global_types.h"

template <class T>
class fir_t
{
public:
    bool operator==(const fir_t &other) const {
        return true
               && offs == other.offs
               && bits == other.bits
               && coef == other.coef
            ;
    }
    bool operator!=(const fir_t &other) const {return !operator==(other);}
//    bool isValid() const {
//        if (coef.empty())
//            return false; // empty is not a valid FIR
//        if (offs < 0 || offs >= static_cast<int>(coef.size())) {
//            return false;
//        }
//        if ((bits < 1) || (bits > 32)) {
//            return false;
//        }
//        return true;

//    };
    bool isEmpty() const {
        return coef.empty();
    }

    //    friend QTextStream &operator<<(QTextStream &s, const fir_t &channel);

    int offs = 0;
    unsigned int bits = 1;
    std::vector<T> coef;
};

typedef fir_t<int> intfir_t;
// print fir contents
//void print_fir (const intfir_t * fir);
//void print_dblfir (const dblfir_t * fir);

// Validator for intfir_t struct, returns 1 when ok, 0 if bad
//int fir_valid (const intfir_t * fir);

//void deletefir(intfir_t *fir);

// Deconvolution filter generation
int create_deconvolution_dblFIR (fir_t<decoder_fir_coef_t> * engine, const std::vector<double> &imp_orig, const std::vector<double> &imp_req);

// Do convolution with range checking
//  void fir_convolve (const intfir_t * fir, const int *in, unsigned int inlen, int *out, unsigned int outlen);

//void fir_convolve_vector (const intfir_t &fir, const std::vector<short> &in, std::vector<int> *out);
//void fir_convolve_vector (const intfir_t &fir, const std::vector<double> &in, std::vector<double> *out);

// in - short int, out - int
//void fir_convolve_s (const intfir_t * fir, const short int *in, unsigned int inlen, int *out, unsigned int outlen);
//  void fir_convolve_d (const intfir_t * fir, const double *in, unsigned int inlen, double *out, unsigned int outlen);

template <class U, class V, class W>
void convolve(const std::vector<U> &u, const std::vector<V> &v, std::vector<W> *out)
{
    assert(out);
    if (!out)
        return;
    out->clear();
    size_t u_size = u.size();
    size_t v_size = v.size();
    if (u_size == 0 || v.size() == 0)
        return;
    size_t out_size = u_size + v_size - 1;
    out->resize(out_size);
    for (size_t i = 0; i < u_size; i++)
        for (size_t j = 0; j < v_size; j++)
        {
            (*out)[i + j] += u[i] * v[j];
        }
}

#endif                          /* _FIR_H */
