#ifndef _DECODER_H
#define _DECODER_H 1

#include <vector>

#include "../include/adcm16.h"

#ifdef USE_FFT
#include "rfftw.h"
#endif

#include "../dsplib/lmmin.h"

typedef struct {
    int niter = 0;
    double fnorm = 0;
    //  double err[3] = {};
} fitstatus_t;

enum {FIT_ALGO_ACCURATE=0, FIT_ALGO_COARSE=1};

enum { MAX_FITGAUS_POINTS = 21 };
enum { MAX_FITSINE_POINTS = ADCM_MAX_N_SAMP };

//    struct lm_ws_t;

class FitterBase {
public:
    FitterBase() = delete;
    FitterBase(const FitterBase &) = delete;
    explicit FitterBase (unsigned int fit_points);
    FitterBase &operator=(const FitterBase &) = delete;
    ~FitterBase ();
    unsigned int getMaxFitPoints () const;
protected:
    struct lm_ws_t *ws;
};

class FitterGaus : public FitterBase {
public:
    explicit FitterGaus (unsigned int fit_points);
    int fit (int fixpar, const double *x, int len, double *par, fitstatus_t *fs); // not reentrant
};

#ifdef FIT_SINE

class FitterSine : public FitterBase {
public:
    explicit FitterSine (unsigned int fit_points);
    int fit (int fixpar, const double *x, int len, double *par0, double *par, fitstatus_t *fs);
protected:
#ifdef USE_FFT
    fftw_plan fftw_p;
    rfftw_plan fftwr_p;
    int fftw_p_size;
#endif
};
#endif

// Coarse approximation of gauss curve (return chi2)
void FindGausApprox (const int *x, int len, double *par);
double FindGausDevNorm (const int *x, int len, double *par);
void FindGausApproxD (const double *x, int len, double *par);
double FindGausDevNormD (const double *x, int len, double *par);

int QuickFitGaus (const double *x, int len, double *par, fitstatus_t *fs);

// New interface

class GausPar {
public:
    GausPar(double p0 = 0, double p1 = 0, double p2 = 0) : a(p0), x0(p1), sigma(p2) { }
    explicit GausPar(const double p[3]) { set(p); }
    void get(double p[3]) const { p[0] = a; p[1] = x0; p[2] = sigma; }
    void set(const double p[3]) { a = p[0]; x0 = p[1]; sigma = p[2]; }
    bool operator==(const GausPar &r) const { return a==r.a && x0==r.x0 && sigma== r.sigma; }
    double a = 0;
    double x0 = 0;
    double sigma = 0;
};

double GausFcn(const GausPar &par, double x);

class GaussianApproximator {
public:
    // Coarse approximation of gauss curve (return chi2)
    static GausPar FindGausApprox (const std::vector<int> &x);
    static GausPar FindGausApprox (const std::vector<double> &x);
    static GausPar FindGausApproxShifted (const std::vector<int> &x);
    static GausPar FindGausApproxShifted (const std::vector<double> &x);
    static double FindGausDevNorm (const std::vector<int> &x, const GausPar &par);
    static double FindGausDevNorm (const std::vector<double> &x, const GausPar &par);
    static double FindGausDevNormShifted (const std::vector<int> &x, const GausPar &par);
    static double FindGausDevNormShifted (const std::vector<double> &x, const GausPar &par);
};



#endif                          /* _DECODER_H */
