/*
**    Copyright 2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "HistMerger.h"

#include <QTimer>

#include "histmanager.h"
#include "dsplib/th1.h"

HistMerger::HistMerger(QObject *parent)
    : QObject(parent),
    histList(new HistList),
    sendTimer(new QTimer(this))
{
    connect(sendTimer, &QTimer::timeout, this, &HistMerger::sendHistograms);
    sendTimer->setSingleShot(true);
    sendTimer->setInterval(1000);
    sendTimer->start();
}

void HistMerger::addHistograms(const HistList &r)
{
    for (auto i : r.keys()) {
        const auto &rhist = r[i];
        if (!histList->contains(i))
            (*histList)[i] = rhist;
        else
            (*histList)[i] += rhist;
    }
}

void HistMerger::sendHistograms()
{
    HistList list;
    for (const hist_index_t & i : histList->keys()) {
        const int seq = (*histList)[i].get_modseq();
        if (seq != modseq[i]) {
            modseq[i] = seq;
            list[i] = (*histList)[i];
        }
    }
    if (!list.isEmpty())
        emit histogramsChanged(list);
    sendTimer->start();
}

void HistMerger::clearAllHists()
{
    for (auto & i : *histList)
        i.clear();
    sendHistograms();
}

void HistMerger::clearHist(const hist_index_t &index)
{
    (*histList)[index].clear();
    sendHistograms();
}
