/*
**    Copyright 2006-2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef HISTMANAGER_H
#define HISTMANAGER_H

#include <QMap>
#include <QMetaType>
#include <QSharedPointer>
#include <QString>

#include "dsplib/th1.h"
#include "hist_manager/hist_manager_types.h"

class conf_mem_t;
class QTextStream;

typedef QList<hist_type_t> HistTypeList;

class HistList: public QMap<hist_index_t, th1>
{
public:
    HistTypeList histTypes() const;
    friend QTextStream &operator<<(QTextStream &s, const HistList &r);
};

Q_DECLARE_METATYPE(HistList)

class QTimer;
class AdcmEvent;
class conf_mem_t;
class adcm_decstat_t;

class HistManager: public QObject
{
    Q_OBJECT
public:
    explicit HistManager(QObject *parent);
    void fill(const AdcmEvent &ev);
    int fillEvDt (const AdcmEvent &ev, quint64 prevEvTs);
    void fill_status(const adcm_decstat_t &st);
    // HistList getHistList () const {return hl;}

Q_SIGNALS:
    void histListModified(const HistList &);
    void histogramsChanged(const HistList &);

public Q_SLOTS:
    void setAdcConfMem(const conf_mem_t &value);
    void clearAllHists();
    void clearHist(const hist_index_t &index);
    void sendHistograms();

protected Q_SLOTS:
    void recreate_histograms();

private:
    int nch() const;
    void addHist(hist_type_t type, const th1 &h);
    void addHist(hist_type_t type, int channel, const th1 &h);

    conf_mem_t * const conf_mem;
    QTimer * const sendHistTimer;
    HistList hl;
};

#endif /* HISTMANAGER_H */
