//
//    Copyright 2010-2019 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <qwt_plot_marker.h>
#include <qwt_scale_engine.h>

#include <QFileDialog>
#include <QDebug>
#include <QTextStream>
#include <QSettings>

#include "ui_WaveformViewAdvancedUi.h"
#include "WaveformViewAdvanced.h"
#include "qxw/WaveformPlot.h"
#include "qlog/qlog.h"

WaveformViewAdvanced::WaveformViewAdvanced(QWidget *parent)
    : QWidget(parent),
      ui(new Ui::WaveformViewAdvancedUi),
      currentTimeMarker(new QwtPlotMarker())
{
    ui->setupUi(this);
    //    currentTimeMarker->setLineStyle(QwtPlotMarker::VLine);
    //    currentTimeMarker->setLinePen(QPen(QColor(Qt::green)));
    //    currentTimeMarker->setTitle("Current Time");
    connect(ui->checkBox_FFT, &QCheckBox::toggled,
            this, &WaveformViewAdvanced::FFTChanged);

    connect(ui->checkBoxLogX, &QCheckBox::toggled,
            this, &WaveformViewAdvanced::logXChanged);

    connect(ui->checkBoxLogX, &QCheckBox::toggled,
            this, &WaveformViewAdvanced::updateControls);

    connect(ui->checkBox_Filter, &QCheckBox::toggled,
            this, &WaveformViewAdvanced::filteringChanged);

    connect(ui->checkBox_Filter, &QCheckBox::toggled,
            this, &WaveformViewAdvanced::updateControls);

    connect(ui->checkBox_Averaging, &QCheckBox::toggled,
            this, &WaveformViewAdvanced::averagingChanged);

    connect(ui->checkBox_OffsetComp, &QCheckBox::toggled,
            this, &WaveformViewAdvanced::offsetCompChanged);

    connect(ui->pushButtonClear, &QPushButton::clicked,
            this, &WaveformViewAdvanced::actionClear);

    connect(this, &WaveformViewAdvanced::actionClear,
            this, &WaveformViewAdvanced::clear);

    connect(ui->pushButtonExport, &QPushButton::clicked,
            this, &WaveformViewAdvanced::actionExport);

    ui->checkBox_FFT->setChecked(false);
    updateControls();
}

WaveformViewAdvanced::~WaveformViewAdvanced()
{
    delete currentTimeMarker;
    delete ui;
}

void WaveformViewAdvanced::setAutoscale( bool b )
{
    setup.autoscale = b;
}

void WaveformViewAdvanced::setBipolarDisplay( bool b )
{
    setup.bipolarDisplay = b;
}

void WaveformViewAdvanced::updateControls()
{
    ui->checkBoxLogX->setEnabled(ui->checkBox_FFT->isChecked());
    ui->checkBox_OffsetComp->setEnabled(! ui->checkBox_Filter->isChecked());
}

//void WaveformViewAdvanced::update()
//{
//    updateControls();
////    if (!d->model)
////        return;

//    if (d->enableAverager) {
//        d->averager.add(getData());
//    }
//    const WaveformDataAll &wda = getPlotData();
//    ui->waveformPlot->setAllData(wda);
//    QString yTitle = QString("Amplitude, %1").arg(d->enableFFT ? tr("dBFS") : tr("bins"));
//    QwtText yText(yTitle);
//    yText.setFont(font());
//    ui->waveformPlot->setAxisTitle(QwtPlot::yLeft, yText);
//    if (setup.logx)
//        ui->waveformPlot->setAxisScaleEngine(QwtPlot::xBottom, new QwtLogScaleEngine);
//    else
//        ui->waveformPlot->setAxisScaleEngine(QwtPlot::xBottom, new QwtLinearScaleEngine);
//            /*
//            //  wp->setAxisScaleEngine(QwtPlot::yLeft, new QwtLog10ScaleEngine);
//            //  wp->axisScaleEngine(QwtPlot::yLeft)->setAttribute(QwtScaleEngine::Floating);
//            //  wp->setAxisOptions (QwtPlot::yLeft, QwtAutoScale::Floating | QwtAutoScale::Logarithmic);
//            //  wp->setAxisScale (QwtPlot::yLeft, INPFFT_MIN, INPFFT_MAX);
//            //  wp->setxrange (0, H);
//        */

//    ui->waveformPlot->replot();
//}

void WaveformViewAdvanced::clear()
{
    wda.clear();
    currentTimeMarker->hide();
    ui->waveformPlot->clear();
    ui->waveformPlot->replot();
}

void WaveformViewAdvanced::on_checkBoxLogX_toggled(bool state)
{
    setup.logx = state;
    update();
}

WaveformPlot *WaveformViewAdvanced::getPlot()
{
    return ui->waveformPlot;
}

void WaveformViewAdvanced::setWaveformData( const WaveformDataAll &w )
{
    wda = w;
    getPlot()->setAllData(w);
    if (setup.logx)
        ui->waveformPlot->setAxisScaleEngine(QwtPlot::xBottom, new QwtLogScaleEngine);
    else
        ui->waveformPlot->setAxisScaleEngine(QwtPlot::xBottom, new QwtLinearScaleEngine);
    ui->waveformPlot->replot();
}

void WaveformViewAdvanced::setCurrentTimeMarker(double)
{
    //    currentTimeMarker->setXValue(v);
    //    if (! currentTimeMarker->plot())
    //        currentTimeMarker->attach(getPlot());
}

void WaveformViewAdvanced::setXMarkers(const std::vector<double> &v)
{
    getPlot()->setXMarkers(v);
}
