//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//


#ifndef QXW_QXW_RANDGEN_H
#define QXW_QXW_RANDGEN_H

namespace qxw {

// An unnormalized gaussian function with mean=0 and sigma
// Divide result by sqrt(2*Pi) = 2.50662827463100024 if normalized output needed
double gauss (double x, double sigma);

//  Machine independent random number generator.
//  Produces uniformly-distributed floating points in [0,1]
//  Identical sequence on all machines of >= 32 bits.
//  Periodicity > 10**14
double rndm ();

// returns a uniform deviate on the interval ]x1, x2].
double ranuni (double x1, double x2);

//      Return a number distributed following a gaussian with sigma
double rangaus (double sigma);

// Simulate detector pulse
// amplitude a, time t, risetime rt, falltime ft
double detpulse (double a, double t, double rt, double ft);
}

#endif                          /* QXW_QXW_RANDGEN_H */
