#include "Analizer.h"
#include "RingBuffer.h"
#include "lib-common/MpdRawTypes.h"

#include <QDebug>
#include <QElapsedTimer>
#include <QTimer>

namespace  {
const int LOOP_SIZE = 1000;
}
Analizer::Analizer(int _index, RingBuffer<char> *dataRB, RingBuffer<EventMetaInfo> *evRB):
    QObject(0),
    analizedSize(0),

    byteRB(dataRB),
    eventRB(evRB),
    lastMetaInfo(nullptr),
//    newRawDataWC(_newDataWC),
    enabled(false),
    id(_index),
    runElapsedTimer(new QElapsedTimer()),
    runTimer(new QTimer(this)),
    stat(id)
{
    connect(runTimer, &QTimer::timeout, this, &Analizer::run);
    runTimer->setInterval(0);
    runTimer->setSingleShot(true);
}

Analizer::~Analizer()
{
    delete runElapsedTimer;
}

void Analizer::run()
{
    bool badSync=false;
    runElapsedTimer->start();
    while(enabled && timeLeftForRun()>0){
        if(!cleanUpFreedSpace())
            continue;
        Q_ASSERT(eventRB->waitFreeSpace(1,0)); // Just to enshure that there is enough free space
//        bool ret = eventRB->waitFreeSpace(1,0);
//        Q_ASSERT(ret);
//        if(eventRB->getWriterPos()==0 && eventRB->getReaderPos(0)!=0)
//            sendLog(QString("::analize() metaInfoBuff->getWriterPos()==0"));
        EventMetaInfo *i = eventRB->getWritePointer();
//        int pos = eventRB->getWriterPos();
        Q_ASSERT(i->free);

        // Ready to read new raw data

//        int dataSize = byteRB->getDataSize();
//        bool timeout=false;
//        while(dataSize < (int)(analizedSize+sizeof(MpdEventHdr))){
//            if(!newRawDataWC->wait(wcMutex, timeLeft())){
//                timeout=true;
//                break;// todo decrease this timeout
//            }
//            dataSize = byteRB->getDataSize();
//            sendLog(QString("::analize() timeout on MpdEventHdr"));
//            continue;
//        }
//        if(timeout)
//            continue;
        if(byteRB->getDataSize() < analizedSize+sizeof(MpdEventHdr)) {
            if(!byteRB->waitDataSpace(analizedSize+sizeof(MpdEventHdr), timeLeftForRun()))
                break;
        }

        i->reset();
        byteRB->readData(reinterpret_cast<char*>(&mpdHdr),
                         analizedSize, sizeof(MpdEventHdr));
        if(mpdHdr.sync == MPD_SYNC_NORM){
            i->type = MPD_SYNC_NORM;
        } else if(mpdHdr.sync == MPD_SYNC_EOB) {
            i->type = MPD_SYNC_EOB;
        } else if(mpdHdr.sync == MPD_SYNC_RUN_START || mpdHdr.sync == MPD_SYNC_RUN_STOP) {
            int fullSize = mpdHdr.length+sizeof(MpdHdr);
            if(byteRB->getDataSize() < analizedSize+fullSize) {
                if(!byteRB->waitDataSpace(analizedSize+fullSize, timeLeftForRun()))
                    break;
            }

            if(lastMetaInfo == nullptr) {
                const int part1 = qMin(fullSize, byteRB->getContinuosTrashSize());
                byteRB->trashData(part1);
                byteRB->freeData(part1);
                if(fullSize != part1){
                    const int part2 = fullSize-part1;
                    byteRB->trashData(part2);
                    byteRB->freeData(part2);
                }
            } else {
                lastMetaInfo->skipData += fullSize;
                analizedSize += fullSize;
            }
            stat.totalSize += fullSize;
            continue;
        } else {
            if(!badSync) {
                ++stat.badSync;
                qWarning() << clientDescr << QString("::analize() bad sync; analizedSize=%1")
                              .arg(analizedSize);
            }
            badSync = true;
            // todo search in local loop
            if(lastMetaInfo == nullptr) {
                byteRB->trashData(1);
                byteRB->freeData(1);
            } else {
                ++(lastMetaInfo->corrupted);
                ++analizedSize;
            }
            ++stat.corruptedSize;
            ++stat.totalSize;
            continue;
        }
        if(badSync)
            badSync = false;

        //MPD header is valid wait for full event
        const int fullSize = sizeof(MpdEventHdr)+mpdHdr.length;
        if(byteRB->getDataSize() < analizedSize+fullSize) {
            if(!byteRB->waitDataSpace(analizedSize+fullSize, timeLeftForRun()))
                break;
        }
//        while(dataSize < analizedSize+fullSize){
//            if(!newRawDataWC->wait(wcMutex, timeLeft())){
//                timeout=true;
//                break;// todo decrease this timeout
//            }
//            dataSize = byteRB->getDataSize();
//        }
//        if(timeout){
////            log(QString("::analize() timeout on MpdData Payload"));
//            continue;
//        }

//        if(prevEvNum+1!= mpdHdr.evNum)
//            qInfo()<<"Analizer::run() prevEvNum="<<prevEvNum<<" mpdHdr.evNum="<<mpdHdr.evNum;
//        prevEvNum = mpdHdr.evNum;
        i->evNum = mpdHdr.evNum;
        i->data1 = byteRB->getReadPointer(analizedSize);
        i->dataPos1 = byteRB->getReaderPos(analizedSize);
        i->size1 = qMin(fullSize, byteRB->getContinuosDataSize(analizedSize));
        i->size2 = fullSize-i->size1;
        if(i->size2) {
            i->data2 = byteRB->getReadPointer(analizedSize+i->size1);
        }
        i->free = false;
        lastMetaInfo = i;
//        if(metaInfoBuff->writePos==0)
//            qDebug()<<"first pos evNum="<<mpdHdr.evNum;
        i->evNum = mpdHdr.evNum;
        stat.addEvent(fullSize);
        stat.totalSize += fullSize;
        if(stat.isNextPeriod(i->createTime)){
            emit statUpdated(stat);
            stat.newPeriod();
        }
        analizedSize += fullSize;
//        log(QString("::analize() added event pos=%1 evNum=%2")
//            .arg(eventRB->getWriterPos()).arg(mpdHdr.evNum));

        eventRB->writeData();
    }

    runTimer->start();
}

void Analizer::startAnalize(uint stateIndex)
{
    if(enabled){
        qWarning() << clientDescr << "analizer double start";
        return;
    }
    enabled = true;
    emit updateRunState(true, true, stateIndex);
    runTimer->start();
}

void Analizer::stopAnalize(uint stateIndex)
{
    if(!enabled){
        qWarning()<< clientDescr <<"analizer double stop";
        return;
    }
    enabled = false;
    runTimer->stop();
    emit updateRunState(true, false, stateIndex);
}

void Analizer::resetAnalize(uint stateIndex)
{
    qDebug()<<"Analizer index #"<<id<< clientDescr <<" start resetting";
    analizedSize=0;

#ifdef RESET_ROUGH
//    qInfo()<<"RESET_ROUGH"<<id<<eventRB->getStrStatus();
    int val = eventRB->getContinuosDataSize(0);
    eventRB->trashData(val);
    val = eventRB->getContinuosDataSize(0);
    if(val)
        eventRB->trashData(val);
    Q_ASSERT(eventRB->getDataSize()==0);
    while(eventRB->getTrashSize()){
        EventMetaInfo *i = eventRB->getClearPointer(0);
        i->free = true;
        eventRB->freeData();
    }
    cleanUpFreedSpace(0);

    Q_ASSERT(eventRB->getFreeSize()==eventRB->getAllSize());
#else
    int pendingBytes = byteRB->getDataSize();
    byteRB->trashData(pendingBytes);
    while(eventRB->getFreeSize() != eventRB->getAllSize()){
        cleanUpFreedSpace(50);
    }
    pendingBytes = byteRB->getDataSize();
    byteRB->trashData(pendingBytes);
#endif


    lastMetaInfo=nullptr;
//    newRawDataWC;
//    wcMutex;
//    allEventsSem; //resetted in Combiner

    Q_ASSERT(!enabled);
//    currentEvent; // uses from Combiner
    stat.clear();
    qDebug()<<"Analizer index #"<<id<< clientDescr <<" resetting completed";
    emit resetted(true, stateIndex);
}

void Analizer::clientDescrChanged(const QString &descr)
{
    clientDescr = descr;
}

bool Analizer::cleanUpFreedSpace(int timeoutMs)
{
    bool hasFree = eventRB->getFreeSize();

    int toFree = eventRB->getTrashSize();
    if(hasFree) {
        if(toFree==0)
            return true;
    } else {
        if(toFree==0) {
            // minimize using of timeLeftForRun()
            if(timeoutMs==-1)
                timeoutMs = timeLeftForRun();

            if(!eventRB->waitTrashSpace(1, timeoutMs)) {
                // There were no free space and we could not wait for toFreeSpace
                return false;
            }
            toFree = eventRB->getTrashSize();
            Q_ASSERT(toFree>0);
        }
    }

    //We have at least 1 info to free
//    log(QString("::cleanUpFreedSpace() clean-up: %1 events").arg(toFree));
    for(int shift=0; shift<toFree; ++shift){
        EventMetaInfo *i = eventRB->getClearPointer(0);
//        log(QString("cleanUpFreedSpace event pos=%1 evNum=%2")
//            .arg(eventRB->getReaderPos()).arg(i->evNum));
        byteRB->trashData(i->size1, i->dataPos1);
        byteRB->freeData(i->size1, i->dataPos1);
        analizedSize -= i->size1;
        if(i->size2){
            byteRB->trashData(i->size2, 0);
            byteRB->freeData(i->size2, 0);
            analizedSize -= i->size2;
        }
        if(i->corrupted || i->skipData){
            const int skipTotal = i->corrupted + i->skipData;
            const int skip1 = qMin(skipTotal, byteRB->getContinuosDataSize(0));
            byteRB->trashData(skip1);
            byteRB->freeData(skip1);
            if(skipTotal != skip1){
                const int skip2 = skipTotal - skip1;
                byteRB->trashData(skip2);
                byteRB->freeData(skip2);
            }
            analizedSize -= skipTotal;
        }
        Q_ASSERT(analizedSize>=0);
        i->free = true;
        if(lastMetaInfo==i)
            lastMetaInfo = nullptr;
//        bool ret = metaInfoBuff->trashSpace.tryAcquire(1, 0);
        eventRB->freeData(1);
//        Q_ASSERT(ret);
    }
//    bool ret = metaInfoBuff->trashSpace.tryAcquire(toFree, 0);
//    metaInfoBuff->freeData(toFree);
//    Q_ASSERT(ret);
    return true;
}
