#ifndef ANALIZER_H
#define ANALIZER_H

#include <QObject>
#include <QVector>

#include "params.h"
#include "EventMetaInfo.h"
#include "lib-common/MpdRawTypes.h"

template<class T> class RingBuffer;
class QSemaphore;
class QElapsedTimer;
class QTimer;

struct AnalyzerStat {
    AnalyzerStat() : id(0) {}

    AnalyzerStat(int _id):id(_id) {
        clear();
    }

    void clear() {
        totalSize = 0;
        corruptedSize = 0;
        badSync = 0;
        evSize = 0;
        evCnt = 0;
        periodSize = 0;
        periodCnt = 0;
        timeMark.invalidate();
    }

    bool isNextPeriod(const QElapsedTimer &curT) const {
        return periodCnt>EV_SIZE_PERIOD_MIN_CNT &&
                ( !timeMark.isValid() || (timeMark.msecsTo(curT)>EV_SIZE_PERIOD_MS));
    }
    void newPeriod() {
        periodSize = 0;
        periodCnt = 0;
        timeMark.start();
    }

    void addEvent(quint32 size) {
        evSize += size;
        ++evCnt;
        periodSize += size;
        ++periodCnt;
    }

    int id;
    quint64 totalSize;
    quint64 corruptedSize;
    quint64 badSync;
    quint64 evSize;
    quint64 evCnt;
    quint64 periodSize;
    quint64 periodCnt;
    QElapsedTimer timeMark;
};
Q_DECLARE_METATYPE(AnalyzerStat)

class Analizer : public QObject
{
    Q_OBJECT
public:
    Analizer(int _index, RingBuffer<char> *dataRB, RingBuffer<EventMetaInfo> *evRB);
    virtual ~Analizer();
    int getId() const { return id; }
    RingBuffer<EventMetaInfo> *getEventRingBuff() const { return eventRB; }

public slots:
    void startAnalize(uint stateIndex);
    void stopAnalize(uint stateIndex);
    void resetAnalize(uint stateIndex);
    void clientDescrChanged(const QString &descr);

signals:
    void updateRunState(bool isAnalizer, bool isInRunMode, uint stateIndex) const;
    void resetted(bool isAnalizer, uint stateIndex) const;
    void statUpdated(const AnalyzerStat &s) const;

private slots:
    void run();

private:
    bool cleanUpFreedSpace(int timeoutMs=-1);
    inline qint64 timeLeftForRun() const { return qMax(100-runElapsedTimer->elapsed(),(qint64)0); }

    int analizedSize;
    MpdEventHdr mpdHdr;
    RingBuffer<char> *byteRB;
    RingBuffer<EventMetaInfo> *eventRB;
    EventMetaInfo *lastMetaInfo; // remember to trash corrupted data
    bool enabled;
    const int id;
    QElapsedTimer * const runElapsedTimer;
    QTimer * const runTimer;
//    quint32 prevEvNum;
    QString clientDescr;
    AnalyzerStat stat;
};

#endif // ANALIZER_H
