#ifndef FILEWRITER_H
#define FILEWRITER_H

#include <QObject>
#include <QFile>
#include <QMetaType>
#include <QElapsedTimer>
#include "RingBuffer.h"
#include "TcpServerStats.h"

class QTimer;

struct FileWriterStats {
    FileWriterStats() { clear(); }
    void clear(){
        writtenFileSize = 0;
        missedFileSize = 0;
        fileName.clear();
        predefStorage.clear();
    }

    quint64 writtenFileSize;
    quint64 missedFileSize;
    QString fileName;
    QString predefStorage;
    QElapsedTimer timeStamp;
};
Q_DECLARE_METATYPE(FileWriterStats)

class FileWriter : public QObject
{
    Q_OBJECT
public:
    explicit FileWriter(RingBuffer<char> * const writerRB, QObject *parent = nullptr);
    ~FileWriter();

signals:
    void statusUpdated(const FileWriterStats &) const;
    void fileOpened(QString runIndex, int runNumber) const;
    void error(int index, QString errStr); // index of file wrriter: -1

public slots:
    void clear();
    void storageChanged(QString dir);
    void openFileToWrite(QString runIndex, int runNumber, int writerIndex);
    void closeFile();

private slots:
    void writeData();
    void sendStatus();
    void sendMetrics();

private:
    void defineFileName(QString runIndex, int runNumber, int writerIndex);

    QString predefStorage;
    QFile dataFile;
    RingBuffer<char> * const ringBuf;
    FileWriterStats totalStats;
    TcpServerStats * const stats;
    QElapsedTimer writeLatencyElapsedTimer;
    QTimer * const writeTimer;
    QTimer * const statusTimer;
};

#endif // FILEWRITER_H
