#ifndef OUTPUTTCPSERVER_H
#define OUTPUTTCPSERVER_H

#include <QObject>
#include <QHostAddress>
#include <QVector>
#include <QElapsedTimer>
#include "RingBuffer.h"

class QTcpServer;
class QTcpSocket;
class QTimer;
class TcpServerStats;

struct OutputTcpStatus
{
    OutputTcpStatus(int i=0):
        index(i),
        listening(false),
        port(0),
        isFree(true),
        dataPart(0)
    {}

    int index;
    bool listening;
    quint16 port;
    bool isFree;
    double dataPart; // dataSize/allBufferSize
    QHostAddress peer_host;
    quint16 peer_port;
};
Q_DECLARE_METATYPE(OutputTcpStatus)

class OutputTcpServer : public QObject
{
    Q_OBJECT
public:
    explicit OutputTcpServer(int _index, quint16 port=0, QObject *parent = nullptr);
    ~OutputTcpServer();
    RingBuffer<char> *getRingBuffer() const { return ringBuf; }
    int getIndex() const { return index; }

public slots:
    void startListen(int i, bool listen=true);
    void deleteTcp(int i);
    void clear();
    void startWork();
    void finishWork(uint fid);
    void setMonitorPortNum(quint16 portNum);

private slots:
    void writeData();
    void updateConnections();
    void socketDisconnected();
    void sendStatus() const;
    void sendMetrics();

signals:
    void statusUpdated(int index, bool listening, int port, bool isFree, double dataPart) const;
    void statusUpdated(OutputTcpStatus) const;
    void workFinished(int index, uint fid) const;
    void error(int index, const QString &errStr);

private:
    QString getServerName() const;
    bool isFree() const { return tcpSocket==nullptr; }

    const int index;
    quint16 preferedPortNum;
    quint16 tcpPort;
    QTcpServer * const qTcpServer;
    TcpServerStats * const stats;
    QElapsedTimer writeLatencyElapsedTimer;
    QByteArray data;
    QTcpSocket *tcpSocket;
    QString peerHost;
    QTimer * const writeTimer;
    QTimer * const statusTimer;
    bool slowConnection;
    RingBuffer<char> * const ringBuf;
    uint finishId; // 0 - normal work, !0 - finishing state
    mutable OutputTcpStatus status;
    bool runEnabled;
};

#endif // OUTPUTTCPSERVER_H
