#include "StatisticDialog.h"
#include "ui_StatisticDialog.h"
#include <QTimer>

namespace {
enum COL_STAT_T { COL_STAT_T_NAME,
                  COL_STAT_T_FREE_PART,
                  COL_STAT_T_DATA_PART,
                  COL_STAT_T_TRASH_PART,
                  COL_STAT_T_RCV_BUFF_SIZE,
                  COL_STAT_T_READ_BUFFER_SIZE,
                  COL_STAT_T_AVAIL_SIZE,
                  COL_STAT_T_TOTAL};
enum COL_STAT_A { COL_STAT_A_ID,
                  COL_STAT_A_NAME,
                  COL_STAT_A_TOT_SIZE,
                  COL_STAT_A_CORR_SIZE,
                  COL_STAT_A_BAD_SYNC,
                  COL_STAT_A_EV_SIZE,
                  COL_STAT_A_EV_CNT,
                  COL_STAT_A_PERIOD_EV_SIZE,
                  COL_STAT_A_TOTAL};
enum COL_STAT_C { COL_STAT_C_ID,
                  COL_STAT_C_DATAPART,
                  COL_STAT_C_FULL_SIZE,
                  COL_STAT_C_PREF_SIZE,
                  COL_STAT_C_SPEED,
                  COL_STAT_C_TOTAL};
}

StatisticDialog::StatisticDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::StatisticDialog),
    refreshTimer(new QTimer(this))
{
    ui->setupUi(this);
    init();
    refreshTimer->setInterval(100);
    refreshTimer->setSingleShot(true);
    connect(refreshTimer, &QTimer::timeout, this, &StatisticDialog::updateStat);
}

StatisticDialog::~StatisticDialog()
{
    delete ui;
}

void StatisticDialog::setVisible(bool visible)
{
    QDialog::setVisible(visible);
    if(visible)
        updateStat();
}

void StatisticDialog::analyzerStatUpdated(const AnalyzerStat &s)
{
    analyzerStatMapPending[s.id] = s;
    if(!refreshTimer->isActive())
        refreshTimer->start();
}

void StatisticDialog::analizerListUpdated(QVector<Analizer *>, QMap<int, QString> clientNames)
{
    analyzerStatMap.clear();
    QTableWidget *table = ui->tableWidgetAnalyzer;

    while(table->rowCount())
        table->removeRow(0);

    for(int id : clientNames.keys()) {
        addAnalyzer(id, clientNames[id]);
    }
}

void StatisticDialog::init()
{
    {
    QTableWidget *table = ui->tableWidgetClient;
    table->setColumnCount(COL_STAT_T_TOTAL);
#if QT_VERSION >= 0x050000
    table->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
#else
    table->horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);
#endif

    table->setHorizontalHeaderItem(COL_STAT_T_NAME, new QTableWidgetItem("name"));
    table->setHorizontalHeaderItem(COL_STAT_T_DATA_PART, new QTableWidgetItem("Data %"));
    table->setHorizontalHeaderItem(COL_STAT_T_FREE_PART, new QTableWidgetItem("Free %"));
    table->setHorizontalHeaderItem(COL_STAT_T_TRASH_PART, new QTableWidgetItem("Trash %"));
    table->setHorizontalHeaderItem(COL_STAT_T_RCV_BUFF_SIZE, new QTableWidgetItem("Rcv buff size"));
    table->setHorizontalHeaderItem(COL_STAT_T_READ_BUFFER_SIZE, new QTableWidgetItem("Read buffer size"));
    table->setHorizontalHeaderItem(COL_STAT_T_AVAIL_SIZE, new QTableWidgetItem("Avail"));
    }
    {
    QTableWidget *table = ui->tableWidgetAnalyzer;
    table->setColumnCount(COL_STAT_A_TOTAL);
#if QT_VERSION >= 0x050000
    table->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
#else
    table->horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);
#endif

    table->setHorizontalHeaderItem(COL_STAT_A_ID, new QTableWidgetItem("id"));
    table->setHorizontalHeaderItem(COL_STAT_A_NAME, new QTableWidgetItem("name"));
    table->setHorizontalHeaderItem(COL_STAT_A_TOT_SIZE, new QTableWidgetItem("Tot. size"));
    table->setHorizontalHeaderItem(COL_STAT_A_CORR_SIZE, new QTableWidgetItem("Corrupted"));
    table->setHorizontalHeaderItem(COL_STAT_A_BAD_SYNC, new QTableWidgetItem("Bad sync"));
    table->setHorizontalHeaderItem(COL_STAT_A_EV_SIZE, new QTableWidgetItem("Ev size"));
    table->setHorizontalHeaderItem(COL_STAT_A_EV_CNT, new QTableWidgetItem("Ev cnt"));
    table->setHorizontalHeaderItem(COL_STAT_A_PERIOD_EV_SIZE, new QTableWidgetItem("Period ev. size"));
    }
    {
    QTableWidget *table = ui->tableWidgetCombiner;
    table->setColumnCount(COL_STAT_C_TOTAL);
    table->setRowCount(1);
#if QT_VERSION >= 0x050000
    table->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
#else
    table->horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);
#endif

    table->setHorizontalHeaderItem(COL_STAT_C_ID, new QTableWidgetItem("id"));
    table->setHorizontalHeaderItem(COL_STAT_C_DATAPART, new QTableWidgetItem("FW datapart"));
    table->setHorizontalHeaderItem(COL_STAT_C_FULL_SIZE, new QTableWidgetItem("FW full size, MB"));
    table->setHorizontalHeaderItem(COL_STAT_C_PREF_SIZE, new QTableWidgetItem("FW pref size, MB"));
    table->setHorizontalHeaderItem(COL_STAT_C_SPEED, new QTableWidgetItem("FW speed, MB/s"));
    for(int col=0;col<COL_STAT_C_TOTAL;++col)
        table->setItem(0,col,new QTableWidgetItem());
    }

}

void StatisticDialog::updateStat()
{
    if(isHidden())
        return;

    updateAnalStat();
    updateTcpStat();
}

void StatisticDialog::updateCombinerStats(const CombinerStats &_combStat)
{
    const qint64 dtMs = (combStat.timeStamp.isValid() && _combStat.timeStamp.isValid()) ? combStat.timeStamp.msecsTo(_combStat.timeStamp) : 0;
    qint64 fullSize = 0;
    for (int id : _combStat.flushedSize.keys()) {
        if(combStat.flushedSize.contains(id))
            fullSize = _combStat.flushedSize[id] - combStat.flushedSize[id];
    }
    const double speed = 1. * fullSize / dtMs *1e3;
    combStat = _combStat;
    QTableWidget *table = ui->tableWidgetCombiner;
    table->item(0, COL_STAT_C_DATAPART)->setText(QString::number(1e2*_combStat.fwDataPart,'f',1));
    table->item(0, COL_STAT_C_FULL_SIZE)->setText(QString::number(1e-6*_combStat.fwFullSize,'f',3));
    table->item(0, COL_STAT_C_PREF_SIZE)->setText(QString::number(1e-6*_combStat.fwPrefSize,'f',3));
    table->item(0, COL_STAT_C_SPEED)->setText(dtMs ? QString::number(1e-6*speed,'f',3) : QString("-"));
}

void StatisticDialog::updateTcpStatistic(int index, const TcpStatistic &statistic)
{
    tcpStatMapPending[index] = statistic;
    if(!refreshTimer->isActive())
        refreshTimer->start();
}

int StatisticDialog::addAnalyzer(int id, QString name)
{
    QTableWidget *table = ui->tableWidgetAnalyzer;
    Q_ASSERT(getAnalRowById(id)==-1);
    const int row = table->rowCount();
    table->setRowCount(row+1);

    for(int col=0; col< COL_STAT_A_TOTAL; ++col)
        table->setItem(row, col, new QTableWidgetItem(""));

    QTableWidgetItem *i = table->item(row, COL_STAT_A_ID);
    i->setText(QString::number(id));
    table->item(row, COL_STAT_A_ID)->setText(QString::number(id));
    table->item(row, COL_STAT_A_NAME)->setText(name);
    return row;
}

int StatisticDialog::addClient(const TcpStatistic &s)
{
    QTableWidget *table = ui->tableWidgetClient;
    Q_ASSERT(getClientRowById(s.index)==-1);
    int row = table->rowCount();
    table->setRowCount(row+1);


    for(int col=0; col< COL_STAT_T_TOTAL; ++col)
        table->setItem(row, col, new QTableWidgetItem(""));

    QTableWidgetItem *i = table->item(row, COL_STAT_T_NAME);
    i->setText(s.name);
    i->setData(Qt::UserRole, s.index);
    return row;
}

void StatisticDialog::updateTableSize(QTableWidget *table)
{
    if(table==nullptr)
        table = ui->tableWidgetAnalyzer;
    int height=table->horizontalHeader()->height();
    for(int i=0;i<table->rowCount();++i){
        height += table->rowHeight(i);
    }
    height += 5;
    table->setMinimumHeight(height);

    int width=table->verticalHeader()->width();
    for(int i=0;i<table->columnCount();++i){
        width += table->columnWidth(i);
    }
    width += 5;
    table->setMinimumWidth(width);
}

int StatisticDialog::getAnalRowById(int id) const
{
    QTableWidget *table = ui->tableWidgetAnalyzer;
    for(int row=0; row<table->rowCount(); ++row)
        if(table->item(row, COL_STAT_A_ID)->text().toInt() == id)
            return row;
    return -1;
}

int StatisticDialog::getClientRowById(int id) const
{
    QTableWidget *table = ui->tableWidgetClient;
    bool ok;
    for(int row=0; row<table->rowCount(); ++row)
        if(table->item(row, COL_STAT_T_NAME)->data(Qt::UserRole).toInt(&ok) == id && ok)
            return row;
    return -1;
}

void StatisticDialog::updateAnalStat()
{
    QTableWidget *table = ui->tableWidgetAnalyzer;
    table->setUpdatesEnabled(false);
    for (int id : analyzerStatMapPending.keys()){
        const AnalyzerStat s = analyzerStatMapPending.take(id);
        int row = getAnalRowById(s.id);
        if(row==-1)
            row = addAnalyzer(s.id, QString("Anal stat #%1").arg(s.id));
        Q_ASSERT(row!=-1);

        table->item(row, COL_STAT_A_TOT_SIZE)->setText(QString::number(s.totalSize));
        table->item(row, COL_STAT_A_CORR_SIZE)->setText(QString::number(s.corruptedSize));
        table->item(row, COL_STAT_A_BAD_SYNC)->setText(QString::number(s.badSync));
        table->item(row, COL_STAT_A_EV_SIZE)->setText(QString::number(s.evSize));
        table->item(row, COL_STAT_A_EV_CNT)->setText(QString::number(s.evCnt));
        table->item(row, COL_STAT_A_PERIOD_EV_SIZE)->setText(s.periodCnt ? QString::number(s.periodSize/s.periodCnt) : "-");
        analyzerStatMap[id] = s;
    }

    updateTableSize(table);
    table->setUpdatesEnabled(true);
}

void StatisticDialog::updateTcpStat()
{
    QTableWidget *table = ui->tableWidgetClient;
    table->setUpdatesEnabled(false);
    for (int id : tcpStatMapPending.keys()){
        TcpStatistic s;
        s = tcpStatMapPending.take(id);
        int row = getClientRowById(s.index);
        if(row==-1)
            row = addClient(s);

        table->item(row, COL_STAT_T_RCV_BUFF_SIZE)->setText(QString("%L1 B").arg(s.rcvBufSize));
        table->item(row, COL_STAT_T_READ_BUFFER_SIZE)->setText(QString("%L1 B").arg(s.readBufferSize));
        table->item(row, COL_STAT_T_AVAIL_SIZE)->setText(QString("%L1 B").arg(s.bytesAvailable));

        table->item(row, COL_STAT_T_FREE_PART)->setText(QString::number(s.freePart, 'f', 1));
        table->item(row, COL_STAT_T_DATA_PART)->setText(QString::number(s.dataPart, 'f', 1));
        table->item(row, COL_STAT_T_TRASH_PART)->setText(QString::number(s.trashPart, 'f', 1));

        tcpStatMap[id] = s;
    }

    updateTableSize(table);
    table->setUpdatesEnabled(true);
}
