#ifndef TCPCLIENT_H
#define TCPCLIENT_H

#include <QHostAddress>
template<class T> class RingBuffer;
class QTcpSocket;

class Analizer;
class AnalyzerStat;
class QHostInfo;
class QSemaphore;
class QTimer;
class ProgramDescription;
class ProgramInterface;
class DnsLookup;

struct TcpStatistic {
    int index;
    QString name;
    float freePart;
    float dataPart;
    float trashPart;
    qint64 rcvBufSize;
    qint64 readBufferSize;
    qint64 bytesAvailable;
};

Q_DECLARE_METATYPE(QHostAddress)
Q_DECLARE_METATYPE(TcpStatistic)

class TcpClient : public QObject
{
    Q_OBJECT
public:
    TcpClient(int i, RingBuffer<char> *byteRB);
    virtual ~TcpClient();

    QString getHostAddr() const { return hostAddr.toString(); }
    int getHostPort() const { return hostPort; }

public slots:
    void stopReadOut();
    void startReadOut();
    void startRun(uint stateIndex);
    void stopRun(uint stateIndex);
    void resetReadOut(uint stateIndex);
    void setAddress(const ProgramDescription &descr, const ProgramInterface &interface);
    void clientDescrChanged(const QString &descr);
    void analyzerStatUpdated(const AnalyzerStat &aStat);

private slots:
    void connectToHost();
    void readData();
    void displayError(QAbstractSocket::SocketError socketError);
    void connected();
    void disconnected();
    void doSendMetrics();
    void updateStatus();
    void gotHostInfo(const QHostInfo &hostInfo);
    void resizeBuffer();
    void low_space_timer_slot();

signals:
    void dataAcquired(int index, QByteArray);
    void sendStatus(int index, bool isConnected, QString state);
    void sendStatistic(int index, const TcpStatistic &statistic);
    void updateRunState(bool isAnalizer, bool isInRunMode, uint stateIndex);
    void gotError(QString errStr="");
    void resetted(bool isAnalizer, uint stateIndex);
    void inputBufferSizeUpdated(int index, quint64 size) const;

private:
    void disconnectFromHost();
    bool isConnectionAlive() const;
    QString getName() const;

    QString programIndex;
    const int index;
    QString clientDescr;
    bool readoutEnabled;
    bool idleReadout;
    QTcpSocket *tcpSocket;
    DnsLookup * const dnsLookup;
    QHostAddress hostAddr;
    QString hostName;
    quint32 hostPort;
    QTimer * const reconnectTimer;
    QTimer * const delayReadTimer;
    QTimer * const statusTimer;
    QTimer * const sendMetricsTimer;
    QTimer * const resizeTimer;
    QTimer * low_space_timer;
    int connectingAttempts;
    quint64 packageCounter;

    RingBuffer<char> *byteRB;
    bool noFreeSpace;
    TcpStatistic statistic;
    int loopRecivedSize;
    uint curStateIndex;
    bool srcBusy;

    bool waitingResize;
    int preferedBufSize;
};

#endif // TCPCLIENT_H
