#include "DeviceConfig.h"
#include "QDebug"

const QString CHANNELS_THR = "channels_thr";
const QString CHANNELS_BASELINE = "channels_baseline";

DeviceConfig::DeviceConfig()
{
    init_default_config();
}

void DeviceConfig::init_default_config()
{
    for(int i=0; i<N_CH; ++i)
        channel_thr_map[i] = 0;

    lut_path = "";
    enabled = true;
}

void DeviceConfig::root_to_values(RootConfig root)
{
    device_index.first = root.get_data<quint16>("device_id");
    device_index.second = root.get_data<quint64>("device_serial");

    lut_path = root.get_data<QString>("lut_path");
    enabled = root.get_data<bool>("enabled");

    for(QString child_name : root.children().keys()){
        if(child_name == CHANNELS_THR){
            RootConfig child = root.child(child_name);
            for(QString field : child.data().keys()){
                int channel = field.toInt();
                double thr = child.get_data<double>(field);
                channel_thr_map[channel] = thr;
            }
        }

        if(child_name == CHANNELS_BASELINE){
            RootConfig child = root.child(child_name);
            for(int ch=0; ch<N_CH; ++ch){
                double channel_baseline = child.get_data<double>(QString::number(ch));
                baseLine.append(channel_baseline);
            }
        }
    }
}

RootConfig DeviceConfig::values_to_root()
{
    RootConfig root;

    root.set_data("device_id", device_index.getDevId());
    root.set_data("device_serial", device_index.getSerial());
    root.set_data("serial", device_index.getSerialStr());

    root.set_data("lut_path", lut_path);
    root.set_data("enabled", enabled);

    RootConfig root_channels_thr = RootConfig(CHANNELS_THR);
    for(int i : channel_thr_map.keys()){
        root_channels_thr.set_data(QString::number(i), channel_thr_map[i]);
    }
    root.append_child(root_channels_thr);

    RootConfig root_channels_baseline = RootConfig(CHANNELS_BASELINE);
    for(int ch=0; ch < baseLine.size(); ++ch){
        root_channels_baseline.set_data(QString::number(ch), baseLine[ch]);
    }
    root.append_child(root_channels_baseline);

    return root;
}
