#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "QMainWindow"
#include "mongo/StartupOptions.h"
#include "configuration-manager/ConfigurationManager.h"
#include "Config.h"
#include "DeviceWidget.h"
#include "device-discover/DiscoverDialog.h"
#include "mldiscover/DeviceIndex.h"
#include "pnp-server/PNPServer.h"
#include "QVBoxLayout"

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(StartupOptions &start_opt, QWidget *parent = nullptr);
    ~MainWindow();

signals:
    void change_program_description(const ProgramDescription &) const;
    void close_program(const ProgramDescription &) const;
    void delete_all();
    void config_updated();

private slots:
    void set_selected_device();
    void add_log(QtMsgType severity, const QString &log, const QString &thrName);
    void update_device_config(DeviceConfig device_config);
    void update_status(DeviceStatus device_status);

    void on_actionConfiguration_manager_triggered();
    void switch_to(QString program_index, QString configuration_name);
    void on_actionFind_device_triggered();
    void on_actionSet_hit_width_extension_triggered();
    void on_actionLut_c_triggered();
    void on_actionAbout_triggered();
    void on_pushButtonLog_clicked(bool checked);

private:
    void init();
    void init_gui();
    void populate_devices_widgets();
    void create_device_widget(DeviceConfig device_config);
    void populate_devices_table();
    void update_program_description();
    void set_window_title();
    int get_device_row(const DeviceIndex &index);
    DeviceIndex get_index_by_row(int row);
    static void update_table_size(QTableWidget *t);

    Ui::MainWindow *ui;
    Config config;
    ConfigurationManager *configuration_manager;
    DiscoverDialog *discover_dialog;
    ProgramDescription program_description;
    PNPServer *pnp_server;
    QMap<DeviceIndex, DeviceWidget*> device_widgets;
};

#endif // MAINWINDOW_H
