#ifndef MSCDEVICE_H
#define MSCDEVICE_H

#include "mregdev/qtmregdevice.h"
#include "DeviceConfig.h"
#include "QMetaType"

struct DeviceStatus {
    DeviceStatus(DeviceIndex i=DeviceIndex()) :
        index(i),
        temp(0),
        connected(false),
        status("disconnected")
    {}

    DeviceIndex index;
    double temp;
    bool connected;
    QString fwStr;
    fw_version_t fwRev;
    QString status;
    QVector<quint32> ch_cnt;
};
Q_DECLARE_METATYPE(DeviceStatus)

class MscDevice : public QtMregDevice
{
    Q_OBJECT
public:
    explicit MscDevice(DeviceConfig device_index);
    virtual bool checkLiveMagic();

signals:
    void update_status(DeviceStatus);
    void update_device_config(DeviceConfig);

private slots:
    void write_config_to_device(DeviceConfig last_config);
    void set_status(QString status);
    void find_base_line();
    void write_loot(QByteArray lut_data);
    void soft_clear();

protected Q_SLOTS:
    virtual void statusPoll();
    virtual void on_deviceConnected();
    virtual void on_deviceDisconnected();

private:
    void reset_dac();
    mlink::RegOpVector setChThreshold(QMap<int, int> chThr);


    DeviceConfig last_config;
    bool configValid;
    DeviceStatus device_status;
};

#endif // MSCDEVICE_H
