//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "DevParam.h"
#include <QSettings>
#include "ui_DevParam.h"

DevParam::DevParam(int devId, QWidget *parent) :
    QGroupBox(parent),
    ui(new Ui::DevParam),
    deviceId(devId),
    enabled(true)
{
    ui->setupUi(this);
    setTitle(QString("dev #%1").arg(deviceId));
    loadSettings();
    ui->spinBoxThrF->setMaximum(MAX_VOLTAGE);
    ui->spinBoxThrS->setMaximum(MAX_VOLTAGE);
    ui->spinBoxStrF->setMaximum(MAX_VOLTAGE);
    ui->spinBoxStrS->setMaximum(MAX_VOLTAGE);
    ui->spinBoxHysF->setMaximum(MAX_VOLTAGE);
    ui->spinBoxHysS->setMaximum(MAX_VOLTAGE);
}

DevParam::~DevParam()
{
    saveSettings();
    delete ui;
}

double DevParam::getParam(quint16 id) const
{
    switch (id) {
    case ParamThrF:
        return ui->spinBoxThrF->value();
    case ParamThrS:
        return ui->spinBoxThrS->value();
    case ParamStrF:
        return ui->spinBoxStrF->value();
    case ParamStrS:
        return ui->spinBoxStrS->value();
    case ParamHysF:
        return ui->spinBoxHysF->value();
    case ParamHysS:
        return ui->spinBoxHysS->value();
    }
    return 0;
}

void DevParam::setParam(quint16 id, double value)
{
    switch (id) {
    case ParamThrF:
        ui->spinBoxThrF->setValue(value);
        break;
    case ParamThrS:
        ui->spinBoxThrS->setValue(value);
        break;
    case ParamStrF:
        ui->spinBoxStrF->setValue(value);
        break;
    case ParamStrS:
        ui->spinBoxStrS->setValue(value);
        break;
    case ParamHysF:
        ui->spinBoxHysF->setValue(value);
        break;
    case ParamHysS:
        ui->spinBoxHysS->setValue(value);
        break;
    }
}

void DevParam::setEnabled(bool value)
{
    enabled = value;
    ui->spinBoxThrF->setEnabled(value);
    ui->spinBoxThrS->setEnabled(value);
    ui->spinBoxStrF->setEnabled(value);
    ui->spinBoxStrS->setEnabled(value);
    ui->spinBoxHysF->setEnabled(value);
    ui->spinBoxHysS->setEnabled(value);

    ui->spinBoxThrF->blockSignals(!value);
    ui->spinBoxThrS->blockSignals(!value);
    ui->spinBoxStrF->blockSignals(!value);
    ui->spinBoxStrS->blockSignals(!value);
    ui->spinBoxHysF->blockSignals(!value);
    ui->spinBoxHysS->blockSignals(!value);
}

void DevParam::on_spinBoxThrF_valueChanged(double val)
{
    saveSettings();
    if(enabled) emit valueChanged(deviceId, ParamThrF, val);
}

void DevParam::on_spinBoxThrS_valueChanged(double val)
{
    saveSettings();
    if(enabled) emit valueChanged(deviceId, ParamThrS, val);
}

void DevParam::on_spinBoxStrF_valueChanged(double val)
{
    saveSettings();
    if(enabled) emit valueChanged(deviceId, ParamStrF, val);
}

void DevParam::on_spinBoxStrS_valueChanged(double val)
{
    saveSettings();
    if(enabled) emit valueChanged(deviceId, ParamStrS, val);
}

void DevParam::on_spinBoxHysF_valueChanged(double val)
{
    saveSettings();
    if(enabled) emit valueChanged(deviceId, ParamHysF, val);
}

void DevParam::on_spinBoxHysS_valueChanged(double val)
{
    saveSettings();
    if(enabled) emit valueChanged(deviceId, ParamHysS, val);
}

void DevParam::loadSettings()
{
    QSettings settings("u40ve-8spi.ini", QSettings::IniFormat);
    settings.beginGroup(QString("set%1").arg(deviceId));
    double val;
    bool ok;
    val = settings.value("thrF").toDouble(&ok);
    if(ok){
        ui->spinBoxThrF->blockSignals(true);
        ui->spinBoxThrF->setValue(val);
        ui->spinBoxThrF->blockSignals(false);
    }
    val = settings.value("thrS").toDouble(&ok);
    if(ok){
        ui->spinBoxThrS->blockSignals(true);
        ui->spinBoxThrS->setValue(val);
        ui->spinBoxThrS->blockSignals(false);
    }
    val = settings.value("strF").toDouble(&ok);
    if(ok){
        ui->spinBoxStrF->blockSignals(true);
        ui->spinBoxStrF->setValue(val);
        ui->spinBoxStrF->blockSignals(false);
    }
    val = settings.value("strS").toDouble(&ok);
    if(ok){
        ui->spinBoxStrS->blockSignals(true);
        ui->spinBoxStrS->setValue(val);
        ui->spinBoxStrS->blockSignals(false);
    }
    val = settings.value("hysF").toDouble(&ok);
    if(ok){
        ui->spinBoxHysF->blockSignals(true);
        ui->spinBoxHysF->setValue(val);
        ui->spinBoxHysF->blockSignals(false);
    }
    val = settings.value("hysS").toDouble(&ok);
    if(ok){
        ui->spinBoxHysS->blockSignals(true);
        ui->spinBoxHysS->setValue(val);
        ui->spinBoxHysS->blockSignals(false);
    }
    settings.endGroup();
}

void DevParam::saveSettings()
{
    QSettings settings("u40ve-8spi.ini", QSettings::IniFormat);
    settings.beginGroup(QString("set%1").arg(deviceId));
    settings.setValue("thrF",ui->spinBoxThrF->value());
    settings.setValue("thrS",ui->spinBoxThrS->value());
    settings.setValue("strF",ui->spinBoxStrF->value());
    settings.setValue("strS",ui->spinBoxStrS->value());
    settings.setValue("hysF",ui->spinBoxHysF->value());
    settings.setValue("hysS",ui->spinBoxHysS->value());
    settings.endGroup();
}
