//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef DEVPARAM_H
#define DEVPARAM_H

#include <QGroupBox>

namespace Ui {
class DevParam;
}

const double MAX_VOLTAGE = 2.5;

enum ParamId {
    ParamThrF=0,ParamThrS=1,
    ParamStrF=2,ParamStrS=3,
    ParamHysF=4,ParamHysS=5
};
class DevParam : public QGroupBox
{
    Q_OBJECT

public:
    explicit DevParam(int devId, QWidget *parent = nullptr);
    ~DevParam();
    double getParam(quint16 id) const;
    void setParam(quint16 id, double value);
    void setEnabled(bool);

private slots:
    void on_spinBoxThrF_valueChanged(double val);
    void on_spinBoxThrS_valueChanged(double val);
    void on_spinBoxStrF_valueChanged(double val);
    void on_spinBoxStrS_valueChanged(double val);
    void on_spinBoxHysF_valueChanged(double val);
    void on_spinBoxHysS_valueChanged(double val);

signals:
    void valueChanged(int deviceId, quint16 paramId, double val);

private:
    void loadSettings();
    void saveSettings();

    Ui::DevParam *ui;
    const int deviceId;
    bool enabled;
};

#endif // DEVPARAM_H
