//
//    Copyright 2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "SpillConfig.h"

#include <QDebug>
#include <QSettings>

SpillConfig::SpillConfig()
    :
      durPhase1(0),
      durPhase2(0),
      durPhase3(1000),
      durPhase4(1),
      durPhase5(1000),
      source(SPILL_AUTO),
      softSpill(false),
      timerPeriod(7000),
      spillPlotStep(62500),
      trigTimeStep(62)
{
}

bool SpillConfig::operator==(const SpillConfig &rhs) const
{
    return true
            && durPhase1 == rhs.durPhase1
            && durPhase2 == rhs.durPhase2
            && durPhase3 == rhs.durPhase3
            && durPhase4 == rhs.durPhase4
            && durPhase5 == rhs.durPhase5
            && source == rhs.source
            && softSpill == rhs.softSpill
            && timerPeriod == rhs.timerPeriod
            && spillPlotStep == rhs.spillPlotStep
            && trigTimeStep == rhs.trigTimeStep
            ;
}

QDebug operator <<(QDebug stream, const SpillConfig &spill)
{
    return stream
            << QString("Spill phase 1: %1 ms").arg(spill.durPhase1) << endl
            << QString("Spill phase 2: %1 ms").arg(spill.durPhase2) << endl
            << QString("Spill phase 3: %1 ms").arg(spill.durPhase3) << endl
            << QString("Spill phase 4: %1 ms").arg(spill.durPhase4) << endl
            << QString("Spill phase 5: %1 ms").arg(spill.durPhase5) << endl
            << QString("Spill timer period: %1 ms").arg(spill.timerPeriod) << endl
            << QString("Spill source select: %1").arg(spill.source);
}

bool SpillConfig::root_to_variables(RootConfig root)
{
    int val;
    bool ok;
    durPhase1 = root.get_data<uint>("durPhase1", durPhase1);
    durPhase2 = root.get_data<uint>("durPhase2", durPhase2);
    durPhase3 = root.get_data<uint>("durPhase3", durPhase3);
    durPhase4 = root.get_data<uint>("durPhase4", durPhase4);
    durPhase5 = root.get_data<uint>("durPhase5", durPhase5);

    val = root.get_data<int>("source", source);
        switch (val) {
        case SPILL_OFF:
        case SPILL_TIMER:
        case SPILL_TTL:
        case SPILL_WR:
        case SPILL_AUTO:
        case SPILL_GATE:
            source = static_cast<spill_sel_t>(val);
            break;
        default:
            source = SPILL_AUTO;
    }
    softSpill = root.get_data<bool>("softSpill", softSpill);
    timerPeriod = root.get_data<quint32>("timerPeriod", timerPeriod);
    spillPlotStep = root.get_data<uint>("spillPlotStep", spillPlotStep);
    trigTimeStep = root.get_data<uint>("trigTimeStep", trigTimeStep);
    return true;
}

RootConfig SpillConfig::variables_to_root()
{
    RootConfig root_to_return = RootConfig("Spill");

    root_to_return.set_data("durPhase1", durPhase1);
    root_to_return.set_data("durPhase2", durPhase2);
    root_to_return.set_data("durPhase3", durPhase3);
    root_to_return.set_data("durPhase4", durPhase4);
    root_to_return.set_data("durPhase5", durPhase5);

    if(source == SPILL_OFF) root_to_return.set_data("source", 0);
    if(source == SPILL_TIMER) root_to_return.set_data("source", 1);
    if(source == SPILL_TTL) root_to_return.set_data("source", 2);
    if(source == SPILL_WR) root_to_return.set_data("source", 3);
    if(source == SPILL_AUTO) root_to_return.set_data("source", 7);
    if(source == SPILL_GATE) root_to_return.set_data("source", 8);

    root_to_return.set_data("softSpill", softSpill);
    root_to_return.set_data("timerPeriod", timerPeriod);
    root_to_return.set_data("spillPlotStep", spillPlotStep);
    root_to_return.set_data("trigTimeStep", trigTimeStep);

    return root_to_return;
}

