//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef TRIGCHANNELCONFIG_H
#define TRIGCHANNELCONFIG_H

#include <QWidget>
#include <QMap>

namespace Ui {
class TrigChannelConfig;
}
class Config;
class QTableWidget;
class TrigChannelState;

class TrigChannelConfig : public QObject
{
    Q_OBJECT

public:
    explicit TrigChannelConfig(QTableWidget *tableWidget = nullptr);
    void updateConfig();
    void init(Config *conf);
    void runUpdated();

public slots:
    void updateChState(const TrigChannelState &chState);
private slots:
    void onXOffToggled(bool checked);
    void tableCellChanged(int row, int col);

signals:
    void trigEnabled(int ch, bool enabled);
    void xOffEnabled(int ch, bool enabled);
    void configUpdated(quint16 xOffConf);

private:
    void setChNumer(int newChNum);
    void setTableHeaders();
    void addRow(int row);
    int chToRow(int ch) const { return ch; }
    int rowToCh(int row) const { return row; }
    void updateTableSize();
    void updateColVisibility();
    bool xOffEnabled(int ch) const;

    QTableWidget * const table;
    Config *config;
    int chNum;
    quint16 xOffConf;
    quint16 xOffLed;
    quint16 timeoutLed;
    bool changesAllowed;
};

#endif // TRIGCHANNELCONFIG_H
