//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef VMETRIGSWIDGET_H
#define VMETRIGSWIDGET_H

#include "run-control-client-manager/RunStates.h"
#include "pnp-server/PNPMessage.h"

#include <QWidget>
#include <QMap>

class DeviceDescription;
class VmeTrigConfig;
class VmeTrigDevice;
class VmeTrigStatus;
class Config;

namespace Ui {
class VmeTrigsWidget;
}

class VmeTrigsWidget : public QWidget
{
    Q_OBJECT

public:
    explicit VmeTrigsWidget(QWidget *parent = nullptr);
    ~VmeTrigsWidget();
    void init(Config *conf);
    void updateDevicesList(const QVector<DeviceDescription> devices);
    QString get_run_index() const { return last_got_run_index; }
    int get_run_number() const { return last_got_run_number; }
    RUN_STATES getRunState() const { return lastRunState; }

public slots:
    void onRunStart(QString run_index, int run_number);
    void onRunStop();
    void gotProgramDescription(const ProgramDescription &pd);
    void gotClosedProgram(const ProgramDescription &pd);
    void setControlEnable(bool enabled);
    void insertAvailableDevice(const DeviceDescription &dd);

private slots:
    void on_pushButtonAddVmeTrig_clicked();
    void on_pushButtonRemoveVmeTrig_clicked();
    void on_tableWidgetVmeTrig_itemSelectionChanged();

    void trigStatusUpdated(const VmeTrigStatus &status);
    void setStatus(const QString &status);
    void trigRunStateUpdated(quint64 serial, RUN_STATES state, QString run_index, int run_number);
    void vmeReadoutToggled(bool readout);
    void wrCheckToggled(bool readout);
    void checkOutOfDate();

signals:
    void deleteTrig(quint64 serial); // serial==0 - for all
    void resetEvNum(QString run_index, int run_number);
//    void resetEvNumFinished(int run_number, RUN_STATES state); //deprecated?
    void vmeReadoutUpdated(quint64 serial, bool readout);
    void deviceDiscovered(const DeviceDescription &dd);
    void updateTabState(bool isOk, int tabIndex=2);
    void runError(QString run_index, int run_number, QString errStr);
    void vmeMapUpdated(QMap<quint64, QString> map) const;
    void evNumRequest(int checkIndex) const;
    void evNumResponse(int checkIndex, quint64 serial, quint64 evNum) const;

private:
    void addTrig(const VmeTrigConfig &vc);
    void updateEnableAddButton();
    quint64 getSerialByRow(int row) const;
    int gerRowBySerial(quint64 serial) const;
    void updateControlEnable();
    int getSelectedRow() const;
    bool getSummaryState() const;
    void updateTablePalette();
    void sendVmeMap() const;

    Ui::VmeTrigsWidget *ui;
    Config * config;
    QMap<quint64, VmeTrigDevice*> trigs;
    QMap<quint64, QThread*> trigThreads;
    QMap<quint64, RUN_STATES> trigStates;
    QMap<quint64, VmeTrigStatus> trigStatus;

    QVector<DeviceDescription> allDevices;
    QString last_got_run_index;
    int last_got_run_number;
    RUN_STATES lastRunState;
    bool runEnable;
    QTimer * const checkOutOfDateTimer;
    QMap<QUuid, ProgramDescription> programMap;
    QMap<quint64, QString> trigHostMap;
    bool controlEnabled;
};

#endif // VMETRIGSWIDGET_H
