#ifndef HISTMNGRCFG_H
#define HISTMNGRCFG_H

#include <QMetaType>
#include <QVector>

#include "mldiscover/DeviceIndex.h"

struct HistId {
    static int tdcCh;
    static int tdcTaiTime;
    static int tdcTime;
    static int tdcWidth;
    static int tdcDoublePulseTime;
    static int tdcHitEv;
    static int inl;
    static int inlCheck;
    static int errors;
    static const int nch;
};

struct HistCfg
{
    HistCfg():
        HistCfg(false, -1, "", 1, 0, 0, false)
    {}

    HistCfg(bool _en, int gId, QString _title, int _nbins, double _min, double _max, bool isFittable=true, bool baseHist=true):
        histGroupEn(_en),
        groupId(gId),
        title(_title),
        nBins(_nbins),
        min(_min),
        max(_max),
        fittable(isFittable),
        showFit(false),
        devCh(0),
        isBaseHist(baseHist)
    {}

    QString getGroupTitle() const { return histGroupEn ? title : (title+" (off)"); }
    bool hasStopCh() const;

    bool histGroupEn; // depends on groupId only, same for all ch
    int groupId; // id in comboBox
    QString title;
    int nBins;
    double min;
    double max;

    bool fittable;
    bool showFit; // depends on ch and groupId

    DeviceIndex devIndex;
    int devCh; // -1 - group doesn't use device/ch selection

    bool isBaseHist;
};

struct HistMngrCfg
{
    HistMngrCfg() :
        curHistId(-1),
        useStopCh(false),
        stopCh(0),
        inlHistCodeDens(true),
        globalEnabled(true),
        taiTsWindow(3600),
        mwMin(-100),
        mwMax(100),
        log_y_axis(false)
    {
    }
    void init(bool isInlEnabled = false);
    void load();
    void save();
    void matchWinUpdated(int min, int max);
    QString getGroupToolTip(int groupId, bool with_suffix=false) const;
    bool doTdcProcessing() const;
    DeviceIndex getStopIndex() const;
    QList<DeviceIndex> getDevOrder() const;
    int getMaxChNum() const;

    QString curHistTitle;
    int curHistId;
    bool useStopCh; // worker
    quint16 stopCh; // worker
    bool inlHistCodeDens; // hist
    DeviceIndex stopDevIndex; // worker
    bool globalEnabled; // worker
    int taiTsWindow; // worker
    int mwMin; // match window min in ns
    int mwMax; // match window max in ns
    bool log_y_axis;

    QVector<HistCfg> histCfg;
    QMap<DeviceIndex, int> devChOffset;
    QMap<int, QString> toolTipSuffix; // not to save
};
Q_DECLARE_METATYPE(HistMngrCfg)

#endif // HISTMNGRCFG_H
