#ifndef HISTMNGRWIDGET_H
#define HISTMNGRWIDGET_H

#include <QElapsedTimer>
#include <QMap>
#include <QWidget>

#include "HistMngrCfg.h"
#include "HistogramItem.h"
#include "DeviceCalibration.h"

#include "histogramm/Histogramm.h"
#include "HistWorker.h"

namespace Ui {
class HistMngrWidget;
}

class DeviceIndex;
class DecodedData;
class QwtPlotCurve;
class QTime;
class QTimer;
class QSemaphore;

class CompleteEvent;
class TdcChHit;
class TdcEvent;

#include "InlCheckResults.h"

struct InlStat
{
    QElapsedTimer time;
    QMap<int, double> count;
};

class HistMngrWidget : public QWidget
{
    Q_OBJECT

public:
    explicit HistMngrWidget(QWidget *parent = nullptr);
    ~HistMngrWidget();
    HistMngrCfg init(bool isInlEnabled = false);
    static QVector<double> makeDNLCorrectionTable(const QVector<double> code_density);
    void updateInlCorr(QMap<DeviceIndex, DeviceCalibration> allCal);
    void updateInlCorr(const DeviceIndex &index, const QMap<int, QVector<double> > &corrMap);
    HistContainer getHists() const { return histContainer; }
    HistMngrCfg getCfg() const { return cfg; }

signals:
    void inlCorrUpdated(const DeviceIndex &index, const QMap<int, QVector<double> > &corrMap);
    void inlCheckUpdated(InlCheckResults res);
    void hist_cfg_updated(const HistMngrCfg &);
    void recreateTaiHist(int winSec=0);
    void clearHist(int id=-1, DeviceIndex=DeviceIndex(), int=0);

private slots:
    void refreshCurrentPlotC() {refreshCurrentPlotA();}
    void refreshCurrentPlotA(bool force=false);
    void on_comboBoxGroup_currentIndexChanged(int);
    void on_checkBoxHistGroupEn_toggled(bool checked);
    void on_comboBoxDev_currentIndexChanged(int);
    void on_spinBoxCh_valueChanged(int);
    void on_groupBoxStopCh_toggled(bool val);
    void on_comboBoxStopDev_currentIndexChanged(int);
    void on_spinBoxStopCh_valueChanged(int ch);
    void on_spinBoxMwMin_valueChanged(int min);
    void on_spinBoxMwMax_valueChanged(int max);
    void on_radioButtonInl_toggled(bool checked);
    void on_pushButtonClear_clicked();
    void on_pushButtonClearAll_clicked();
    void on_checkBoxGlobalEnable_toggled(bool checked);
    void on_groupBoxFit_toggled(bool checked);
    void on_checkBoxLogY_toggled(bool checked);
    void on_checkBoxZeroBin_clicked(bool checked);

    void matchWinUpdated(int matchWin);

public slots:
    void refreshCurrentPlotB(const HistContainer &hc, bool force=false);
    void inlCheckRequest();
    void devListUpdated(const HistDevType &devChOffset, const HistContainer &hc);

    void resetHistMngr();

private:
    void addDevice(const DeviceIndex &index);
    void updateFit(const Histogramm *cutHist);
    DeviceIndex getDevIndex(bool getStopDev, int idInComboBox=-1) const;
    DeviceIndex getCurDev(bool getStopDev=false) const;
    int getCurCh() const;
    HistCfg& getCurHistCfg(); // return current histCfg
    Histogramm *getCurHist();
    HistCfg onCurHistChanged(); // update GUI for current hist & return histCfg
    int getTotalChNum(const QMap<DeviceIndex, int> &devChOffsetOld);
    bool hasHistById(int id) const;
    Histogramm *getHistById(int id);
    Histogramm *getHistByIdIndexCh(int id, const DeviceIndex &index, int ch);
    void updateCbStopDev(QList<DeviceIndex> list);
    QVector<double> getInl(const TdcEvent &ev, const TdcChHit &tdcEv) const;
    QVector<double> getInl(const DeviceIndex &index, int ch) const;
    void addBaseHist(HistCfg cfg, int nbins, int min, int max);
    void updateHistToolTip();
    void updateSpinBoxMaxCh(bool stopDev);

    Ui::HistMngrWidget *ui;
    QElapsedTimer lastRefreshTime;
    QTimer * const refreshTimer;
    QMap<int, HistogramItem *> histQwtItems;
    QMap<DeviceIndex, QMap<int, QwtPlotCurve *> > inlCurvs;
    QMap<DeviceIndex, QMap<int, HistogramItem *> > devChHists;
    HistContainer histContainer;
    QSemaphore * histSem;
    HistMngrCfg cfg;
    InlResults inlFromConfig;
    InlStat inlStat;
    QList<DeviceIndex> knownDevices;
    bool hide_zero_bin;
};

#endif // HISTMNGRWIDGET_H
