#ifndef TDCANALYZER_H
#define TDCANALYZER_H

#include <QObject>
#include <QByteArray>

#include "Config.h"
#include "DataTypes.h"
#include "DeviceData.h"
#include "mldiscover/DeviceIndex.h"

class QSemaphore;
class QTimer;

#pragma pack(push, 1)
struct __attribute__ ((packed)) DataBlockHeader
{
    DataBlockHeader()
        : data_len(0), specific_bits(0), data_type(0){}
    quint16 data_len;
    quint16 specific_bits:12;
    quint16 data_type:4;
};
#pragma pack(pop)

class TDCAnalyzer : public QObject
{
    Q_OBJECT
public:
    explicit TDCAnalyzer(QSemaphore *_analyzerSem,
                          QSemaphore *_histSem,
                          QSemaphore *_writerSem,
                          QObject *parent = nullptr);

signals:
    void addLog(QString) const;
    void deviceDataUpdated(const DecodedData &data) const;
    void deviceStatsUpdated(const AnalyzerStats &stats) const;
    void rawDataUpdated(const QStringList &data);

public slots:
    void dataAccuired(QByteArray newData);
    void setRawDataEveryEv(bool everyEv);
    void resetStats();
    void requestRawData(int events);
    void invertUpdated(InvertMapType _invertMap);
    void set_can_skip(bool skip);
    void set_show_skip_info(bool show);

private slots:
    void read();
    void sendStat();

private:
    void sendLog(QString s) const;
    quint32 readDevice(uint upperLimitToDevLength);
    bool findNextEventHeader();
    bool readEvent();
    void flushData();
    void decodeTdcBlock(const quint32* payloadPtr);
    void decodeTtvxsBlock(const quint32* payloadPtr);
    void decodeTdcStat(const quint32* payloadPtr);
    void printRawTxt(uint size, QString header=QString(), int shift=0);
    void printRawWord(const void *word, QString suffix=QString());

    QSemaphore * const analyzerSem;// free space in analyzer queue
    QSemaphore * const histSem;// free space in histogram queue
    QSemaphore * const writerSem;// free space in writer queue (raw or decoded)

    QTimer * const statTimer;
    QTimer * const readTimer;
    QByteArray data;
    uint curPos;
    uint curEventStartPos;
    const MpdEventHdr *mpdEventHdr;
    const MpdDeviceHdr *mpdDeviceHdr;
    const MpdMStreamHdr *mstreamHdr;
    const AdcDataType1 *tsTAI;
    const DataBlockHeader *dataBlockHdr; // seems will be deprecated
    const VMEDataTypes *vmeData;

    quint32 lastEventNumAtPrevFrame;
    quint32 prevEventNum;

    bool previousProceed;
    bool enabled;

//    QMap<DeviceIndex, DeviceData> deviceData;
    AnalyzerStats analyzerStats;
    DecodedData decodedData;
    CompleteEvent curCompleteEv;
    TdcEvent curTdcEv;
    TtvxsEvent curTtvxsEv;
    TdcStat curTdcStat;
    TdcChHit curTdcHit;
    bool signalInverted;
    QStringList rawDataTxt;
    int rawDataEvents; // number events that should be listed (tab rawData)
    bool printBadEvent;
    bool rawDataEveryEv;
    QStringList lastErrStr;
    int clear_err_stack;
    InvertMapType invertMap;
    bool can_skip;
    bool show_skip_info;
};


#endif // TDCANALYZER_H
