#include "DeviceGlob.h"

#include <QThread>
#include <QProcess>
#include <QSemaphore>
#include <QObject>

#include "base/threadQuit.h"

DeviceGlob::DeviceGlob(DeviceIndex i, bool debug) :
    QObject(nullptr),
    index(i),
    dev(new TDCDevice(i.getDevId(), i.getSerial(), debug)),
    devThread(new QThread()),
    mstream(nullptr),
    msThread(new QThread()),
    setup_index(0),
    dev_online(false),
    dev_warn(false)
{
    dev->moveToThread(devThread);

    devThread->setObjectName(QString("Dev. %1").arg(i.getSerialStr()));

    QObject::connect(dev, &QObject::destroyed, devThread, &QThread::quit);
}

DeviceGlob::~DeviceGlob()
{
    delete devThread;
    delete msThread;
}

void DeviceGlob::startThreads()
{
    devThread->start();
}

QColor DeviceGlob::getColor() const
{
    return dev_online ? (dev_warn ? Qt::red : Qt::darkGreen) : Qt::darkGray;
}

void DeviceGlob::deleteDevice(DeviceIndex i)
{
    if(!i.isNull() && i!=index)
        return;
    deleteLater();
}
