#include "DeviceSetup.h"

#include <QSettings>
#include <QBitArray>


DeviceSetup::DeviceSetup():
    enabled(true),
    run(false),
    order_index(-1),
    latency(8),
    matchWin(10),
    hptdcHdr(false),
    hptdcTrl(false),
    leadEdge(true),
    trailEdge(true),
    stat_readout(true),
    stat_period(1000),
    trigPeriod(125000000),
    pll_bypass(true),
    setup_index(0),
    nch(0),
    index(DeviceIndex())
{
    qRegisterMetaType<Adjust_setup>();
    trigsetup.all = 0;
    setNCh(0);
}

bool DeviceSetup::root_to_variables(RootConfig root)
{
    enabled = root.get_data<bool>("enabled", enabled);
    order_index = root.get_data<int>("order_index", order_index);
    latency = root.get_data<quint32>("latency", latency);
    matchWin = root.get_data<quint32>("matchWin", matchWin);
    hptdcHdr = root.get_data<bool>("hptdcHdr", hptdcHdr);
    hptdcTrl = root.get_data<bool>("hptdcTrl", hptdcTrl);
    leadEdge = root.get_data<bool>("leadEdge", leadEdge);
    trailEdge= root.get_data<bool>("trailEdge", trailEdge);
    stat_readout= root.get_data<bool>("stat_readout", stat_readout);
    stat_period= root.get_data<quint16>("stat_period", stat_period);
    trigPeriod= root.get_data<quint32>("trigPeriod", trigPeriod);
    pll_bypass = root.get_data<bool>("pll_bypass", pll_bypass);
    nch = root.get_data<int>("nch", nch);

    index.first = root.get_data<quint16>("device_id", index.first);
    index.second = root.get_data<quint32>("device_serial", index.second);

    for(QString child_name : root.children().keys()){
        if(child_name == "trigsetup"){
            RootConfig trigsetup_root = root.child(child_name);
            reg_trig_ctrl trigsetup_tmp;
            trigsetup_tmp.root_to_variables(trigsetup_root);
            trigsetup = trigsetup_tmp;
        }

        if(child_name == "adjust_setup"){
            RootConfig adjust_setup_root =  root.child(child_name);
            Adjust_setup adjust_set_tmp;
            adjust_set_tmp.root_to_variables(adjust_setup_root);
            adjust_set = adjust_set_tmp;
        }

        if(child_name == "chTdcEn"){
            RootConfig chTdcEn_root = root.child(child_name);
            for(QString data : chTdcEn_root.data().keys()){
                chTdcEn[data.toInt()] = chTdcEn_root.get_data<bool>(data);
            }
        }

        if(child_name == "stat_preset"){
            RootConfig preset_root = root.child(child_name);
            stat_preset.root_to_variables(preset_root);
        }

    }
    return true;
}

RootConfig DeviceSetup::variables_to_root()
{
    RootConfig root_to_return;
    root_to_return.set_data("enabled", enabled);
    root_to_return.set_data("run", run);
    root_to_return.set_data("order_index", order_index);
    root_to_return.set_data("latency", latency);
    root_to_return.set_data("matchWin", matchWin);
    root_to_return.set_data("hptdcHdr", hptdcHdr);
    root_to_return.set_data("hptdcTrl", hptdcTrl);
    root_to_return.set_data("leadEdge", leadEdge);
    root_to_return.set_data("trailEdge", trailEdge);
    root_to_return.set_data("stat_readout", stat_readout);
    root_to_return.set_data("stat_period", stat_period);
    root_to_return.set_data("trigPeriod", trigPeriod);
    root_to_return.set_data("pll_bypass", pll_bypass);
    root_to_return.set_data("nch", nch);
    setNCh(nch);

    root_to_return.set_data("device_id", index.first);
    root_to_return.set_data("device_serial", index.second);

    RootConfig root_trigsetup = trigsetup.variables_to_root();
    root_to_return.append_child(root_trigsetup);

    RootConfig root_adjust_setup = adjust_set.variables_to_root();
    root_to_return.append_child(root_adjust_setup);

    RootConfig root_chTdcEn("chTdcEn");
    for(int i : chTdcEn.keys()) {
        root_chTdcEn.set_data(QString("%1").arg(i,2,10,QChar('0')), chTdcEn[i]);
    }
    root_to_return.append_child(root_chTdcEn);

    RootConfig root_preset = stat_preset.variables_to_root();
    root_to_return.append_child(root_preset);

    return root_to_return;
}

void DeviceSetup::setNCh(int n)
{
    bool defTdcEn = chTdcEn.empty() ? true : chTdcEn.first();
    for(int ch=0; ch<n; ++ch) {
        if(!chTdcEn.contains(ch))
            chTdcEn[ch] = defTdcEn;
    }
    nch = n;
}

bool DeviceSetup::compare(const DeviceSetup& s1, const DeviceSetup& s2)
{
    if(s1.order_index == -1) {
        if(s2.order_index == -1) {
            return s1.index.getSerial() < s2.index.getSerial();
        } else {
            return true;
        }
    } else {
        if(s2.order_index == -1) {
            return true;
        } else {
            if(s1.order_index == s2.order_index)
                return s1.index.getSerial() < s2.index.getSerial();
            else
                return s1.order_index < s2.order_index;
        }
    }
}

bool Adjust_setup::root_to_variables(RootConfig root)
{
    readout_fifo_size =  root.get_data<quint8>("readout_fifo_size", readout_fifo_size);
    mode_rc_compression =  root.get_data<bool>("mode_rc_compression", mode_rc_compression);
    leading_resolution =  root.get_data<quint16>("leading_resolution", leading_resolution);
    enable_pair =  root.get_data<bool>("enable_pair", enable_pair);
    dll_mode =  root.get_data<quint16>("dll_mode", dll_mode);
    dll_clock=  root.get_data<quint16>("dll_clock", dll_clock);
    core_clock_source =  root.get_data<quint16>("core_clock_source", core_clock_source);
    return true;
}

RootConfig Adjust_setup::variables_to_root()
{
    RootConfig root_to_return("adjust_setup");
    root_to_return.set_data("readout_fifo_size", readout_fifo_size);
    root_to_return.set_data("mode_rc_compression", mode_rc_compression);
    root_to_return.set_data("leading_resolution", leading_resolution);
    root_to_return.set_data("enable_pair", enable_pair);
    root_to_return.set_data("dll_mode", dll_mode);
    root_to_return.set_data("dll_clock", dll_clock);
    root_to_return.set_data("core_clock_source", core_clock_source);
    return root_to_return;
}

StatPreset StatPresetFabric::getPreset(int type)
{
    StatPreset sp;
    sp.type = (StatPreset::STAT_TYPE)type;
    switch (type) {
    case StatPreset::STAT_DEBUG:
    {
        sp.label = "debug";
        sp.regs << 0x0226;
        return sp;
    }
    case StatPreset::STAT_Test:
    {
        sp.label = "test";
        sp.regs.clear();
        sp.regs << 0x0227;
        return sp;
    }
    case StatPreset::STAT_DEFAULT:
    default:
        return StatPreset();
    }
}

QList<StatPreset> StatPresetFabric::getAllPresets()
{
    QList<StatPreset> all_presets;
    for(int i=0; i<StatPreset::STAT_PRESET_CNT; ++i)
        all_presets << getPreset(i);
    return all_presets;
}

bool StatPreset::root_to_variables(RootConfig root)
{
    label = root.get_data<QString>("label");
    if(!label.isEmpty()) {
        for(auto p : StatPresetFabric::getAllPresets()){
            if(p.label == label){
                *this = p;
                return true;
            }
        }
    }
    *this = StatPreset();
    return true;
}

RootConfig StatPreset::variables_to_root()
{
    RootConfig root_to_return("stat_preset");
    root_to_return.set_data("label", label);
    RootConfig root_regs;
    for(int i=0; i<regs.size(); ++i) {
        root_regs.set_data(QString("%1").arg(i, 2, 10, QChar('0')),
                           regs[i]);
    }
    root_regs.item_name= "regs";
    root_to_return.append_child(root_regs);
    return  root_to_return;
}
