#ifndef DEVICESETUP_H
#define DEVICESETUP_H

#include <mongo/RootConfig.h>
#include <mldiscover/DeviceIndex.h>
#include <QMetaType>
#include <QBitArray>

class QSettings;

typedef union {
    quint16 all;
    struct {
        unsigned int timer_en: 1;
        unsigned int ttl_en: 1;
        unsigned int ttc_en: 1;
        unsigned int vxs_en: 1;
        unsigned int unused2: 12;
    } bit;
    bool root_to_variables(RootConfig root)
    {
        bit.timer_en =  root.get_data<uint>("timer_en", bit.timer_en);
        bit.ttl_en = root.get_data<uint>("ttl_en", bit.ttl_en);
        bit.ttc_en = root.get_data<uint>("ttc_en", bit.ttc_en);
        bit.vxs_en = root.get_data<uint>("vxs_en", bit.vxs_en);
        return true;
    }
    RootConfig variables_to_root()
    {
        RootConfig root_to_return("trigsetup");
        root_to_return.set_data("timer_en", bit.timer_en);
        root_to_return.set_data("ttl_en", bit.ttl_en);
        root_to_return.set_data("ttc_en", bit.ttc_en);
        root_to_return.set_data("vxs_en", bit.vxs_en);

        return root_to_return;
    }
} reg_trig_ctrl;


struct StatPreset {
    enum STAT_TYPE {STAT_DEFAULT,
                    STAT_DEBUG,
                    STAT_Test,
                    STAT_PRESET_CNT};
    StatPreset() {
        type = STAT_DEFAULT;
        label = "default";
        regs    << 0x004B // Board Temperature, RO
                << 0x004C // F/W Version, RO
                << 0X004D // F/W Revision, RO

                << 0x4001 // MCU
                << 0x4002
                << 0x4003
                << 0x4004
                << 0x4005
                << 0x4006
                << 0x4007
                << 0x4008
                << 0x4009
                << 0x400A
                << 0x400B
                << 0x400C;
    }
    bool root_to_variables(RootConfig root);
    RootConfig variables_to_root();

    STAT_TYPE type;
    QString label;
    QVector<quint16> regs;
};

class StatPresetFabric {
public:
    static StatPreset getPreset(int type);
    static QList<StatPreset> getAllPresets();
};

class Adjust_setup
{
public:
    Adjust_setup(){ clear(); }
    void clear() {
        readout_fifo_size = 7;
        mode_rc_compression=true;
        leading_resolution=0;
        enable_pair=false;
        dll_mode=2;
        dll_clock=3;
        core_clock_source=1;
    }
    inline bool operator!=(const Adjust_setup &s) const {
        return
        readout_fifo_size != s.readout_fifo_size ||
        mode_rc_compression != s.mode_rc_compression ||
        leading_resolution != s.leading_resolution ||
        enable_pair != s.enable_pair ||
        dll_mode != s.dll_mode ||
        dll_clock != s.dll_clock ||
        core_clock_source != s.core_clock_source;
    }
    bool root_to_variables(RootConfig root);
    RootConfig variables_to_root();

    quint8 readout_fifo_size;
    bool mode_rc_compression;
    quint16 leading_resolution;
    bool enable_pair;
    quint16 dll_mode;
    quint16 dll_clock;
    quint16 core_clock_source;
};
Q_DECLARE_METATYPE(Adjust_setup)

class DeviceSetup
{
public:
    DeviceSetup();
    bool root_to_variables(RootConfig root);
    RootConfig variables_to_root();
    int getNCh() const { return nch; }
    void setNCh(int n);
    void copy_config(const DeviceSetup &s) {
        run = s.run;
        latency = s.latency;
        matchWin = s.matchWin;
        hptdcHdr = s.hptdcHdr;
        hptdcTrl = s.hptdcTrl;
        leadEdge = s.leadEdge;
        trailEdge = s.trailEdge;
        adjust_set = s.adjust_set;
        pll_bypass = s.pll_bypass;
        trigsetup = s.trigsetup;
        trigPeriod = s.trigPeriod;
        stat_readout = s.stat_readout;
        stat_period = s.stat_period;
    }
    static bool compare(const DeviceSetup &s1, const DeviceSetup &s2);

    bool enabled;
    bool run;
    int order_index;

    quint32 latency; // in 24ns steps
    quint32 matchWin; // in 24ns steps

    bool hptdcHdr;
    bool hptdcTrl;

    bool leadEdge;
    bool trailEdge;

    bool stat_readout;
    quint16 stat_period; // in ms
    quint32 trigPeriod; // in 24ns step

    DeviceIndex index;
    reg_trig_ctrl trigsetup;
    Adjust_setup adjust_set;
    bool pll_bypass;
    QMap<int, bool> chTdcEn;
    StatPreset stat_preset;
    quint32 setup_index;

private:
    int nch;
};
Q_DECLARE_METATYPE(DeviceSetup)

#endif // DEVICESETUP_H
