#ifndef TDCDEVICE_H
#define TDCDEVICE_H


#include "mregdev/qtmregdevice.h"
#include "DeviceSetup.h"
#include "TdcStatus.h"

class QtMregDevice;

typedef union {
    quint32 all;
    struct {
        unsigned int leading_resolution: 3;  // 2:0
        unsigned int width_select: 4;        // 6:3
        unsigned int unused: 1;              // 7
        unsigned int mode_rc_compression: 1; // 8
        unsigned int mode_rc: 1;             // 9
        unsigned int DLL_Mode: 2;            // 11:10
        unsigned int enable_leading: 1;      // 12
        unsigned int enable_trailing: 1;     // 13
        unsigned int enable_pair: 1;         // 14
        unsigned int core_clock_source: 2;   // 16:15
        unsigned int dll_clock_source: 3;    // 19:17
        unsigned int unused2: 4;             // 23:20
        unsigned int enable_matching: 1;     // 24
        unsigned int unused3: 7;             // 31:25
    } bit;
} reg_measurement_ctrl;

typedef union {
    quint32 all;
    struct {
        unsigned int enable_local_header: 1;      // 0
        unsigned int enable_local_trailer: 1;     // 1
        unsigned int unused: 2;                   // 3:2
        unsigned int enable_error_mark: 1;        // 4
        unsigned int unused2: 3;                  // 7:5
        unsigned int readout_fifo_size: 3;        // 10:8
        unsigned int unused3: 1;                  // 11
        unsigned int max_event_size: 4;           // 15:12
        unsigned int reject_readout_fifo_full: 1; // 16
        unsigned int unused4: 3;                  // 19:17
        unsigned int dead_time: 2;                // 21:20
        unsigned int unused5: 10;                 // 31:22
    } bit;
} reg_read_ctrl;


class TDCDevice : public QtMregDevice
{
    Q_OBJECT
public:
    typedef enum {TRIG_NEGATIVE = 0, TRIG_POSITIVE = 1} TrigPolarity;
    explicit TDCDevice(quint16 devId, quint64 devSerial, bool isDebug=false);
    void readStatistics();
    void clearStatistics();
    virtual bool checkLiveMagic();

signals:
    void statusUpdated(const TdcStatus &);
    void deviceInfoUpdated(DeviceIndex index, int chNum);
    void evNumResponse(int checkIndex, quint64 serial,quint64 evNum);
    void deviceSetupWritten(DeviceIndex index, quint32 setup_index);

public slots:
    void writeSetup();
    void setSetup(const DeviceSetup &s);
    void writeSetupForce();
    void softReset();
    void blockDeviceRun(bool _isBlock);
    void reset_error_counters();
    void evNumRequest(int checkIndex);

protected slots:
    virtual void statusPoll();
    virtual void on_deviceConnected();
    virtual void on_deviceDisconnected();

private:
    mlink::RegOpVector WriteTrigSetup(reg_trig_ctrl trigsetup);
    void StartRun();
    void StopRun();
    int getNumberOfChannels() const;
    void writeTime();
    void readoutHptdcSetup();
    bool waitEndOfRun();
    void resetEvNumAndStats();
    mlink::RegOpVector writeStatCfg();
    mlink::RegOpVector getErrCntRegOp() const;
    void read_err_cnt(const mlink::RegOpVector &regRes, int pos, TdcErrCounters &errCnt);
    void read_bmc_power(const mlink::RegOpVector &regRes, int pos);
    bool update_err_cnt();

protected:
    void sendLog(const QString &);
    bool waitHptdcInit();
    bool debug;
    DeviceSetup setup;
    DeviceSetup setupCache;
    bool setupCacheValid;
    TdcStatus status;
    QMap<int, quint32> hitCnt;
    quint64 hitCntTime;
    bool isBlock;
    QElapsedTimer standByTime;
    QElapsedTimer standByDoneTime;
};

#endif // TDCDEVICE_H
