#include "TdcStatistic.h"

#include <QStringList>

namespace {
double getBusyVal(const quint64 &c1, const quint64 &c2, const quint64 &evNumAccepted, const TdcStatistic &st)
{
    const quint64 dEvNum = evNumAccepted-st.evNumAccepted;
    if(dEvNum==0) return 0;
    return 1./125*(c1-c2)/dEvNum;
}
}

QString TdcStatistic::getBusyStr(bool full, const TdcStatistic &st) const
{
    if(evNumAccepted==st.evNumAccepted) return QString();
    QString str = getBusyTotalStr(st);
    if(full) {
        str.prepend(QString("%1\n%2\n").arg(getBusyTdcStr(st)).arg(getBusyReadoutStr(st)));
    }
    return str;
}

QString TdcStatistic::getBusyTotalStr(const TdcStatistic &st) const
{
    return QString("%1 us").arg(getBusyTotal(st), 0, 'f', 2);
}

QString TdcStatistic::getBusyTdcStr(const TdcStatistic &st) const
{
    return QString("%1 us").arg(getBusyTdc(st), 0, 'f', 2);
}

QString TdcStatistic::getBusyReadoutStr(const TdcStatistic &st) const
{
    return QString("%1 us").arg(getBusyReadout(st), 0, 'f', 2);
}

double TdcStatistic::getBusyTotal(const TdcStatistic &st) const
{
    return getBusyVal(busy_total, st.busy_total, evNumAccepted, st);
}

double TdcStatistic::getBusyTdc(const TdcStatistic &st) const
{
    return getBusyVal(busy_tdc, st.busy_tdc, evNumAccepted, st);
}

double TdcStatistic::getBusyReadout(const TdcStatistic &st) const
{
    return getBusyVal(busy_readout, st.busy_readout, evNumAccepted, st);
}

QString TdcStatistic::getRunTime() const
{
    quint64 sec = getRunTimeSec();
    quint64 min = sec/60;
    quint64 h = min/60;
    QString str = QString("%1:%2:%3")
            .arg(h)
            .arg(min % 60, 2, 10, QChar('0'))
            .arg(sec % 60, 2, 10, QChar('0'));
    return str;
}

quint64 TdcStatistic::getRunTimeSec() const
{
    return run_time/125000000;
}

QString TdcErrCounters::toString() const
{
    QStringList errCntStr;
    if(hwipLinkDown)
        errCntStr << QString("hwipLinkDown: %1").arg(hwipLinkDown);
    if(regIoErr)
        errCntStr << QString("regIoErr: %1").arg(regIoErr);
    if(sfpErr)
        errCntStr << QString("sfpErr: %1").arg(sfpErr);
    if(hwIpErr)
        errCntStr << QString("hwIpErr: %1").arg(hwIpErr);
    if(txErr)
        errCntStr << QString("txErr: %1").arg(txErr);
    if(rxErr)
        errCntStr << QString("rxErr: %1").arg(rxErr);
    return errCntStr.join('\n');
}
