#ifndef TDCSTATISTIC_H
#define TDCSTATISTIC_H

#include <QString>

struct TdcErrCounters
{
    TdcErrCounters()
    {
        clear();
    }
    void clear() {
        hwipLinkDown=0;
        regIoErr=0;
        sfpErr=0;
        hwIpErr=0;
        txErr=0;
        rxErr=0;
    }
    void operator-=(const TdcErrCounters &cnt) {
        hwipLinkDown -= cnt.hwipLinkDown;
        regIoErr -= cnt.regIoErr;
        sfpErr -= cnt.sfpErr;
        hwIpErr -= cnt.hwIpErr;
        txErr -= cnt.txErr;
        rxErr -= cnt.rxErr;
    }
    QString toString() const;

    quint64 hwipLinkDown;
    quint64 regIoErr;
    quint64 sfpErr;
    quint64 hwIpErr;
    quint64 txErr;
    quint64 rxErr;
};

struct TdcStatistic
{
    TdcStatistic() {
        powerOnErrCnt.clear();
        clear();
    }
    void clear() {
        fwWithNewCnt=false;
        evNumCandidateRun=0;
        evNumAccepted=0;
        evNumBlocked=0;
        run_time=0;
        busy_total=0;
        busy_tdc=0;
        busy_readout=0;
//        timeValidLoss=0;
//        rxCodeError=0;
//        pllUnlockCnt=0;
    }
    QString getBusyStr(bool full, const TdcStatistic &st = TdcStatistic()) const;
    QString getBusyTotalStr(const TdcStatistic &st = TdcStatistic()) const;
    QString getBusyTdcStr(const TdcStatistic &st = TdcStatistic()) const;
    QString getBusyReadoutStr(const TdcStatistic &st = TdcStatistic()) const;
    double getBusyTotal(const TdcStatistic &st = TdcStatistic()) const;
    double getBusyTdc(const TdcStatistic &st = TdcStatistic()) const;
    double getBusyReadout(const TdcStatistic &st = TdcStatistic()) const;
    quint64 getBlocked() const {return fwWithNewCnt ? evNumBlocked : (evNumCandidateRun-evNumAccepted);}
    QString getRunTime() const;
    quint64 getRunTimeSec() const;

    bool fwWithNewCnt;
    quint64 evNumCandidateRun;
    quint64 evNumAccepted;
    quint64 evNumBlocked;
    quint64 run_time;
    quint64 busy_total;
    quint64 busy_tdc;
    quint64 busy_readout;
//    quint32 timeValidLoss;
//    quint32 rxCodeError;
//    quint32 pllUnlockCnt;
    TdcErrCounters errCnt;
    TdcErrCounters powerOnErrCnt;
};
#endif // TDCSTATISTIC_H
