#ifndef CALIBRATIONDIALOG_H
#define CALIBRATIONDIALOG_H

#include <QDialog>
#include <QMap>

class DeviceSetup;
class DeviceCalibration;

namespace Ui {
class CalibrationDialog;
}

class CalibrationDialog : public QDialog
{
    Q_OBJECT

public:
    explicit CalibrationDialog(QWidget *parent = 0);
    ~CalibrationDialog();

signals:
    void calibrationRequest();
    void clearCalibration();

public slots:
    void updateBaseline(const QMap<int, double> &baseline,
                        const QMap<int, double> &baseline4,
                        const QMap<int, double> rms=QMap<int, double>(),
                        const QMap<int, double> rms4=QMap<int, double>());
    void updateDacOffset(const QMap<int, int> &thr);
    void setChNumber(int num);
    void init(const DeviceCalibration &devCal);
    void updateInl(const QMap<int, QVector<double> > &inlCorr);
    void enableControl(bool enabled);

private slots:
    void on_pushButtonCalibrate_clicked();
    void on_pushButtonClear_clicked();
    void on_pushButtonClose_clicked();

private:
    QString getThrText(int dacValue) const;
    void initTable();
    void setTableInl(int ch, bool inlOk);
    void setItemValue(int ch, int col, const QMap<int, double> &chData, bool isRms);

    Ui::CalibrationDialog *ui;
    QMap<int, double> chBaseline;
    QMap<int, double> chBaselineRms;
    QMap<int, double> chBaseline4;
    QMap<int, double> chBaseline4Rms;
    QMap<int, int> dacOffset;
    QMap<int, bool> inlPresent;
    int chNum;
};

#endif // CALIBRATIONDIALOG_H
