#ifndef CONFIG_H
#define CONFIG_H

#include <QString>

#include "CalConfig.h"
#include "DeviceSetup.h"
#include "mongo/RootConfig.h"
#include "TQDC2StratupOptions.h"
#include "zmq-lib/ZmqTypes.h"

class Config : public CalConfig
{
public:
    Config(const TQDC2StartupOptions& startup_options);
    void read_config();
    void write_config();
    void loadSettings();
    void saveSettings();
    DeviceSetup &getCurrentConfig();
    DeviceCalibration &getCurrentCalibration();
    bool isRun();
    void updateInl(const DeviceIndex &index,
                   const QMap<int, QVector<double> > &inlCorr,
                   double temp);
    void updateBaseCal(const DeviceIndex &index,
                       const QMap<int, double> &baseline,
                       const QMap<int, double> &baseline4,
                       const QMap<int, int> &thr,
                       double temp);
    void clearCalibration(const DeviceIndex &index);

protected:
    virtual void reloadCalibration();

public:
    QString program_type;
    QString program_index;
    QString configuration_name;
    RootConfig default_root;

    bool separateChEn;
    bool writeEnabled;
    bool dataFileIsRaw;
    QString fileDir;
    QString filePrefix;
    bool fileTime;
    QString fileSuffix;
    uint rawDataLimit;

    DeviceIndex curDev;
    QMap<DeviceIndex, DeviceSetup> knownSetups;
    ZmqConfig zmq_config;
private:
    void update_root();
    RootConfig root;
};

#endif // CONFIG_H
