#ifndef DEVICEDATA
#define DEVICEDATA

#include "mldiscover/DeviceIndex.h"
#include <vector>
#include <QVector>
#include <map>

namespace  {
const double HPTDC_TS_STEP_NS = 0.024/1.024;
}

struct  TqdcEdge
{
    TqdcEdge(quint16 ts, quint32 t, bool l):time(t),tdcTs(ts),leading(l){}
    TqdcEdge(){ clear(); }
    void clear() {
        time=0;
        tdcTs=0;
        leading=true;
    }

    quint32 time;
    quint16 tdcTs;
    bool leading;
};

struct  TqdcEvent
{
    TqdcEvent(){ clear(); }
    void clear() {
        ch=0;
        tdcTs=0;
        leadingTime=0;
        trailingTime=0;
        width=0;
    }
    double getLeadTimeNs(const QVector<double> &inlCorr) const {
        return HPTDC_TS_STEP_NS*(inlCorr.value(leadingTime%1024,0)+leadingTime);
    }
    double getLeadTimeNs() const {
        return HPTDC_TS_STEP_NS*leadingTime;
    }
    double getTrailTimeNs() const {
        return HPTDC_TS_STEP_NS*trailingTime;
    }
    double getWidthNs() const {
        return HPTDC_TS_STEP_NS*(trailingTime-leadingTime);
    }

    quint8 ch;
    quint16 tdcTs;
    quint32 leadingTime;
    quint32 trailingTime;
    quint8 width;
};

struct AdcEvent
{
    AdcEvent () { clear(); }
    void clear() {
        ch = 0;
        adcTs = 0;
        mode = 0;
        wf.clear();
    }

    quint8 ch;
    quint16 adcTs; // 8ns intervals from event TS
    uint mode; // 1-cal; 2-samp; 3-sumQ
    std::vector<double> wf;
//    quint32 sumQ; // 18-bit, sum of adc samples
};

struct  CompleteEvent
{
    CompleteEvent(){ clear(); }
    void clear() {
        evNum=0;
        taiSec=0;
        taiFlags=0;
        taiNSec=0;
        adcEvents.clear();
        tqdcEvents.clear();
    }

    quint32 evNum;
    quint32 taiSec;
    quint32 taiFlags:2;
    quint32 taiNSec:30;
    std::map<int, QVector<AdcEvent> > adcEvents; // per channel
    std::map<int, QVector<TqdcEvent> > tqdcEvents; // per channel
};

struct DeviceData
{
    DeviceData () { clear(); }
    void clear() {
        events.clear();
        errors.clear();
    }

    DeviceIndex index;
    std::vector<CompleteEvent> events;
    std::vector<quint16> errors;
};
Q_DECLARE_METATYPE(DeviceData)

struct AnalyzerStats
{
    AnalyzerStats () { clear(); }
    void clear() {
        adcHits.clear();
        tdcHits.clear();
        formatErrorCnt=0;
        eventDecoded=0;
        sizeDecoded=0;
        decodeTimeNs=0;
    }

    quint32 eventDecoded;
    quint32 sizeDecoded;
    quint32 lastEvNum;
    qint64 decodeTimeNs; // decode time 200-300 us per max adcWin
    int formatErrorCnt;
    DeviceIndex index;
    QMap<int, quint32> adcHits; // per channel
    QMap<int, quint32> tdcHits; // per channel
};
Q_DECLARE_METATYPE(AnalyzerStats)

#endif // DEVICEDATA

