#include "DeviceSetup.h"

#include <QSettings>

namespace  {
inline QString getChStr(int i) {
    return QString("%1").arg(i, 2, 10, QChar('0'));
}
}
DeviceSetup::DeviceSetup():
    enabled(true),
    run(false),
    adcPreWin(2),
    readCellNumber(8),
    latency(8),
    matchWin(10),
    hptdcStandby(false),
    invertInput(false),
    invert_thr_trig(false),
    invert_zs_thr(false),
    zsEn(true),
    trigPeriod(125000000),
    selfTrigDelay(0),
    maxSampleCnt(0),
    nch(0)
{
    trigsetup.all = 0;
    setNCh(16);
}

bool DeviceSetup::root_to_variables(RootConfig root, QMap<int, int> &thrOffset)
{
    index.first = root.get_data<quint64>("device_id");
    index.second = root.get_data<quint64>("device_serial");
    enabled =  root.get_data<bool>("enabled", enabled);
//    run =  root.get_data<bool>("run", run);

    adcPreWin =  root.get_data<int>("adcPreWin", adcPreWin);
    readCellNumber =  root.get_data<int>("readCellNumber", readCellNumber);
    latency =  root.get_data<int>("latency", latency);
    matchWin =  root.get_data<int>("matchWin", matchWin);

    hptdcStandby =  root.get_data<bool>("hptdcStandby", hptdcStandby);
    invertInput =  root.get_data<bool>("invertInput", invertInput);
    invert_thr_trig =  root.get_data<bool>("invert_thr_trig", invert_thr_trig);
    invert_zs_thr=  root.get_data<bool>("invert_zs_thr", invert_zs_thr);

    zsEn = root.get_data<bool>("zsEn", zsEn);
    trigPeriod = root.get_data<quint32>("trigPeriod", trigPeriod);
    selfTrigDelay = root.get_data<quint16>("selfTrigDelay", selfTrigDelay);
    maxSampleCnt = root.get_data<int>("maxSampleCnt", maxSampleCnt);

    trigsetup.root_to_variables(root.child("trigsetup"));

    auto child = root.child("chTdcEn");
    for(QString ch : child.data().keys()){
        chTdcEn[ch.toInt()] = child.get_data<bool>(ch);
    }
    child = root.child("chTdcEn");
    for(QString ch : child.data().keys()){
        chTdcEn[ch.toInt()] = child.get_data<bool>(ch);
    }
    child = root.child("chAdcEn");
    for(QString ch : child.data().keys()){
        chAdcEn[ch.toInt()] = child.get_data<bool>(ch);
    }
    child = root.child("chTrigEn");
    for(QString ch : child.data().keys()){
        chTrigEn[ch.toInt()] = child.get_data<bool>(ch);
    }
    child = root.child("chThr");
    for(QString ch : child.data().keys()){
        chThr[ch.toInt()] = child.get_data<int>(ch);
    }
    child = root.child("chInv");
    for(QString ch : child.data().keys()){
        chInv[ch.toInt()] = child.get_data<bool>(ch);
    }
    child = root.child("chGain");
    for(QString ch : child.data().keys()){
        chGain[ch.toInt()] = child.get_data<bool>(ch);
    }

    child = root.child("thrOffset");
    for(QString ch : child.data().keys()){
        thrOffset[ch.toInt()] = child.get_data<int>(ch);
    }

    trigsetup.root_to_variables(root.child("trigsetup"));

    return !index.isNull();
}

RootConfig DeviceSetup::variables_to_root(DeviceCalibration cal)
{
    RootConfig root_to_return(QString("device_%1").arg(index.getSerialStr()));
    root_to_return.set_data("device_id", index.first);
    root_to_return.set_data("device_serial", index.second);

    root_to_return.set_data("enabled", enabled);
    root_to_return.set_data("run", run);

    root_to_return.set_data("adcPreWin", adcPreWin);
    root_to_return.set_data("readCellNumber", readCellNumber);
    root_to_return.set_data("latency", latency);
    root_to_return.set_data("matchWin", matchWin);
    root_to_return.set_data("hptdcStandby", hptdcStandby);
    root_to_return.set_data("invertInput", invertInput);
    root_to_return.set_data("invert_thr_trig", invert_thr_trig);
    root_to_return.set_data("invert_zs_thr", invert_zs_thr);

    root_to_return.set_data("zsEn", zsEn);
    root_to_return.set_data("trigPeriod", trigPeriod);
    root_to_return.set_data("selfTrigDelay", selfTrigDelay);
    root_to_return.set_data("maxSampleCnt", maxSampleCnt);

    RootConfig trigsetup_root;
    trigsetup_root = trigsetup.variables_to_root();
    trigsetup_root.item_name = "trigsetup";
    root_to_return.append_child(trigsetup_root);


    if(chTdcEn.size() != 0){
        RootConfig ch_tdc_en_root;
        for(int i : chTdcEn.keys())
        {

            ch_tdc_en_root.set_data(getChStr(i), chTdcEn[i]);
        }
        ch_tdc_en_root.item_name = "chTdcEn";
        root_to_return.append_child(ch_tdc_en_root);
    }

    if(chAdcEn.size() != 0){
        RootConfig ch_adc_en_root;
        for(int i : chAdcEn.keys())
        {
            ch_adc_en_root.set_data(getChStr(i), chAdcEn[i]);
        }
        ch_adc_en_root.item_name = "chAdcEn";
        root_to_return.append_child(ch_adc_en_root);
    }

    if(chTrigEn.size() != 0){
        RootConfig ch_trig_en_root;
        for(int i : chTrigEn.keys())
        {
            ch_trig_en_root.set_data(getChStr(i), chTrigEn[i]);
        }
        ch_trig_en_root.item_name = "chTrigEn";
        root_to_return.append_child(ch_trig_en_root);
    }

    if(chThr.size() != 0){
        RootConfig ch_thr_root;
        for(int i : chThr.keys())
        {
            ch_thr_root.set_data(getChStr(i), chThr[i]);
        }
        ch_thr_root.item_name = "chThr";
        root_to_return.append_child(ch_thr_root);
    }

    if(chInv.size() != 0){
        RootConfig ch_inv_root;
        for(int i : chInv.keys())
        {
            ch_inv_root.set_data(getChStr(i), chInv[i]);
        }
        ch_inv_root.item_name = "chInv";
        root_to_return.append_child(ch_inv_root);
    }

    if(chGain.size() != 0){
        RootConfig ch_gain_root;
        for(int i : chGain.keys())
        {
            ch_gain_root.set_data(getChStr(i), chGain[i]);
        }
        ch_gain_root.item_name = "chGain";
        root_to_return.append_child(ch_gain_root);
    }

    if(!cal.thrOffset.isEmpty()) {
        RootConfig thr_offset_root("thrOffset");
        for(int i : cal.thrOffset.keys())
            thr_offset_root.set_data(getChStr(i), cal.thrOffset[i]);
        root_to_return.append_child(thr_offset_root);
    }

    return root_to_return;
}

bool DeviceSetup::loadSettings(QSettings &s)
{
    bool ok, globOk=true;
    index.first = s.value("dev_id").toULongLong(&ok);
    globOk &= ok;
    index.second = s.value("dev_ser").toULongLong(&ok);
    globOk &= ok;
    enabled = s.value("enabled", enabled).toBool();
    adcPreWin = s.value("adcPreWin").toULongLong(&ok);
    globOk &= ok;
    readCellNumber = s.value("readCellNumber").toULongLong(&ok);
    globOk &= ok;
    latency = s.value("trigDelay").toULongLong(&ok);
    globOk &= ok;
    matchWin = s.value("matchWin", matchWin).toULongLong(&ok);
    globOk &= ok;
    nch = s.value("nch").toULongLong(&ok);
    globOk &= ok;
    trigPeriod = s.value("trigPeriod", trigPeriod).toUInt(&ok);
    globOk &= ok;
    selfTrigDelay = s.value("selfTrigDelay", selfTrigDelay).toUInt(&ok);
    globOk &= ok;
    setNCh(nch);
    invertInput = s.value("invertInput").toBool();
    zsEn = s.value("zsEn").toBool();
    invert_thr_trig = s.value("invert_thr_trig").toBool();
    invert_zs_thr = s.value("invert_zs_thr").toBool();
    trigsetup.all = s.value("trigsetup").toInt(&ok);
    globOk &= ok;

    s.beginReadArray("ch");
    for(int ch=0; ch<nch && globOk; ++ch){
        s.setArrayIndex(ch);
        chThr[ch] = s.value("thr").toInt(&ok);
        globOk &= ok;
        chInv[ch] = s.value("inv").toBool();
        chTdcEn[ch] = s.value("tdcEn").toBool();
        chAdcEn[ch] = s.value("adcEn").toBool();
        chGain[ch] = s.value("chGain").toBool();
        chTrigEn[ch] = s.value("chTrigEn").toBool();
        globOk &= ok;
    }
    s.endArray();
    return globOk;
}

void DeviceSetup::saveSettings(QSettings &s, DeviceCalibration deviceCalibration) const
{
    s.setValue("dev_id", index.first);
    s.setValue("dev_ser", index.second);
    s.setValue("enabled", enabled);
    s.setValue("adcPreWin", adcPreWin);
    s.setValue("readCellNumber", readCellNumber);
    s.setValue("trigDelay", latency);
    s.setValue("matchWin", matchWin);
    s.setValue("nch", nch);
    s.setValue("trigPeriod", trigPeriod);
    s.setValue("selfTrigDelay", selfTrigDelay);
    s.setValue("invertInput",invertInput);
    s.setValue("zsEn", zsEn);
    s.setValue("invert_thr_trig", invert_thr_trig);
    s.setValue("invert_zs_thr", invert_zs_thr);
    s.setValue("trigsetup", trigsetup.all);
    s.beginWriteArray("ch");
    for(int ch=0; ch<nch; ++ch) {
        s.setArrayIndex(ch);
        s.setValue("tdcEn", chTdcEn[ch]);
        s.setValue("adcEn", chAdcEn[ch]);
        s.setValue("thr", chThr[ch]);
        s.setValue("inv", chInv[ch]);
        s.setValue("chTrigEn", chTrigEn[ch]);
        s.setValue("chGain", chGain[ch]);
    }
    s.endArray();
    if(!deviceCalibration.thrOffset.isEmpty()) {

    }
}

void DeviceSetup::setThreshold(int thr)
{
    for(int ch=0; ch<nch; ++ch){
        if(chTdcEn[ch] || chAdcEn[ch])
            chThr[ch] = thr;
    }
}

void DeviceSetup::setNCh(int n)
{
    int defThr = chThr.empty() ? 0 : chThr.first();
    bool defInv = chInv.empty() ? false : chInv.first();
    bool defTdcEn = chTdcEn.empty() ? true : chTdcEn.first();
    bool defAdcEn = chAdcEn.empty() ? true : chAdcEn.first();
    for(int ch=nch; ch<n; ++ch){
        chThr[ch] = defThr;
        chInv[ch] = defInv;
        chTdcEn[ch]  = defTdcEn;
        chAdcEn[ch]  = defAdcEn;
        chTrigEn[ch] = true;
        chThr[ch] = 0;
        chInv[ch] = false;
        chGain[ch] = false;
    }
    nch = n;
}

bool DeviceSetup::isGain4() const
{
    return !chGain.isEmpty() && chGain.first();
}

