#include "HistMngrCfg.h"

#include <QSettings>

int HistId::tdcCh=-1;
int HistId::adcCh=-1;
int HistId::tdcTime=-1;
int HistId::tdcDoublePulseTime=-1;
int HistId::adcTime=-1;
int HistId::tdcAdcDt=-1;
int HistId::tdcHitEv=-1;
int HistId::adcHitEv=-1;
//int HistId::HIST_CH_Q=-1;
//int HistId::HIST_CH_SUMQ=-1;
int HistId::chSamples=-1;
int HistId::chAmpl=-1;
int HistId::groupChAmpl=-1;
int HistId::chInl=-1;
int HistId::errors=-1;
const int HistId::nch = 16;

void HistMngrCfg::init(bool isInlEnabled)
{
    int id=0;
    int maxGroupId = 0;

    HistId::tdcTime = id;
    histCfg.push_back(HistCfg(true, maxGroupId++, id++, "TDC Hit Time, ns", 10240, 0, 10240));

    HistId::tdcDoublePulseTime = id;
    histCfg.push_back(HistCfg(true, maxGroupId++, id++, "TDC Double-Pulse Time, ns", 10240, 0, 10240));
//    histTitles[HIST_TDC_DOUBLE_PULSE_TIME] = QString("TDC Double-Pulse Time, ns");
//    ++id;

    HistId::adcTime = id;
    histCfg.push_back(HistCfg(true, maxGroupId++, id++, "ADC Hit Time, ns", 10240, 0, 10240));

    HistId::tdcAdcDt = id;
    histCfg.push_back(HistCfg(true, maxGroupId++, id++, "dTime(TDC-ADC), ns", 2048, -1024, 1024));

    HistId::tdcCh = id;
    histCfg.push_back(HistCfg(true, maxGroupId++, id++, "TDC Hits by channel", HistId::nch, 1, HistId::nch+1));

    HistId::adcCh = id;
    histCfg.push_back(HistCfg(true, maxGroupId++, id++, "ADC Hits by channel", HistId::nch, 1, HistId::nch+1));

    HistId::tdcHitEv = id;
    histCfg.push_back(HistCfg(true, maxGroupId++, id++, "TDC Hits per event", 100, 1, 100));

    HistId::adcHitEv = id;
    histCfg.push_back(HistCfg(true, maxGroupId++, id++, "ADC Hits per event", 100, 1, 100));

    HistId::chSamples = id;
    for(int ch=0; ch<HistId::nch; ++ch) {
        histCfg.push_back(HistCfg(true, maxGroupId, id++, QString("ADC Samples ch:%1").arg(ch+1), 65536/4, -32768, 32768, ch));
    }
    ++maxGroupId;

    HistId::chAmpl = id;
    HistId::groupChAmpl = maxGroupId;
    for(int ch=0; ch<HistId::nch; ++ch) {
        histCfg.push_back(HistCfg(true, maxGroupId, id++, QString("ADC Amplitude ch:%1").arg(ch+1), 65536/4, -32768, 32768, ch));
    }
    ++maxGroupId;

    if(isInlEnabled) {
        HistId::chInl = id;
        for(int ch=0; ch<HistId::nch; ++ch) {
            histCfg.push_back(HistCfg(true, maxGroupId, id++, QString("INL ch:%1").arg(ch+1), 1024, 0, 1024, ch, false));
        }
        ++maxGroupId;
    }
//    HIST_CH_Q = id;
//    for(int i=0; i<N_CH; ++i){
//        histTitles[id] = QString("Q #%1").arg(i);
//        ++id;
//    }
//    HIST_CH_SUMQ = id;
//    for(int i=0; i<N_CH; ++i){
//        histTitles[id] = QString("Sum Q #%1").arg(i);
//        ++id;
//    }

//    HIST_CH_RAW_HITS = id;
//    for(int i=0; i<N_CH; ++i){
//        histMap[id]->SetMinimum( = QString("Raw hits #%1").arg(i);
//        ++id;
//    }
//    HIST_CH_T = id;
//    for(int i=0; i<N_CH; ++i){
//        histTitles[id] = QString("Time #%1").arg(i);
//        ++id;
//    }
//    HIST_CH_Q = id;
//    for(int i=0; i<N_CH; ++i){
//        histTitles[id] = QString("Q #%1").arg(i);
//        ++id;
//    }

//    HIST_CH_SUMQ = id;
//    histTitles[HIST_CH_SUMQ] = QString("TQDC Channels: SumQ");
//    histTRoot[HIST_CH_SUMQ] = new TH1I(histTitles[HIST_CH_SUMQ].toStdString().c_str(),
//                                         histTitles[HIST_CH_SUMQ].toStdString().c_str(),
//                                         N_CH, 1, N_CH);
//    ++id;

//    HIST_ERRORS = id;
//    histTitles[HIST_ERRORS] = QString("TQDC Errors bits");
//    histTRoot[HIST_ERRORS] = new TH1I(histTitles[HIST_ERRORS].toStdString().c_str(),
//                                         histTitles[HIST_ERRORS].toStdString().c_str(),
//                                         N_CH, 1, N_CH);
}

void HistMngrCfg::load()
{
    QSettings s;
    s.beginGroup("Histogram");
    curHistTitle = s.value("curHistType").toString();
    useStopCh = s.value("useStopCh").toBool();
    stopCh = s.value("stopCh").toInt();
    amplHistWithMax = s.value("amplHistWithMax").toBool();
    globalEnabled = s.value("enabled").toBool();

    QStringList subGroups = s.childGroups();
    for(int i=0; i<histCfg.size(); ++i) {
        HistCfg &hCfg = histCfg[i];
        if(!subGroups.contains(hCfg.title))
            continue;
        s.beginGroup(hCfg.title);
        hCfg.histGroupEn = s.value("en", hCfg.histGroupEn).toBool();
        hCfg.showFit = s.value("showFit", hCfg.showFit).toBool();
        s.endGroup();
    }
    s.endGroup();
}

void HistMngrCfg::save()
{
    QSettings s;
    s.beginGroup("Histogram");
    s.setValue("curHistType", curHistTitle);
    s.setValue("useStopCh", useStopCh);
    s.setValue("stopCh", stopCh);
    s.setValue("amplHistWithMax", amplHistWithMax);
    s.setValue("enabled", globalEnabled);

    for(int i=0; i<histCfg.size(); ++i) {
        HistCfg &hCfg = histCfg[i];
        s.beginGroup(hCfg.title);
        s.setValue("en", hCfg.histGroupEn);
        s.setValue("showFit", hCfg.showFit);
        s.endGroup();
    }
    s.endGroup();
}

QString HistMngrCfg::getGroupToolTip(int groupId) const
{
    QString tt;
    if(groupId==-1)
        return tt;
    for (const HistCfg &cfg : histCfg) {
        if(cfg.groupId!=groupId)
            continue;
        if(cfg.histId == HistId::tdcTime)
            tt = "Histogram fills with hit time from HPTDC part of event";
        if(cfg.histId == HistId::adcTime)
            tt = "Histogram fills with hit time from ADC part of event";
        if(cfg.histId == HistId::tdcCh) tt = "Hist. fills with ch. number of HPTDC hit";
        if(cfg.histId == HistId::adcCh) tt = "Hist. fills with ch. number on every ADC waveform";
        if(cfg.histId == HistId::tdcDoublePulseTime) tt = "Hist fills with ";
        if(cfg.histId == HistId::tdcAdcDt) tt = "Hist. fills with time differance between last HPTDC hit and last waveform";
        if(cfg.histId == HistId::tdcHitEv) tt = "Hist. fills with number of HPTDC hits from all channels per event";
        if(cfg.histId == HistId::adcHitEv) tt = "Hist. fills with number of separated ADC waveform from all channels per event";
        if(cfg.histId == HistId::chSamples) tt = "Hist. fills with every sample from ADC waveform";
        if(cfg.histId == HistId::chAmpl) tt = "Histogram fills with either maximum or minimum sample of waveform";
        if(cfg.histId == HistId::chInl) tt = "INL distribution";
        if(cfg.histId == HistId::errors) tt = "Histogram fills with error bits from HPTDC event";
        if(!tt.isEmpty()) break;
    }
    return tt;
}

bool HistMngrCfg::doTdcProcessing() const
{
    return false ||
            histCfg.value(HistId::tdcHitEv).histGroupEn ||
            histCfg.value(HistId::tdcAdcDt).histGroupEn ||
            histCfg.value(HistId::tdcCh).histGroupEn ||
            histCfg.value(HistId::tdcTime).histGroupEn ||
            histCfg.value(HistId::chInl).histGroupEn ||
            histCfg.value(HistId::tdcDoublePulseTime).histGroupEn ||
            histCfg.value(HistId::errors).histGroupEn;
}

bool HistMngrCfg::doAdcProcessing() const
{
    return false ||
            histCfg.value(HistId::adcTime).histGroupEn ||
            histCfg.value(HistId::adcCh).histGroupEn ||
            histCfg.value(HistId::tdcAdcDt).histGroupEn ||
            histCfg.value(HistId::adcHitEv).histGroupEn ||
            histCfg.value(HistId::chSamples).histGroupEn ||
            histCfg.value(HistId::chAmpl).histGroupEn;
}

bool HistId::isInlId(int id)
{
    return (chInl != -1) && (id >= chInl) && (id < chInl+nch);
}
