#ifndef HISTMNGRCFG_H
#define HISTMNGRCFG_H

#include <QVector>

struct HistId {
    static bool isInlId(int id);

    static int tdcCh;
    static int adcCh;
    static int tdcTime;
    static int tdcDoublePulseTime;
    static int adcTime;
    static int tdcAdcDt;
    static int tdcHitEv;
    static int adcHitEv;
    //static int HIST_CH_Q;
    //static int HIST_CH_SUMQ;
    static int chSamples;
    static int chAmpl;
    static int groupChAmpl;
    static int chInl;
    static int errors;
    static const int nch;
};

struct HistCfg
{
    HistCfg():
        HistCfg(false, -1, -1, "", 0, 0, 0, -1, false)
    {}

    HistCfg(bool _en, int gId, int hId, QString _title, double _nBins, double _min, double _max, int _ch=-1, bool isFittable=true):
        histGroupEn(_en),
        groupId(gId),
        histId(hId),
        ch(_ch),
        title(_title),
        nBins(_nBins),
        min(_min),
        max(_max),
        fittable(isFittable),
        showFit(false)
    {}

    void setStopCh(int stopCh, bool useStop) {
        hasStopCh = true;
        this->stopCh = stopCh;
        useStopCh = useStop;
    }
    QString getGroupTitle() const { return histGroupEn ? title : (title+" (off)"); }


    bool histGroupEn; // depends on groupId only, same for all ch
    int groupId; // id in comboBox
    int histId; // through id
//    bool histPerCh;
    int ch; // no ch selection in case ch == -1
    QString title;
    double nBins;
    double min;
    double max;

    bool fittable;
    bool showFit; // depends on ch and groupId

    bool hasStopCh;
    bool useStopCh;
    int stopCh;
};

struct HistMngrCfg
{
    HistMngrCfg() :
        curHistId(-1),
        useStopCh(false),
        stopCh(0),
        amplHistWithMax(true),
        globalEnabled(true)
    {
    }
    void init(bool isInlEnabled = false);
    void load();
    void save();
    QString getGroupToolTip(int groupId) const;
    bool doTdcProcessing() const;
    bool doAdcProcessing() const;

    QString curHistTitle;
    int curHistId;
    bool useStopCh;
    quint16 stopCh;
    bool amplHistWithMax; // true - with max; false - with minial
    bool globalEnabled;
    QString title;

    QVector<HistCfg> histCfg;
};

#endif // HISTMNGRCFG_H
