#ifndef TQDCANALYZER_H
#define TQDCANALYZER_H

#include <QObject>
#include <QByteArray>

#include "DataTypes.h"
#include "DeviceData.h"

class QSemaphore;
class QTimer;

enum {
    DATA_BLOCK_TDC = 0x0,
    DATA_BLOCK_ADC = 0x1
};

#pragma pack(push, 1)

struct __attribute__ ((packed)) DataBlockHeader
{
    DataBlockHeader()
        : dataLength(0), reserved(0), ch(0), type(0) {}
    quint16 dataLength;
    quint8 reserved;
    quint8 ch:4;
    quint8 type:4;
};

#pragma pack(pop)


class TQDCAnalyzer : public QObject
{
    Q_OBJECT
public:
    explicit TQDCAnalyzer(QSemaphore *_analyzerSem,
                          QSemaphore *_histSem,
                          QSemaphore *_writerSem,
                          QObject *parent = nullptr);

signals:
    void addLog(QString) const;
    void deviceDataUpdated(const DeviceData &data, bool writeDecodedEnabled) const;
    void deviceStatsUpdated(const AnalyzerStats &stats) const;
    void rawDataUpdated(const QStringList &data);
    void gotStopEvernt(quint32 stopEvNum);

public slots:
    void dataAccuired(QByteArray newData, bool);
    void resetStats();
    void requestRawData(uint events);
    void stopWrite();
    void startWrite(QString, bool rawData);
    void prepareForStop(quint32 evNum);

private slots:
    void read();
    void sendStat();

private:
    void sendLog(QString s) const;
    bool skipData();
    int readDevice(int upperLimitToDevLength);
    bool findNextEventHeader();
    bool readEvent();
    void flushData();
    void decodeTdcBlock(const quint32* payloadPtr);
    void decodeAdcBlock(const quint32* payloadPtr);
    void printRawTxt(uint size, QString header=QString(), int shift=0);

    QSemaphore * const analyzerSem;// free space in analyzer queue
    QSemaphore * const writerSem;// free space in writer queue (raw or decoded)
    QSemaphore * const histSem;// free space in histogram queue

    QTimer * const statTimer;
    QTimer * const readTimer;
    QByteArray data;
    int curPos;
    uint curEventStartPos;
    MpdEventHdr *mpdEventHdr;
    MpdDeviceHdr *mpdDeviceHdr;
    MpdMStreamHdr *mstreamHdr;
    AdcDataType1 *tsTAI;
    VMEDataTypes *vmeData;
    DataBlockHeader *dataBlockHdr;

    quint32 lastEventNumAtPrevFrame;
    quint32 prevEventNum;

    bool previousProceed;
    bool enabled;

//    QMap<DeviceIndex, DeviceData> deviceData;
    DeviceData deviceData;
    AnalyzerStats analyzerStats;
    CompleteEvent curCompleteEvent;
    TqdcEvent curTqdcEv;
    AdcEvent curAdcEv;
    bool signalInverted;
    bool writeDecodedEnabled;
    QStringList rawDataTxt;
    uint rawDataEvents;
    bool printBadEvent;
    bool lookForStop;
    quint32 stopEvNum;
    QMap<DeviceIndex, QMap<int, QString> > lastHptdcErr;
};


#endif // TQDCANALYZER_H
