#ifndef TQDCFILEWRITER_H
#define TQDCFILEWRITER_H

#include <QFile>
#include <QVector>
#include <QMap>
#include <QTextStream>
#include <QMetaType>

class QString;
class QSemaphore;
class DeviceData;
class QTimer;

struct WriterStatistic
{
    WriterStatistic() {
        clear();
    }

    void clear() {
        fileSize = 0;
        evWritten = 0;
        writeTimeNs = 0;
    }

    qint64 fileSize;
    quint64 evWritten;
    qint64 writeTimeNs;
};
Q_DECLARE_METATYPE(WriterStatistic)

class TqdcFileWriter : public QObject
{
    Q_OBJECT
public:
    TqdcFileWriter(QSemaphore *writerSem);
//    void write(const QMap<quint32, QVector<double> > &counters);

public slots:
    void writeRawData(const QByteArray &data, bool write);
    void writeDecodedData(const DeviceData &data, bool write);
    void startWrite(QString fileName, bool rawData);
    void stopWrite();
    void prepareForStop(quint32 evNum);

private slots:
    void sendStat();

signals:
    void fileStatisticUpdated(WriterStatistic s);
    void started(QString fileName);
    void stopped(QString fileName);
    void gotStopEvernt(quint32 stopEvNum);

private:

    QFile dataFile;
//    QTextStream textStream;
    QSemaphore * const writerSem;
    bool writeEnabled;
    bool rawDataWriter;
    quint32 stopEvNum;
    bool lookForStopEv;
    quint32 lastWrittenEvNum;
    QTimer * const statusTimer;
    WriterStatistic statistic;
};

#endif // TQDCFILEWRITER_H
