//
//    Copyright 2014 AFI Electronics
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef TQDCREGS_H
#define TQDCREGS_H

#include "mregdev/mregdevice.h"

enum { MEM_BIT_SELECT_CTRL = 1<<13 };

// Control registers
enum {
    REG_DEVICE_CTRL = mlink::MregDeviceAdcm::ADCM16_CTRL_REG,//0x40, //64
    REG_CH_DPM_KS = mlink::MregDeviceAdcm::ADCM16_CH_DPM_KS_REG, //0x4A, // 74
};

// Trigger control & statistic
enum {
    REG_TRIG_CSR = 0x100,
    REG_TRIG_SRC = 0x101,
    REG32_TRIG_TIMER_PERIOD = 0x102, // 32bit RW
    REG32_TRIG_EVENT_NUM_LOAD = 0x104, // 32bit RW
    REG_TRIG_QUEUE = 0x106, // RO
    REG_SELF_TRIG_MASK = 0x107,
    REG_TRIG_SELFTRIG_DELAY = 0x108, // RW
    REG_TRIG_BUSY_STATUS = 0x109, // RO
    REG64_TRIG_CNT_ALL = 0x110, // 64bit RO
    REG64_TRIG_CNT_CANDID = 0x114, // 64bit RO
    REG64_TRIG_CNT_ACCEPTED = 0x118, // 64bit RO
};

// Hit counter
enum {
    REG_HIT_CNT_CTRL   = 0x600, // RW, 0 - Counters lock
    REG64_HIT_CNT_TIME = 0x604, // 48-bit timestamp (8ns)
    REG32_HIT_CNT_CH1  = 0x680,
//    REG32_HIT_CNT_CH16 = 0x69E,
};

enum {
    REG_TRIG_CSR_BIT_CNT_RESET = 1,
    REG_TRIG_CSR_BIT_RUN_CNT_RESET = 2,
    REG_TRIG_CSR_BIT_CNT_LOCK = 4,
};

enum {
    REG_TRIG_SRC_BIT_TIMER = 1,
    REG_TRIG_SRC_BIT_LEMO = 2,
    REG_TRIG_SRC_BIT_TTC = 4,
};

// TDC Core Registers
enum {
    REG32_TDC_CH_EN_1 = 0x200,
    REG32_TDC_CH_EN_2 = 0x202,
    REG32_TDC_GEN_CTRL = 0x220,
    REG32_TDC_READ_CTRL = 0x222,
    REG32_TDC_MEASURE_CTRL = 0x224,
    REG32_TDC_TRIG_WIN_SETUP = 0x226
//    REG32_TDC_RC_ADJUST_SETUP = 0x228 //unused
};

// HPTDCs Status Registers
enum {
    REG_HPTDC1_STATUS_1 = 0x280,
    REG_HPTDC1_STATUS_2 = 0x281,
    REG_HPTDC1_STATUS_3 = 0x282,
    REG_HPTDC1_STATUS_4 = 0x283,

    REG_HPTDC2_STATUS_1 = 0x280,
    REG_HPTDC2_STATUS_2 = 0x281,
    REG_HPTDC2_STATUS_3 = 0x282,
    REG_HPTDC2_STATUS_4 = 0x283
};

enum {
    REG32_TDC_GEN_CTRL_BIT_STANDBY = 1
};

// ADC Core Registers
enum {
    REG_ADC_LAT = 0x300,
    REG_ADC_MATCH_WIN = 0x301,
    REG_ADC_PREWIN = 0x302,
    REG_ADC_WIN = 0x303, // Samples to readout
    REG_ADC_CH_EN = 0x304,
    REG_ADC_MODE = 0x305, // 1:0 - mode (1-norm,2-baseline)
    REG_ADC_GAIN_CRTL = 0x306, // 7:0 - for ch pairs
    REG_ADC_DAC_INVERT = 0x307, // 1-negative, 0-positive
    REG_ADC_CMP_STATE = 0x33E, // RO, Comparators State
    REG_ADC_CH_DATA_BASE = 0x3F0, // RO, ADC Channel #0 Data
};

enum {
    REG_ADC_MODE_NORM = 1,
    REG_ADC_MODE_BASELINE = 2,
};

// WR Time Emulator Registers
enum {
    REG_TAI_0 = 0x400, // tai sec 15:0
    REG_TAI_1 = 0x401, // tai sec 31:16
    REG_TAI_2 = 0x402, // tai sec 39:32
    REG_TAI_UPDATE = 0x403, // any write for update
};

// Deserializers control
enum {
    REG_DES_CTRL = 0x500,
    REG_DES_STATUS = 0x501,
    REG_DES_IDELAY_TAP_VAL = 0x503,
    REG_DES_IDELAY_LOAD_MASK = 0x504,
};

//  == Board Level Registers ==
// MCU SPI Slave
enum {
    REG_BMC_VERSION    = 0x4000,
    REG_PLL_STATUS     = 0x4001,
    REG_PLL_UNLOCK_CNT = 0x4002, // [12:0] - unlock counter
    REG_BMC_TEMP_1     = 0x4003,
    REG_BMC_TEMP_2     = 0x4004,
    REG_BMC_TEMP_3     = 0x4005,
    REG_BMC_TEMP_4     = 0x4006,
    REG_BMC_REVERSION  = 0x4007
};

// SPI communication to ADCs

enum {
    REG_ADC0_SPI_ADDR =  0x4100, // 6:0 - addr; 7-r/w bit
    REG_ADC0_SPI_WRITE = 0x4101, // 7:0 - write data
    REG_ADC0_SPI_READ =  0x4102, // 7:0 - frm adc ch 1,4,5,8
    REG_ADC1_SPI_ADDR =  0x4108,
    REG_ADC1_SPI_WRITE = 0x4109,
    REG_ADC1_SPI_READ =  0x410A,
};

// SPI control under DAC AD5308/AD5318/AD5328
enum {
    REG_DAC1_CTRL = 0x4200, // for channels 0..7
    REG_DAC2_CTRL = 0x4201 // for channels 8..15
};

enum {
    DAC_CMD_RESET = 0xF000, // for channels 0..7
    DAC_CMD_SINGLE_UPDATE = 0xA002 // for channels 8..15
};

enum {
    REG_DEVICE_CTRL_BIT_RUN = 0x8000
};

enum {
    ADC_CLK_CTRL_BIT_INC            = 0x8000,
    ADC_CLK_CTRL_BIT_ADC_RESET      = 0x4000,
    ADC_CLK_CTRL_BIT_STATUS_RESET   = 0x2000,
    ADC_CLK_CTRL_BIT_RESET          = 0x0100,
    ADC_CLK_CTRL_ALL_ADCS           = 0x00FF
};

//enum {
//    ADC_SPI_PATTERN_LOW_REG   = 0x10,
//    ADC_SPI_PATTERN_HIGH_REG  = 0x12,
//    ADC_SPI_DECIMATOR_CTRL_REG = 0x16
//};

// AD9252
enum {AD9252_CHIP_ID = 0x09};
enum {
    AD9252_SPI_12BITS_MODE = 0x3
};

enum {
    AD9252_SPI_DEVICE_UPDATE_BIT = 0x1
};

enum {
    AD9252_SPI_RESET = 0x3C,
    AD9252_SPI_RUN = 0x18
};


enum {
    AD9252_SPI_CHIP_PORT_CONFIG_REG = 0x0,
    AD9252_SPI_CHIP_ID_REG = 0x1,
    AD9252_SPI_CHIP_GRADE_REG = 0x2,
    AD9252_SPI_SERIAL_CONTROL_REG = 0x21,
    AD9252_SPI_DEVICE_UPDATE_REG = 0xFF
};

enum {AD5622_WRITE_COMMAND = 0x180};

// LTM9011
enum LTM9011_ADC_SPI_REG {
    LTM9011_ADC_SPI_A0_REG  = 0, // reset
    LTM9011_ADC_SPI_A1_REG  = 1, // format and power-down
    LTM9011_ADC_SPI_A2_REG  = 2, // output mode
    LTM9011_ADC_SPI_A3_REG  = 3, // test pattern msb
    LTM9011_ADC_SPI_A4_REG  = 4, // test pattern lsb
};

#endif // TQDCREGS_H
