#include <QElapsedTimer>
#include <QProcess>
#include <QThread>

#include "DeviceGlob.h"
#include "base/threadQuit.h"

DeviceGlob::DeviceGlob(QObject *parent) :
    QObject(parent),
    dev(nullptr),
    devThread(new QThread(this)),
    mstreamProc(new QProcess(this))
{
}

DeviceGlob::DeviceGlob(DeviceIndex i, QObject *parent):
    DeviceGlob(parent)
{
    connectToDev(i);
}

DeviceGlob::~DeviceGlob()
{
    if(dev) {
        disconnectFromDev();
    }
    mstreamProc->blockSignals(true);
    mstreamProc->close();
    if(mstreamProc->state()==QProcess::Running && !mstreamProc->waitForFinished(10000)) {
        mstreamProc->terminate();
    }
    delete mstreamProc;
}

void DeviceGlob::connectToDev(DeviceIndex i)
{
    if(dev)
        disconnectFromDev();
    index = i;
    dev = new TtvxsDevice(index);
    dev->moveToThread(&devThread);
    devThread.setObjectName(QString("Dev. %1").arg(i.getSerialStr()));
    connect(dev, &QObject::destroyed, &devThread, &QThread::quit);
    connect(this, &DeviceGlob::deleteHardware, dev, &QtMregDevice::deleteHardware);
}

void DeviceGlob::disconnectFromDev(QElapsedTimer *t)
{
    emit deleteHardware(index);
    bool localTimer = t;
    if(localTimer) {
        t = new QElapsedTimer();
        t->start();
    }

    if(!threadQuiting(&devThread, 5000, t)) {
        qWarning() << "Failed to stop thread for dev:" << index.getIdent();
    }
    dev = nullptr;
    if(localTimer)
        delete t;
}

void DeviceGlob::startThread()
{
    devThread.start();
}
