#ifndef TTVXSDEVICE_H
#define TTVXSDEVICE_H

#include <QObject>
#include <QMetaType>

#include "base/bmc/BmcStatus.h"
#include "TtvxsAuxCounters.h"
#include "TtvxsConfig.h"

#include "mregdev/qtmregdevice.h"
#include "mregdev/WrEpStatus.h"
#include "mldiscover/DeviceIndex.h"
#include "regio/regop.h"

class HwipErrorCounters
{
public:
    HwipErrorCounters() {
        clear();
    }
    void clear() {
        hwipLinkDown=0;
        regIoError=0;
        sfpErrorEth=0;
        hwipError=0;
        txErrorEth=0;
        rxErrorEth=0;
    }

    quint64 hwipLinkDown;
    quint64 regIoError;
    quint64 sfpErrorEth;
    quint64 hwipError;
    quint64 txErrorEth;
    quint64 rxErrorEth;
};



class TtvxsStatus
{
public:
    TtvxsStatus() :
        isConnected(false),
        liveMagikOk(false),
        trigState(0),
        timeoutState(0),
        curEvNum(0),
        inputState(0),
        mstreamMultiAck(false),
        mstreamHwBufSize(1)
    {
        memset(buildParams, 0, 4*sizeof (quint32));
        hwipErrors.clear();
        temp = 0;
    }

    DeviceIndex devIndex;
    bool isConnected;
    bool liveMagikOk;
    quint16 trigState;
    quint32 buildParams[4];
    quint32 timeoutState;
    quint64 curEvNum;
    quint16 trigQueue;
    quint32 inputState;
    HwipErrorCounters hwipErrors;
    QString fwStr;
    bool mstreamMultiAck;
    quint8 mstreamHwBufSize;
    double temp;

    TtvxsAuxCounters aux_cnt;
    BmcStatus bmc;
    WrEpStatus wrEpStatus;
};
Q_DECLARE_METATYPE(TtvxsStatus)

class TtvxsDevice : public QtMregDevice
{
    Q_OBJECT
public:
    explicit TtvxsDevice(DeviceIndex devIndex, QObject *parent=nullptr);

signals:
    void statusUpdated(const TtvxsStatus &status);
    void trigEvNumResponse(int check_index, quint64 evNum);
    void devEvNumResponse(int check_index, quint64 devSerial, quint64 evNum);

public slots:
    void writeSetup();
    void setSetup(const TtvxsConfig &cfg);
    void evNumReset(DeviceIndex index=DeviceIndex());
    void onTempChanged(double temp);
    void onRestartAll(QString,int);
    void softClear(DeviceIndex i);
    void clearStat(DeviceIndex i);
    void evNumRequest(int check_index, bool top_level);
    void sendResetTimestamp(DeviceIndex i=DeviceIndex());

protected slots:
    virtual void statusPoll();
    virtual void on_deviceConnected();
    virtual void on_deviceDisconnected();

private:
    mlink::RegOpVector writeStatCfg(const TtvxsConfig &cfg);

    TtvxsConfig configCashe;
    TtvxsStatus status;
    bool configValid;
};

#endif // TTVXSDEVICE_H
