/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "AboutBox.h"

#include <QCoreApplication>
#include <QDateTime>
#include <QMessageBox>

#include "base/base/version.h"

void AboutBox::about(QWidget *parent)
{
    auto date = QDateTime::fromSecsSinceEpoch(GIT_COMMIT_TIMESTAMP);
    QString s = "<h1>"+qApp->applicationName()+"</h1>";
    s += "<h3>" + tr("DAQ Component") + "</h3>";
    QString descr = GIT_DESCR;
    if (! descr.isEmpty())
        s += "<p>" + tr("Version") + " <b>" + descr + "</b>";
    if (GIT_COMMIT_TIMESTAMP != 0)
        s += "<p>" + tr("Released on") + " " + date.toString(Qt::DefaultLocaleShortDate) +"</p>";
    s += "<p>Copyright &copy; 2014-2020";
    s += "<br>"+tr("License") + ": <a href=\"https://www.gnu.org/licenses/gpl-3.0.html\">GPLv3</a>";
    s += "<p>" + tr("Authors");
    s += "<ul>";
    s += "<li>" + tr("Ivan Filippov") + "</li>";
    s += "<li>" + tr("Timur Gandzhelashvili") + "</li>";
    s += "<li>" + tr("Shchipunov Andrey") + "</li>";
    s += "<li>" + tr("Alexey Shutov") + "</li>";
    s += "<li>" + tr("Ilia Slepnev") + "</li>";
    s += "</ul>";
    s += "<p><a href=\"https://afi.jinr.ru/\">AFI Electronics</a>";
    s += "<br><a href=\"https://www.jinr.ru/\">" + tr("Joint Institute for Nuclear Research") + "</a>";
    s += "<p>" +tr("Code")+ ": <a href=\"https://afi-git.jinr.ru/soft/afi-daq/\">https://afi-git.jinr.ru/soft/afi-daq</a>";
    QMessageBox::about (parent, "About", s);
}
