/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "BaseMainWindow.h"

#include <QSettings>
#include <QCloseEvent>
#include <QMenuBar>

#include "AboutBox.h"

BaseMainWindow::BaseMainWindow(QWidget *parent) :
    QMainWindow(parent)
{
    // setupMenu();
}

BaseMainWindow::~BaseMainWindow()
{
}

void BaseMainWindow::closeEvent(QCloseEvent *event)
{
    QSettings settings;
    settings.beginGroup("MainWindow");
    settings.setValue("geometry", saveGeometry());
    settings.setValue("windowState", saveState());
    settings.endGroup();
    event->accept();
}

void BaseMainWindow::setupMenu()
{
    auto fileMenu = menuBar()->addMenu(tr("&File"));
    QIcon iconQuit = QIcon::fromTheme("application-exit");
    fileMenu->addAction(iconQuit,
                        tr("E&xit"),
                        this,
                        SLOT(actionQuit()),
                        QKeySequence::Quit);

    QIcon iconAbout = QIcon::fromTheme("help-about");
    auto helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(iconAbout, tr("About..."), this, SLOT(actionAbout()));
    menuBar()->setVisible(true);
}

void BaseMainWindow::actionQuit()
{
    close();
}

void BaseMainWindow::actionAbout()
{
    AboutBox::about();
}

