/*
**    Reverse DNS lookup (get PTR record)
**
**    Copyright 2016-2020 Ivan Filippov
**    Copyright 2016-2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DNSLOOKUP_H
#define DNSLOOKUP_H

#include <QMap>
#include <QPair>
#include <QObject>
#include <QHostInfo>

typedef QPair<quint32, QString> DnsReverseEntry;
typedef QMap<quint32, QString> DnsReverseMap;

class DnsLookup : public QObject
{
    Q_OBJECT
public:
    explicit DnsLookup(QObject *parent = nullptr);
    QMap<quint32, QString> getHostMap() const { return hostMap; }
    void reverseLookup(const QString &name);

signals:
    void reverseLookupComplete(const QHostInfo &);
    void reverseEntryUpdated(const DnsReverseEntry &);
    void reverseMapUpdated(const DnsReverseMap &);

private slots:
    void reverseLookupCallback(const QHostInfo &h);
    void gotHostInfo(QHostInfo h);

private:
    DnsReverseMap hostMap;
    const Qt::HANDLE mainThreadId;
};

Q_DECLARE_METATYPE(DnsReverseMap)

#endif // DNSLOOKUP_H
