//
//    Copyright 2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef SYSLOGMESSAGESENDER_H
#define SYSLOGMESSAGESENDER_H

#include <QtCore/QtGlobal>
#include "LogPriority.h"
#include <QObject>
#include <QMetaType>
#include <QMap>
#include <QString>

class SyslogMessageSender: public QObject
{
    Q_OBJECT
    SyslogMessageSender() : QObject(), prevMsgHandler(nullptr) {}
public:
    ~SyslogMessageSender();
    void install();
    void remove();

#if QT_VERSION < 0x050000
    static void sendSyslogMessage(QtMsgType type, const char *text);
#else
    static void sendSyslogMessage(QtMsgType type, const QMessageLogContext &context, const QString &text);
#endif

    static void log(int priority, QString message);
    static SyslogMessageSender & instance()
    {
        static SyslogMessageSender obj;
        obj.setObjectName("SyslogMessageSender");
        return obj;
    }

signals:
    void newMsg(QtMsgType type, const QString &msg, const QString &threadName);

private:
    static QString trimmedText(const QString &text);
    static QMap<QString, int> syslog_prio_map;
#if QT_VERSION < 0x050000
    QtMsgHandler prevMsgHandler;
#else
    QtMessageHandler prevMsgHandler;
#endif

};

#endif // SYSLOGMESSAGESENDER_H
