/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "ThreadController.h"

#include <QDebug>
#include <QThread>

enum { USE_THREADS = 0 };

ThreadController::ThreadController(QObject *_obj, QObject *parent)
    : QObject(parent),
    obj(_obj)
{
    if (USE_THREADS) {
        objThread = new QThread(this);
        objThread->setObjectName(obj->objectName());
        connect(objThread, &QThread::finished, obj,
                &QObject::deleteLater, Qt::DirectConnection); // deleteLater is thread-safe
        connect(obj, &QObject::destroyed,
                objThread, &QThread::quit);
        objThread->start(QThread::LowPriority);
        obj->setParent(nullptr);
        obj->moveToThread(objThread);
    } else {
        obj->setParent(this);
    }
}

ThreadController::~ThreadController()
{
    if (!obj)
        return;

    if (!objThread || objThread == thread()) {
        delete obj;
        return;
    }
    objThread->quit();

    const quint64 sleepMs = 10;
    const qint64 timeoutMs = 20000;
    for (quint64 i=0; i < timeoutMs / sleepMs; i++) {
        if (objThread->isFinished())
            break;
        thread()->msleep(sleepMs);
    }
    if (!objThread->isFinished()) {
        qCritical() << QString("Thread '%1' is still running").arg(objThread->objectName());
        objThread->terminate();
        objThread->wait();
        qCritical() << QString("Thread '%1' terminated").arg(objThread->objectName());
    } else {
        qDebug() << QString("Thread '%1' finished").arg(objThread->objectName());
    }
}
