#ifndef BMCSTATUS_H
#define BMCSTATUS_H

#include <QColor>
#include <QDebug>
#include <QMap>
#include <QString>

#include "fpga_mcu_regs_v2_0.h"

struct BmcPower {
    BmcPower()
    {
        clear();
    }
    void clear() {
        vol.clear();
        cur.clear();
        setupInProgress=false;
    }
    static QString getIdStr(quint16 addr_or_id);
    QString getSummary(bool withCurrent = true) const;
    bool isCriticalCurrent() const;

    QMap<quint16, quint16> vol;
    QMap<quint16, quint16> cur;
    bool setupInProgress;
};

struct BmcStatus {
    BmcStatus()
    {
        clear();
    }
    void clear() {
        dev_id = 0;
        onboard_pll_status.raw = 0;
        pllUnlockCnt = 0;
        mcuVer.b.major = 0;
        mcuVer.b.minor = 0;
        mcuRev = 0;
        overall_status = 0;
        power_status = 0;
        pll_status = 0;
        temp.clear();
        power.clear();
    }
    QColor get_overall_status_col() const {
        switch (overall_status) {
        case 0b00: return Qt::gray;
        case 0b01: return Qt::green;
        case 0b10: return Qt::yellow;
        case 0b11: return Qt::red;
        }
        qCritical() << QString("illegal mcu overall_status: 0x%1").arg(overall_status);
        return QString();
    }
    QString get_overall_status_str() const { return get_status_str(overall_status); }
    QString get_power_status_str() const { return get_status_str(power_status); }
    QString get_pll_status_str() const { return get_status_str(pll_status); }
    inline bool onboard_pll_ok() const {
        return onboard_pll_status.b.locked; // inverted in TDC BMC before 2.2-94
    }
    QString getVerStr() const {
        return QString("%1.%2-%3")
            .arg(mcuVer.b.major)
            .arg(mcuVer.b.minor)
            .arg(mcuRev);
    }
    static QString get_status_str(quint16 st) {
        switch (st) {
        case 0b00: return "unknown";
        case 0b01: return "normal";
        case 0b10: return "warning";
        case 0b11: return "critical";
        }
        return QString();
    }

    quint16 dev_id;
    fpga_mcu_reg_pll_t onboard_pll_status;
    quint16 pllUnlockCnt;
    bmc_version_t mcuVer;
    quint16 mcuRev;
    quint16 overall_status;
    quint16 power_status;
    quint16 pll_status;

    QList<double> temp;
    BmcPower power;
};

#endif // BMCSTATUS_H
