#ifndef DISCOVERDIALOG_H
#define DISCOVERDIALOG_H

#include <QDialog>
#include <QMap>
#include <QSet>
#include <QTime>

#include "base/DnsLookup.h"
#include "mldiscover/DeviceIndex.h"
#include "mldiscover/DeviceDescription.h"

class MldpListener;
class QButtonGroup;
class QTableWidget;

namespace Ui {
class DiscoverDialog;
}

class DiscoverDialog : public QDialog
{
    Q_OBJECT

public:
    explicit DiscoverDialog(QWidget *parent = nullptr);
    ~DiscoverDialog();
    QVector<DeviceDescription> getSelectedDevices() const { return selectedDevices.values().toVector(); }
    QVector<int> getSelectedRows(bool free) const;
    void setDeviceIdFilter(QSet<quint16> devices) { deviceIdFilter = devices; updateShownDevices(); }
    void setDeviceIdFilter(int deviceId) { deviceIdFilter.clear(); addDeviceIdFilter(deviceId); updateShownDevices(); }
    void addDeviceIdFilter(int deviceId) { deviceIdFilter.insert(deviceId); updateShownDevices(); }
    void setSelected(const DeviceDescription &dd);
    void setSelected(QVector<DeviceDescription> list);
    void addSelected(const DeviceDescription &dd);
    void setPrintInfoFull(bool full=true) { printInfoFull = full; }
    void setSingleMode(bool mode=true) { useCheckBoxes = !mode; }
    void show_mac_adresses();
    DeviceDescription getDescriptionByIndex(DeviceIndex index) const;

public slots:
#ifndef QT_NO_DEBUG
    virtual void done(int r) { printInfo = false; QDialog::done(r); }
    virtual void setVisible(bool visible) { printInfo = visible; QDialog::setVisible(visible); }
#endif
    virtual void accept();
    virtual void reject();

private slots:
    void descriptionUpdated(const DeviceDescription &dd);
    void checkOutOfDate();
    void setDeviceEnabled(bool);

    void on_pushButtonTest_clicked();
    void on_pushButtonAddDiscoverDev_clicked();
    void on_pushButtonRemoveDiscoverDev_clicked();
    void on_listWidgetTypes_itemSelectionChanged();
    void on_listWidgetSubnet_itemSelectionChanged();
    void on_pushButtonManualAdd_clicked();
    void tableDoubleClick(int row, int col);
    void resizeTables();
    void checkHostInfoUpdates();

    void on_checkBoxShowMAC_clicked(bool checked);

signals:
    void deviceDiscovered(const DeviceDescription &dd);
    void deviceDiscoveredAll(const DeviceDescription &dd);
    void deviceExpired(const DeviceIndex &i);
private:
    void printCols(bool upper, int col) const;
    void insertDev(DeviceDescription dd);
    void insertRow(bool free, const DeviceDescription dd);
    void removeRow(bool free, DeviceIndex index);
    void moveRows(bool downwards);
    void moveRow(bool downwards, DeviceIndex index);
    void setOnlineState(int row, bool isOnline, DeviceIndex index);

    void updateTableRow(bool free, const DeviceDescription dd, int row=-1);
    void setTableHeaders(bool upper);
    DeviceIndex getDeviceIndexOfRow(bool upper, int row) const;
    int getRowByIndex(bool upper, DeviceIndex index) const;
    void checkRows() const;
    bool checkDeviceIdFilter(quint16 deviceId) const;
    void updateShownDevices();
    void updateDeviceTypeInfo(const DeviceDescription &dd);
    void updateHostInfo(QTableWidget *table);
    void updateSubnetList(const DeviceDescription &dd);
    bool isInShownSubnet(const DeviceDescription &dd) const;
    void addSubnet(quint32 subnet, QString name=QString());
    void deviceTypeMapUpdated(quint16 id);
    bool isDevShown(bool upper, const DeviceDescription &dd) const;
    void updateLocalIp();
    QMap<quint32, QString> getHostMap() const { return dnsLookup.getHostMap(); }

    Ui::DiscoverDialog *ui;
    QMap<DeviceIndex, DeviceDescription> availableDevices;
    QMap<DeviceIndex, DeviceDescription> selectedDevices;
    QMap<DeviceIndex, DeviceDescription> selectedDevices_backup;

    MldpListener const * mldpListener;

    int count;
    QSet<quint16> deviceIdFilter;
    enum {deviceIdAnyDevice = 0xFFFF};
    QTimer * const outOfDateTimer;
    QTimer * const resizeTimer;
    bool useCheckBoxes;
    QMap<QObject*, DeviceIndex> widgetMap;
    QMap<DeviceIndex, bool> devicePreviousState;
    bool printInfo;
    bool printInfoFull;
    QSet<quint16> deviceIdShown;
    QSet<quint32> subnetShown;
    QMultiMap<quint16, QString> deviceTypeMap;
    QSet<quint32> knownSubnets;
    QButtonGroup * const buttonGroupEnabled;
    QSet<quint32> localIp;
    bool show_mac_adress;
    DnsLookup dnsLookup;
};

#endif // DISCOVERDIALOG_H
