#include "ManualAddDialog.h"
#include "ui_ManualAddDialog.h"
#include <QRegExpValidator>
#include <QDebug>
#include <QDialogButtonBox>
#include <QPushButton>

ManualAddDialog::ManualAddDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ManualAddDialog)
{
    ui->setupUi(this);
    QRegExp rx("[0-9a-fA-F]{1,12}");
    QRegExpValidator *rxVal = new QRegExpValidator(rx, this);
    ui->lineEditSerial->setValidator(rxVal);

    QRegExp rx2("[0-9a-fA-F]{1,2}");
    QRegExpValidator *rxVal2 = new QRegExpValidator(rx2, this);
    ui->comboBoxDeviceId->setValidator(rxVal2);

    connect(ui->lineEditSerial, &QLineEdit::textEdited,
            this, &ManualAddDialog::validateData);
    connect(ui->comboBoxDeviceId, &QComboBox::currentTextChanged,
            this, &ManualAddDialog::validateData);
    validateData();
}

quint64 ManualAddDialog::getSerial() const
{
    const auto text = ui->lineEditSerial->text();
    bool ok;
    quint64 serial = text.toULongLong(&ok);
    if(ok) return  serial;
    serial = text.toULongLong(&ok, 16);
    return ok ? serial : 0;
}

quint16 ManualAddDialog::getDeviceId() const
{
    const QString txt = ui->comboBoxDeviceId->currentText();
    for (quint16 id : deviceTypeMap.keys()) {
        if(deviceTypeMap[id] == txt)
            return id;
    }

    bool ok;
    quint16 deviceId = static_cast<quint16>(txt.toUInt(&ok));
    if(ok) return deviceId;
    deviceId = static_cast<quint16>(txt.toUInt(&ok, 16));
    return ok ? deviceId : 0;
}

void ManualAddDialog::setDeviceIdMap(QMap<quint16, QString> deviceTypeMap)
{
    this->deviceTypeMap = deviceTypeMap;
    ui->comboBoxDeviceId->clear();
    for (const quint16 id : deviceTypeMap.keys()) {
        ui->comboBoxDeviceId->insertItem(0, deviceTypeMap[id], id);
    }
}

void ManualAddDialog::validateData() const
{
    ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(getDeviceId() && getSerial());
}

ManualAddDialog::~ManualAddDialog()
{
    delete ui;
}
