#include "StatisticOutput.h"
#include "ui_StatisticOutput.h"

#include "mldiscover/DeviceIndex.h"
#include "mregdev/qtmregdevice.h"

namespace {
enum{
    COL_TYPE,                  COL_SERIAL,
    COL_PKG_COUNT,             COL_PKG_RATE,          COL_RECV_DATA_SIZE,
    COL_RECV_DATA_RATE,        COL_FRAG_COUNT,        COL_RETRANS_FRAG_PARTS,
    COL_FRAG_TOTAL,            COL_FRAG_CLOSED,       COL_FRAG_DROPED,
    COL_FRAG_MISSED,           COL_FRAG_INCOMPLETED,  COL_GOOD_FORMAT_PKG,
    COL_BAD_FORMAT_PKG,        COL_MISSED_EVENTS,     COL_MISSED_EVENTS_RANGES,
    COL_INCOMPLETE_EVENTS,     COL_LAST_EVENT_NUM,    COL_EVENT_COUNT,
    COL_EVENT_RATE,            COL_SLOW_CONN_PERCENT,

    COL_NO_FREE,               COL_DIDNT_HELP,        COL_FREE,
    COL_CHECK_FAIL,            COL_NO_DATA,

    COL_TOTAL
};
}

StatisticOutput::StatisticOutput(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::StatisticOutput),
    show_all(false),
    run(false)
{
    qRegisterMetaType<MStreamStat>();
    ui->setupUi(this);

    init_table();
}

StatisticOutput::~StatisticOutput()
{
    delete ui;
}

void StatisticOutput::init_table()
{
    auto table = ui->tableWidget;
    table->setColumnCount(COL_TOTAL);
    table->setHorizontalHeaderItem(COL_TYPE, new QTableWidgetItem("Type"));
    table->setHorizontalHeaderItem(COL_SERIAL, new QTableWidgetItem("Serial"));
    table->setHorizontalHeaderItem(COL_PKG_COUNT, new QTableWidgetItem("Package count"));
    table->setHorizontalHeaderItem(COL_PKG_RATE, new QTableWidgetItem("Package rate"));
    table->setHorizontalHeaderItem(COL_RECV_DATA_SIZE, new QTableWidgetItem("Receieve data size"));
    table->setHorizontalHeaderItem(COL_RECV_DATA_RATE, new QTableWidgetItem("Receieve data rate (MB/s)"));
    table->setHorizontalHeaderItem(COL_FRAG_COUNT, new QTableWidgetItem("Fragment count"));
    table->setHorizontalHeaderItem(COL_RETRANS_FRAG_PARTS, new QTableWidgetItem("Retransmited frag parts"));
    table->setHorizontalHeaderItem(COL_FRAG_TOTAL, new QTableWidgetItem("Fragment total"));
    table->setHorizontalHeaderItem(COL_FRAG_CLOSED, new QTableWidgetItem("Fragment closed"));
    table->setHorizontalHeaderItem(COL_FRAG_DROPED, new QTableWidgetItem("Fragment dropped"));
    table->setHorizontalHeaderItem(COL_FRAG_MISSED, new QTableWidgetItem("Fragment missed"));
    table->setHorizontalHeaderItem(COL_FRAG_INCOMPLETED, new QTableWidgetItem("Fragment incompleted"));
    table->setHorizontalHeaderItem(COL_GOOD_FORMAT_PKG, new QTableWidgetItem("Good format package"));
    table->setHorizontalHeaderItem(COL_BAD_FORMAT_PKG, new QTableWidgetItem("Bad format package"));
    table->setHorizontalHeaderItem(COL_MISSED_EVENTS, new QTableWidgetItem("Missed events"));
    table->setHorizontalHeaderItem(COL_MISSED_EVENTS_RANGES, new QTableWidgetItem("Missed events ranges"));
    table->setHorizontalHeaderItem(COL_INCOMPLETE_EVENTS, new QTableWidgetItem("Incomplete events"));
    table->setHorizontalHeaderItem(COL_LAST_EVENT_NUM, new QTableWidgetItem("Last event number"));
    table->setHorizontalHeaderItem(COL_EVENT_COUNT, new QTableWidgetItem("Event count"));
    table->setHorizontalHeaderItem(COL_EVENT_RATE, new QTableWidgetItem("Event rate"));
    table->setHorizontalHeaderItem(COL_SLOW_CONN_PERCENT, new QTableWidgetItem("Slow connection percent"));
    table->setHorizontalHeaderItem(COL_NO_FREE, new QTableWidgetItem("No free"));
    table->setHorizontalHeaderItem(COL_DIDNT_HELP, new QTableWidgetItem("Didnt help"));
    table->setHorizontalHeaderItem(COL_FREE, new QTableWidgetItem("Free"));
    table->setHorizontalHeaderItem(COL_CHECK_FAIL, new QTableWidgetItem("Check fail"));
    table->setHorizontalHeaderItem(COL_NO_DATA, new QTableWidgetItem("No data"));

    update_column_visible();

    table->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

void StatisticOutput::add_device(DeviceIndex index)
{
    if(devices.contains(index))
        return;

    devices << index;

    int row = ui->tableWidget->rowCount();
    ui->tableWidget->setRowCount(row+1);
    for(int col=0; col< COL_TOTAL; ++col) {
        ui->tableWidget->setItem(row, col, new QTableWidgetItem());
        Qt::ItemFlags flags = ui->tableWidget->item(row, col)->flags();
        flags &= ~Qt::ItemIsEditable;
        ui->tableWidget->item(row, col)->setFlags(flags);
    }
    ui->tableWidget->item(row, COL_TYPE)->setText(QtMregDevice::getDeviceTypeName(index.getDevId()));
    auto itemSerial = ui->tableWidget->item(row, COL_SERIAL);
    itemSerial->setText(index.getSerialStr());

    itemSerial->setData(Qt::UserRole, QList<QVariant>()
                        << index.getDevId() << index.getSerial());
}

void StatisticOutput::remove_device(DeviceIndex index)
{
    if(!devices.contains(index))
        return;
    int row = index_to_row(index);
    if(row != -1)
        ui->tableWidget->removeRow(row);
    devices.remove(index);
    prev_ms_stat.remove(index);
}

void StatisticOutput::remove_all_devices()
{
    ui->tableWidget->setRowCount(0);
    devices.clear();
    prev_ms_stat.clear();
}

void StatisticOutput::stat_updated(const MStreamStat &ms_stat)
{
    qint64 run_time = run ? ms_stat.upTime.elapsed() : 0;

    double slow_connection_percent = 0;
    if(run_time != 0) {
        slow_connection_percent = (ms_stat.tcpStatus.slow_connection_duration * 100)/run_time;
    }

    int row = index_to_row(ms_stat.index);
    if(row == -1) return;

    double package_rate=0;
    double receive_data_rate=0;
    double event_rate=0;

    auto prev_stat_it = prev_ms_stat.find(ms_stat.index);
    if(prev_stat_it != prev_ms_stat.end()) {
        qint64 milisecs = 0;
        const auto &prev_stat = prev_stat_it.value();
        milisecs = prev_stat.timestamp.msecsTo(ms_stat.timestamp);
        if(milisecs != 0) {
            package_rate = 1000.*(ms_stat.packageCount - prev_stat.packageCount )/ milisecs;
            receive_data_rate = 1000.*(ms_stat.recvDataSize - prev_stat.recvDataSize)/(milisecs*1000000);
            event_rate = 1000.*(ms_stat.evCount - prev_stat.evCount)/milisecs;
        }
    }

    auto table = ui->tableWidget;
    bool updatesEnabled = table->updatesEnabled();
    if(updatesEnabled)
        table->setUpdatesEnabled(false);

    set_table_text(row, COL_PKG_COUNT, ms_stat.packageCount);
    set_table_text(row, COL_PKG_COUNT, ms_stat.packageCount);
    set_table_text(row, COL_PKG_RATE, package_rate);
//    set_table_text(row, COL_PKG_RATE)->setText(QString("%1").arg(package_rate,0,'f',0);
    set_table_text(row, COL_RECV_DATA_SIZE, ms_stat.recvDataSize);
    set_table_text(row, COL_RECV_DATA_RATE, receive_data_rate);
    set_table_text(row, COL_FRAG_COUNT, ms_stat.fragmentCount);
    set_table_text(row, COL_RETRANS_FRAG_PARTS, ms_stat.retransmitedFragParts);
    set_table_text(row, COL_FRAG_TOTAL, ms_stat.fragTotal);
    set_table_text(row, COL_FRAG_CLOSED, ms_stat.fragClosed);
    set_table_text(row, COL_GOOD_FORMAT_PKG, ms_stat.goodFormatPackage);
    set_table_text(row, COL_MISSED_EVENTS, ms_stat.missedEvents);
    set_table_text(row, COL_LAST_EVENT_NUM, ms_stat.lastEvNum);
    set_table_text(row, COL_EVENT_COUNT, ms_stat.evCount);
    set_table_text(row, COL_EVENT_RATE, event_rate);

    set_table_text(row, COL_SLOW_CONN_PERCENT, slow_connection_percent);
    if((slow_connection_percent >= 1) || (ms_stat.tcpStatus.slow_connection_duration >= 10000)) {
        table->showColumn(COL_SLOW_CONN_PERCENT);
        table->item(row, COL_SLOW_CONN_PERCENT)->setBackgroundColor(QColor(248, 86, 27));
    }

    set_table_text(row, COL_FRAG_DROPED, ms_stat.fragDroped);
    if(ms_stat.fragDroped != 0)
        table->showColumn(COL_FRAG_DROPED);
    set_table_text(row, COL_FRAG_MISSED, ms_stat.fragMissed);
    if(ms_stat.fragMissed != 0)
        table->showColumn(COL_FRAG_MISSED);
    set_table_text(row, COL_FRAG_INCOMPLETED, ms_stat.fragIncompled);
    if(ms_stat.fragIncompled != 0)
        table->showColumn(COL_FRAG_INCOMPLETED);
    set_table_text(row, COL_BAD_FORMAT_PKG, ms_stat.badFormatPackage);
    if(ms_stat.badFormatPackage != 0)
        table->showColumn(COL_BAD_FORMAT_PKG);
    set_table_text(row, COL_MISSED_EVENTS_RANGES, ms_stat.missedEventsRanges);
    if(ms_stat.missedEventsRanges != 0)
        table->showColumn(COL_MISSED_EVENTS_RANGES);
    set_table_text(row, COL_INCOMPLETE_EVENTS, ms_stat.incompleteEv);
    if(ms_stat.incompleteEv != 0)
        table->showColumn(COL_INCOMPLETE_EVENTS);

    if(updatesEnabled)
        table->setUpdatesEnabled(true);

    if(prev_stat_it == prev_ms_stat.end())
        prev_ms_stat[ms_stat.index] = ms_stat;
    else
        prev_stat_it.value() = ms_stat;
}

void StatisticOutput::stat_from_receiver_updated(ReceiverStat stat)
{
    int row = index_to_row(stat.index);
    if(row == -1) return;

    bool updatesEnabled = ui->tableWidget->updatesEnabled();
    if(updatesEnabled)
        ui->tableWidget->setUpdatesEnabled(false);

    ui->tableWidget->item(row, COL_NO_FREE)->setText(QString("%L1").arg(stat.noFree));
    ui->tableWidget->item(row, COL_DIDNT_HELP)->setText(QString("%L1").arg(stat.didntHelp));
    ui->tableWidget->item(row, COL_FREE)->setText(QString("%L1").arg(stat.free));
    ui->tableWidget->item(row, COL_CHECK_FAIL)->setText(QString("%L1").arg(stat.checkFail));
    ui->tableWidget->item(row, COL_NO_DATA)->setText(QString("%L1").arg(stat.noData));

    if(updatesEnabled)
        ui->tableWidget->setUpdatesEnabled(true);
}

int StatisticOutput::index_to_row(DeviceIndex index) const
{
    for(int row=0; row<ui->tableWidget->rowCount(); ++row) {
        if(row_to_index(row) == index)
            return row;
    }
    return -1;
}

DeviceIndex StatisticOutput::row_to_index(int row) const
{
    auto item = ui->tableWidget->item(row, COL_SERIAL);
    if(item){
        auto data_list = item->data(Qt::UserRole).toList();
        if(data_list.size()>1) {
            bool ok1,ok2;
            DeviceIndex i(static_cast<quint16>(data_list.first().toUInt(&ok1)),
                          data_list[1].toULongLong(&ok2));
            if(ok1 && ok2)
                return i;
        }
    }
    return DeviceIndex();
}

void StatisticOutput::update_column_visible() const
{
    auto table = ui->tableWidget;
    const auto hidden = !show_all;
    //todo: mask hidden with 'is columns got err'
    table->setColumnHidden(COL_SLOW_CONN_PERCENT, hidden);
    table->setColumnHidden(COL_FRAG_COUNT, hidden);
    table->setColumnHidden(COL_FRAG_TOTAL, hidden);
    table->setColumnHidden(COL_FRAG_CLOSED, hidden);
    table->setColumnHidden(COL_GOOD_FORMAT_PKG, hidden);
    table->setColumnHidden(COL_MISSED_EVENTS, hidden);

    table->setColumnHidden(COL_FRAG_DROPED, hidden);
    table->setColumnHidden(COL_FRAG_MISSED, hidden);
    table->setColumnHidden(COL_FRAG_INCOMPLETED, hidden);
    table->setColumnHidden(COL_BAD_FORMAT_PKG, hidden);
    table->setColumnHidden(COL_MISSED_EVENTS_RANGES, hidden);
    table->setColumnHidden(COL_INCOMPLETE_EVENTS, hidden);

    table->setColumnHidden(COL_NO_FREE, hidden);
    table->setColumnHidden(COL_DIDNT_HELP, hidden);
    table->setColumnHidden(COL_FREE, hidden);
    table->setColumnHidden(COL_CHECK_FAIL, hidden);
    table->setColumnHidden(COL_NO_DATA, hidden);
}

void StatisticOutput::set_table_text(int row, int col, quint64 val)
{
    ui->tableWidget->item(row, col)->setText(QString("%L1").arg(val));
}

void StatisticOutput::set_table_text(int row, int col, quint32 val)
{
    ui->tableWidget->item(row, col)->setText(QString("%L1").arg(val));
}

void StatisticOutput::set_table_text(int row, int col, double val)
{
    ui->tableWidget->item(row, col)->setText(QString("%L1").arg(val));
}

void StatisticOutput::on_checkBox_toggled(bool checked)
{
    show_all = checked;
    update_column_visible();
}

void StatisticOutput::set_run_state(bool _run)
{
    run = _run;
}
