//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CLIENT_H
#define CLIENT_H

#include <QHostAddress>

#include "RunStates.h"
#include "remote-control-server/types.h"
#include "ClientConfig.h"

class QTcpSocket;
class QTimer;
class RootConfig;

Q_DECLARE_METATYPE(QHostAddress)


class Client : public QObject
{
    Q_OBJECT
public:
    Client(int i, QString _progType, QString _progIndex);

    int getClientId() const { return id; }
    QString getProgType() const { return progType; }
    QString getProgIndex() const { return progIndex; }
    QString getIdentStr() const { return QString("Client '%1 #%2'")
                .arg(progType, progIndex); }
    QString getHostAddr() const { return hostAddr.toString(); }
    quint16 getHostPort() const { return hostPort; }
    static QString getProtocolIdStr(CLIENT_PROTO_ID id, bool shortStr = true)
    {
        switch (id) {
        case CLIENT_PROTO_V1:
            return shortStr ? "v1" : "v1 ASCII str";
        case CLIENT_PROTO_V2:
            return shortStr ? "v2" : "v2 Short sync-word";
        case CLIENT_PROTO_V3:
            return shortStr ? "v3" : "v3 Long sync-word";
        default:
            return shortStr ? "unk": "unknown";
        }
    }

public slots:
    void sendRunState(RUN_STATES state, QString _runIndex, int _runNumber);
    void startReadOut(int i);
    void stopReadOut(int i);
    void deleteClient(int i);
    void setAddress(int i, QHostAddress, quint16 port);
    void setEmptyAddress(int i);
    void evNumRequest(int checkIndex);

private slots:
    void reconnectToServer();
    void readCommandResponce();
    void displayError(QAbstractSocket::SocketError socketError);
    void disconnected();
    void checkData();

signals:
    void descrUpdated(int id, QString cmd);
    void statusUpdated(int id, QString status);
    void ready4Run(int id, bool read4Run);
    void runError(QString runIndex, int runNumber, QString errStr);
    void runWarning(QString runIndex, int runNumber, QString errStr);
    void backPressure(QString runIndex, int runNumber, bool backPressure);
    void vmeFlag(int id, bool hasVme);
    void runStateUpdated(int id, RUN_STATES state, QString runIndex, int runNumber);
    void evNumResponse(int checkIndex, int clientIndex, const EvNumSet &evNum) const;
    void config_transfer(int id, const RootConfig &root) const;
    void protocolChanged(int id, CLIENT_PROTO_ID protoId) const;

private:
    void sendCommand(const QString &mess);
    void sendString(const QString &mess);
    void send_byte_array(quint32 type, QByteArray data);
    bool isConnectionAlive() const;
    bool processIncomingMsg();
    bool processIncomingMsgV1();
    bool processIncomingMsgV23();
    bool command_line_parse(QByteArray data);
    void config_parse(QByteArray data);

    const int id; // clientId from config
    QString progType;
    QString progIndex;
    bool enabled;
    QTcpSocket * const tcpSocket;
    QHostAddress hostAddr;
    quint16 hostPort;
    QTimer *reconnectTimer;
    int connectingAttempts;
    QString descrStr;
    QString statusStr;
    bool run;
    QString runIndex;
    int runNumber;
    qint32 block_size;

    //--- old protocol variables
    CLIENT_PROTO_ID protocolId = CLIENT_PROTO_UNKNOWN;
    QByteArray recivedData;
    QTimer *dataTimer;
    bool dataTimerTimeout=false;
    quint64 curruptedSize=0;
};

#endif // CLIENT_H
