/*
**    Copyright 2006-2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "MlinkDeviceInterfaceDebug.h"

#include <cerrno>
#include <QDebug>

namespace mlink {

// Print RAW data (hexadecimal dump)
void MlinkDeviceInterfaceDebug::ml_print_raw_buffer(const void *ptr, size_t size) const
{
    Q_ASSERT(ptr);
    if (!ptr) return;
    const unsigned int *data = (const unsigned int *)ptr;
    qDebug() << QString("Raw dump for %1 bytes (%2 words):").arg(size).arg(size/4);
    size /= 4;
    QString s;
    for (size_t i=0;i<size;i++) {
        if (i%8==0)
            s += QString("  [%1]: ").arg(i, 4, 16, QChar('0'));
        s += QString("%1 ").arg(data[i], 8, 16, QChar('0'));
        if (i%8==7) s += "\n";
    }
    s += "\n";
    qDebug() << s;
}

// ETIMERECV is not a link error, it is normal
void MlinkDeviceInterfaceDebug::print_link_error(int ret, const QString &text) const
{
    bool is_link_err = (-ERR_TIMESEND == ret || -ETIME == ret || -ERR_REMOTEIO == ret);
    if (is_link_err)
        qWarning() << QString("link error: %1").arg(text);
    else
        qDebug() << QString("%1: %2").arg(ml_strerror(-ret)).arg(text);
}

void MlinkDeviceInterfaceDebug::print_frame_error(int ret, const QString &text) const
{
    bool is_link_err = (-ERR_TIMESEND == ret || -ETIME == ret || -ERR_REMOTEIO == ret);
    if (is_link_err)
        qWarning() << QString("frame error: %1").arg(text);
    else
        qDebug() << QString("%1: %2").arg(ml_strerror(-ret)).arg(text);
}

} // namespace mlink
