
#ifndef REGIO_FRAME_UTIL_H
#define REGIO_FRAME_UTIL_H

#include <cstddef>

#include <QtGlobal>

namespace mlink {

    class MlinkFrame
    {
    public:
        enum { // bit fields
            ML_FRAME_VALIDATE_OK = 0,
            ML_FRAME_VALIDATE_SYNC = 0x1,
            ML_FRAME_VALIDATE_LEN = 0x2,
            ML_FRAME_VALIDATE_CRC = 0x4,
        };
        MlinkFrame(const quint32 *frameData, size_t frameLength);
        MlinkFrame() = delete;
        MlinkFrame(const MlinkFrame&) = delete;
        MlinkFrame &operator=(const MlinkFrame&) = delete;

        bool isValid() const;
        int checkStructure() const;
        //! Print detailed raw frame contents
        void frame_print (bool rx_not_tx) const;
        void print_ctrlreq_frame () const;
        void print_ctrlack_frame () const;
        void print_status_frame () const;

        static int checkStructure(const quint32 *frameData, size_t frameLength);
        static void swap_endian (quint32 *frameData, size_t frameLength);
        static void print_raw_frame(const quint32 *frameData, size_t frameLength);
        static quint32 ml_make_crc (const void *buf, int len);

    private:
        const quint32 * const f;
        size_t len;
    };

} // namespace mlink

#endif /* REGIO_FRAME_UTIL_H */
