
#include "mlinkpacketprotocol.h"

#include "regiopacket.h"
#include "mlink/frame_types.h"
#include <cmath>
#include <cstdlib>

namespace mlink {

    double MlinkPacketProtocol::getTimeout()
    {
        return 0.1; // seconds
    }

    int MlinkPacketProtocol::getMaxRetryCount()
    {
        return 8;
    }

    double MlinkPacketProtocol::random_delay(int retry_count)
    {
        return (1.0 + 0.1*(rand() / (RAND_MAX + 1.0))) * pow(1.414, retry_count);
    }

    unsigned long MlinkPacketProtocol::ackSize(const RegIoPacket &tx)
    {
        const size_t txSize = tx.data.size();
        switch (tx.type) {
        case FRAME_TYPE_LOOPTEST: return txSize; //-V525
        case FRAME_TYPE_CTRL_REQ: return txSize;
        case FRAME_TYPE_FVME_REQ: return txSize; // FIXME
        case FRAME_TYPE_EEPROM_REQ: return 2;
        case FRAME_TYPE_RAM_REQ:{
                const PacketData &data = tx.data;
                if (data.size() < 1) return 0; // invalid
                unsigned int blockSize = (data[0] >> 22) & 0x1FF;
                return 1 + blockSize; // header+data
            }
        case FRAME_TYPE_JTAG_SINGLE: return txSize;
        case FRAME_TYPE_JTAG_BLOCK: return txSize;
        default: return 0;
        }
    }

    unsigned int MlinkPacketProtocol::ackType(const RegIoPacket &tx)
    {
        switch (tx.type) {
        case FRAME_TYPE_LOOPTEST: return FRAME_TYPE_LOOPTEST;
        case FRAME_TYPE_CTRL_REQ: return FRAME_TYPE_CTRL_ACK;
        case FRAME_TYPE_FVME_REQ: return FRAME_TYPE_FVME_ACK;
        case FRAME_TYPE_EEPROM_REQ: return FRAME_TYPE_EEPROM_ACK;
        case FRAME_TYPE_RAM_REQ: return FRAME_TYPE_RAM_ACK;
        case FRAME_TYPE_JTAG_SINGLE: return FRAME_TYPE_JTAG_SINGLE_ACK;
        case FRAME_TYPE_JTAG_BLOCK: return FRAME_TYPE_JTAG_BLOCK_ACK;
        default:
            return 0;
        }
    }

} // namespace mlink
