#ifndef REGIO_REGIOBASE_H
#define REGIO_REGIOBASE_H

#include <cstddef>
#include <string>

#include <QtGlobal>

#include "regio/regiopacket.h"

#if ((__GNUC__ > 4) || (__GNUC__ == 4 && __GNUC_MINOR__ >= 1))
#define WARN_UNUSED  __attribute__ ((warn_unused_result))
#else
#define WARN_UNUSED
#endif

namespace mlink {

struct DeviceDriverVersion {
    bool isNull() { return !major && !minor && !patch; }
    unsigned int major = 0;
    unsigned int minor = 0;
    unsigned int patch = 0;
};

    class RegIOBase {
    public:
        RegIOBase(const std::string &name = std::string()); ///< Construct M-Link Register I/O object with specified device name
        virtual ~RegIOBase();
        virtual void reset();
        virtual void rawSend(const RegIoPacket &data) = 0; ///< \deprecated use sendPackets()
        virtual void sendPackets(const PacketList &);
        virtual size_t getCtrlPacketCount() const; ///< return number of control packets received
        virtual size_t getDataSize() const; ///< return number of data words received
        PacketList getCtrlPacketBySeq(int seq, double timeout);
        virtual PacketList getCtrlPackets(size_t packetCount, double timeout) WARN_UNUSED;

        void receive();
        double getPollInterval() const;
        RawPacketList popDataPackets() { RawPacketList pl = rxBufData; rxBufData.clear(); return pl; }
        void setBufSizeLimit(quint32 numPackets) { bufSizeLimit = numPackets; }

        std::string getDevName() const;
        DeviceDriverVersion getDriverVersion() const;
        virtual bool isConnected() = 0;

    protected:
        virtual double getDeviceThroughput() const;
        virtual qint64 getRxBufSizeLimit() const;
        void setDevName(const std::string &name) {devName = name;}
        virtual RawPacketList getRxPackets(double timeout) = 0; ///< get packets from hardware
        void demuxRxBuf(const RawPacketList&);
        static bool packetTypeIsData(int type);

        std::string devName;
        DeviceDriverVersion driverVersion;

        int bufSizeLimit = 0;
        PacketList rxBufCtrl;
        RawPacketList rxBufData;
    };

} // namespace mlink

#endif /* REGIO_REGIOBASE_H */
