//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef REGIO_REGIOPACKET_H
#define REGIO_REGIOPACKET_H

#include <vector>
#include <deque>
#include <string>

#include <QtGlobal>
#include <QMetaType>
#include <QList>
#include <QByteArray>

class QTextStream;
class QDebug;

namespace mlink {

    typedef quint32 PacketRawDataType;
    typedef std::vector<PacketRawDataType> PacketData;

    class RegIoPacket {
    public:
        RegIoPacket(const RegIoPacket &r) = default;
        RegIoPacket() = default;
        RegIoPacket(quint16 _type, quint16 _seq, quint16 _src, quint16 _dst, const PacketData &_data)
            : type(_type), seq(_seq), src(_src), dst(_dst), data(_data) { }
        explicit RegIoPacket(const QByteArray &);
        void dump(const std::string &description = std::string()) const;
        QByteArray toByteArray() const;
        quint16 type = 0;
        quint16 seq = 0;
        quint16 src = 0;
        quint16 dst = 0;
        PacketData data;
    };
    //Q_DECLARE_METATYPE(RegIoPacket);


    std::ostream &operator<< (std::ostream & os, const RegIoPacket &p);
    QTextStream &operator<< (QTextStream & os, const RegIoPacket &p);
    QDebug operator<< (QDebug os, const RegIoPacket &p);

    class RawPacketList: public QList<QByteArray> {};
    class PacketList: public QList<RegIoPacket> {};

    typedef std::vector<PacketRawDataType> PacketRawData;

    std::ostream &operator<< (std::ostream & os, const PacketRawData &p);

} // namespace mlink

#endif // REGIO_REGIOPACKET_H
