#ifndef _MLINK_H
#define _MLINK_H 1

#define ML_DEVICE_NAME "mlink"

#ifdef __KERNEL__
#define ML_PROCFS_NAME ML_DEVICE_NAME
#define DRV_NAME "melink"
#define DEVICE_MAJOR 0 /* Set to 0 to be dynamically assigned */
#endif

/*
 * Using M-Link Specification version 1.0
 * http://afi.jinr.ru/pub/doc/M-Link/M-Link%20Data%20Interface%20Manual.html
 *
 * M-Link data layer operates with 4-byte words only.
 * Physical layer may be 16 bits wide or any other.
 */

enum { ML_UDP_PORT = 33300 };
enum { ML_UDP_RAWDATA_PORT = 33301 };

enum { ML_RX_BUFFER_SIZE = 16384 }; // bytes
enum { ML_RX_BUFFER_WORDS = ML_RX_BUFFER_SIZE/4 }; // words


typedef struct {
    void *buf; // virtual address
    unsigned long size; // size
    int timeout; // timeout in ms, -1 for infinite, 0 for immediate
} ml_dmaop_t;

typedef struct {
    const void *buf; // virtual address
    unsigned long size; // size
    int timeout; // timeout in ms, -1 for infinite, 0 for immediate
} ml_dmaop_ro_t;

typedef struct {
    unsigned int timeout_ms;
    unsigned int mask;
} ml_poll_ev_op_t;

// Events, bit fields. May be OR-ed.
enum {
    ML_EV_BIT_SPILL_ON  = 0x0001,
    ML_EV_BIT_SPILL_OFF = 0x0002,
    ML_EV_BIT_DATA = 0x0004,
    ML_EV_BIT_RX_PKT = 0x0008,
};

#pragma pack(push, 1)
typedef union {
    unsigned int all;
    struct {
        unsigned int link_down: 1; // no connection
        unsigned int data_error: 1; // link data error
        unsigned int pci_error: 1; // pci bus signaled error
        unsigned int remote_error: 1; // error propagation
        unsigned int no_tx_clock: 1; // no clock from remote
        unsigned int no_rx_clock: 1; // no clock from remote
        unsigned int ccb_error_unavailable: 1;
        unsigned int rx_wordalign_error: 1; // word alignment error on receive (remote error)
    } bit;
} mlink_status_t;

typedef union {
    unsigned int all;
    struct {
        unsigned int int_loop_test_unavailable : 1;
        unsigned int serdes_loop_test : 1;
        unsigned int tx_cost : 10;
        unsigned int rx_cost : 10;
        unsigned int unused : 10;
    } bit;
} mlink_caps_t;
#pragma pack(pop)

#endif /* _MLINK_H */
