#ifndef REGIO_REGIOBASE_H
#define REGIO_REGIOBASE_H

#include <cstddef>
#include <string>

#include <QtGlobal>
#include <QObject>
#include <QHostAddress>

#include <regio/regiopacket.h>

namespace mlink {

class RegIOBase: public QObject
{
    Q_OBJECT

public:
    RegIOBase(QObject *parent, const QHostAddress &address = QHostAddress()); ///< Construct M-Link Register I/O object with specified device address
    ~RegIOBase() override = default;
    void setHostAddress(const QHostAddress &address);
    QHostAddress getHostAddress() const { return deviceAddress; }
    virtual void reset();
    virtual void rawSend(const RegIoPacket &data) = 0; ///< \deprecated use sendPackets()
    virtual void sendPackets(const PacketList &);
    virtual size_t getCtrlPacketCount() const; ///< return number of control packets received
    RegIoPacket getCtrlPacketBySeq(int seq, double timeoutSec);
    virtual PacketList getCtrlPackets(size_t packetCount, double timeout) Q_REQUIRED_RESULT;
    virtual unsigned long getMaxDataSize() const = 0;

    std::string getDevName() const;
    unsigned long getDriverVersion() const;
    unsigned long getDriverRevision() const;

Q_SIGNALS:
    void rawDataReceived(const QByteArray &);

protected:
    virtual double getDeviceThroughput() const;
    bool packetTypeIsData(int type);
    virtual bool updateRxBuf(double timeoutMs) = 0;

    QHostAddress deviceAddress;

    int bufSizeLimit = 0;
    PacketList rxBufCtrl;
};

double getCurrentTimestamp();

} // namespace mlink

#endif /* REGIO_REGIOBASE_H */
