#include "CMSubDialog.h"
#include "ui_CMSubDialog.h"

#include <QDebug>
#include <QDesktopWidget>
#include <QJsonDocument>

CMSubDialog::CMSubDialog(TYPE _type, RootConfig root,
                         QString _program_type,
                         QString _program_index,
                         QString _cfg_name,
                         QWidget *parent)
    :
      QDialog(parent),
      ui(new Ui::CMSubDialog),
      type(_type),
      root_config(root),
      program_type(_program_type),
      program_index(_program_index),
      configuration_name(_cfg_name),
      program_index_entered(program_index),
      configuration_name_entered(configuration_name)
{
    ui->setupUi(this);

    if(type != TYPE_RENAME){
        configuration_name_entered.append("(1)");
        ui->button_box_ok_cansel->button(QDialogButtonBox::Ok)->setText(typeStr());
    }

    config_map = base_config.get_config_map(_program_type);
    ui->push_button_and_open->setText(typeStr() + " and open");
    setWindowTitle(type==TYPE_NEW ? "New default configuration"
                                  : typeStr()+" configuration");
    ui->line_edit_program_index->setText(program_index_entered);
    ui->line_edit_config_name->setText(configuration_name_entered);
    check_input_validity();
}

CMSubDialog::~CMSubDialog()
{
    delete ui;
}

void CMSubDialog::on_line_edit_program_index_textEdited(const QString &arg1)
{
    program_index_entered = arg1.trimmed();
    check_input_validity();
}

void CMSubDialog::on_line_edit_config_name_textChanged(const QString &arg1)
{
    configuration_name_entered = arg1.trimmed();
    check_input_validity();
}

void CMSubDialog::check_input_validity()
{
    bool catch_up = program_index_entered.isEmpty() || configuration_name_entered.isEmpty();

    if(type!=TYPE_RENAME || program_index_entered != program_index || configuration_name_entered != configuration_name) {
        catch_up = config_map.value(program_index_entered).contains(configuration_name_entered);
    }

    if(type != TYPE_RENAME){
        ui->button_box_ok_cansel->button(QDialogButtonBox::Ok)->setEnabled(!catch_up);
    }

    ui->push_button_and_open->setEnabled(!catch_up);

    QPalette palette;
    palette.setColor(QPalette::Base, catch_up ? QColor(240,128,128) : Qt::white);
    palette.setColor(QPalette::Text,Qt::black);
    ui->line_edit_program_index->setPalette(palette);
    ui->line_edit_config_name->setPalette(palette);
}

QString CMSubDialog::typeStr() const
{
    switch (type) {
    case TYPE_NEW: return "Create";
    case TYPE_RENAME: return "Rename";
    case TYPE_CLONE: return "Clone";
    }
    return QString();
}

void CMSubDialog::on_button_box_ok_cansel_accepted()
{
    if(type == TYPE_RENAME) {
        if(program_index_entered == program_index
                && configuration_name_entered == configuration_name) {
            return;
        }
        base_config.transform_to_phantom(root_config);
    } else {
        root_config.refresh_oid_all();
    }

    root_config.set_data("program_index", program_index_entered);
    root_config.set_data("configuration_name", configuration_name_entered);

    base_config.write_config(root_config);
}


void CMSubDialog::on_push_button_and_open_clicked()
{
    on_button_box_ok_cansel_accepted();
    emit switch_to(program_index_entered, configuration_name_entered);
    QDialog::done(QDialog::Accepted);
}
