﻿#ifndef CONFIGURATIONMANAGER_H
#define CONFIGURATIONMANAGER_H

#include <QWidget>
#include <QDialog>
#include <QLineEdit>
#include <QTableWidgetItem>
#include <QTreeWidgetItem>
#include <QShortcut>

#include "mongo/RootConfig.h"
#include "mongo/BaseConfig.h"
#include "CMSubDialog.h"

namespace Ui {
class ConfigurationManager;
}

struct root_item{
    RootConfig root;
    QTreeWidgetItem *item;
};

class ConfigurationManager : public QDialog
{
    Q_OBJECT
public:
    explicit ConfigurationManager(
        QWidget *parent = nullptr,
        QString program_type_ = "",
        RootConfig default_root_ = RootConfig()
     );
    ~ConfigurationManager();
    static void populate_tree(RootConfig root, QTreeWidget *tree);
    void reset(QString program_index, QString configuration_name_);

signals:
    void switch_to_config(QString, QString);

private slots:
    void on_pushButton_new_clicked();
    void on_pushButton_rename_clicked();
    void on_pushButton_clone_clicked();
    void on_pushButton_delete_clicked();
    void on_pushButtonRefresh_clicked();
    void switch_to(QString program_index, QString configuration_name);
    void on_pushButton_switch_to_clicked();
    void on_pushButton_close_clicked();
    void config_changed(QTreeWidgetItem *current, QTreeWidgetItem *);
    void on_tree_left_itemDoubleClicked(QTreeWidgetItem *item, int column);

private:
    void exec_sub_dialog(CMSubDialog::TYPE type);
    void populate_left_tree(QTreeWidget *tree);
    void create_subdialog_delete();
    Ui::ConfigurationManager *ui;
    BaseConfig base_config;

    QString program_type;
    QString program_index;
    QString configuration_name;

    QString program_index_selected;
    QString configuration_name_selected;

    RootConfig default_root;
    RootConfig selected_root;
    QMap<QString, QStringList> config_map;
};

#endif // CONFIGURATIONMANAGER_H
