#include "DaqConfig.h"
#include <QCoreApplication>
#include <QDebug>

DaqConfig::DaqConfig(DaqConfigTypes t, QString i):
    globalSet(QSettings::IniFormat, QSettings::UserScope,
              QCoreApplication::organizationName(),
              "daq_config"),
    localSet(QSettings::IniFormat, QSettings::UserScope,
             QCoreApplication::organizationName(),
             qAppName()),
    index(i),
    type(t)
{
    setIndex(index);
}

void DaqConfig::setIndex(QString i)
{
    if(!index.isEmpty()){
        //    endGroup();
            while (!globalSet.group().isEmpty()) {
        //        qWarning()<<"DaqConfig wasn't set to root group befor setIndex";
                globalSet.endGroup();
            }
    }
    index = i.trimmed();
    if(!index.isEmpty()) {
        globalSet.beginGroup(QString("%1_%2").arg(getTypeName()).arg(index));
    }
}

QString DaqConfig::getTypeName() const
{
    return getTypeName(type);
}

QString DaqConfig::getTypeName(DaqConfigTypes t)
{
    switch (t) {
    case DaqConfigAdc64:        return "Adc64";
    case DaqConfigAdc64System:  return "Adc64System";
    case DaqConfigEventBuilder: return "EvB";
    case DaqConfigGenerator:    return "Generator";
    case DaqConfigScheduler:    return "Scheduler";
    case DaqConfigHrbAnalyzer:  return "HrbAnalyzer";
    case DaqConfigHrbSystem:    return "HrbSystem";
    case DaqConfigMStream:      return "MStream";
    case DaqConfigRunControl:   return "RunControl";
    case DaqConfigTdcControl:   return "TDC2-CTRL";
    case DaqConfigTdcViewer:    return "TDC2-VIEWER";
    case DaqConfigTQDC:         return "TQDC2";
    case DaqConfigIdeas:        return "Ideas";
    case DaqConfigVmeDaq:       return "VmeDaq";
    case DaqConfigTtvxs:        return "TTVXS";
    case DaqConfigLand:         return "LAND";
    }
    return QString();
}

QStringList DaqConfig::get_all_types()
{
    QStringList to_return;
    to_return << getTypeName(DaqConfigAdc64)
              << getTypeName(DaqConfigAdc64System)
              << getTypeName(DaqConfigEventBuilder)
              << getTypeName(DaqConfigGenerator)
              << getTypeName(DaqConfigScheduler)
              << getTypeName(DaqConfigHrbAnalyzer)
              << getTypeName(DaqConfigHrbSystem)
              << getTypeName(DaqConfigMStream)
              << getTypeName(DaqConfigRunControl)
              << getTypeName(DaqConfigTdcControl)
              << getTypeName(DaqConfigTdcViewer)
              << getTypeName(DaqConfigTQDC)
              << getTypeName(DaqConfigIdeas)
              << getTypeName(DaqConfigVmeDaq)
              << getTypeName(DaqConfigTtvxs)
              << getTypeName(DaqConfigLand);
    return to_return;
}

void DaqConfig::eraseSettings()
{
    getSettings().remove("");
}
