#include "DaqDataBase.h"
#include <QDebug>

#include <QSqlQuery>
#include <QSqlError>
#include <QSettings>
#include <QThread>

namespace {
const QString DAQ_DB_HOST = "daq-db.he.jinr.ru";
const QString DAQ_DB_NAME = "daq";
const QString DAQ_DB_USER = "bmndaq";
const QString DAQ_DB_PASS = "0yAiikTGjRRHiyvZ";
}

DaqDataBase::DaqDataBase(QObject *p) : QObject(p)
{
    if(!QSqlDatabase::drivers().contains("QMYSQL", Qt::CaseInsensitive)) {
        qCritical() << "Database driver QMYSQL not loaded.";
        emit setStatus("Database driver QMYSQL not loaded", DB_STATUS_ERROR);
        return;
    }

    QSettings settings;

    for (QString section : settings.value("MainWindow/databases").toStringList()){
        if(section.isEmpty())
            continue;

        settings.beginGroup(section);
        if(settings.contains("DbName") && settings.contains("login"))
        {
            QSqlDatabase db = QSqlDatabase::addDatabase("QMYSQL", section);
            db.setHostName(settings.value("HostName").toString());
            db.setDatabaseName(settings.value("DbName").toString());
            db.setUserName(settings.value("login").toString());
            db.setPassword(settings.value("passwd").toString());
            db.setConnectOptions("MYSQL_OPT_CONNECT_TIMEOUT=5");
            db.setConnectOptions("MYSQL_OPT_RECONNECT=1");
            if (!db.isValid()) {
                qCritical() << "Database driver not loaded.";
            }
            if(db.databaseName().isEmpty() || db.userName().isEmpty()){
                QSqlDatabase::removeDatabase(section);
            }
        }
        settings.endGroup();
    }

    if(!getConnection().isOpen()){
        QString section = getPreferedSection();
        QSqlDatabase db = QSqlDatabase::addDatabase("QMYSQL", section);
        db.setHostName(DAQ_DB_HOST);
        db.setDatabaseName(DAQ_DB_NAME);
        db.setUserName(DAQ_DB_USER);
        db.setPassword(DAQ_DB_PASS);
        db.setConnectOptions("MYSQL_OPT_CONNECT_TIMEOUT=5");
        db.setConnectOptions("MYSQL_OPT_RECONNECT=1");
        if (!db.isValid()) {
            qCritical() << "Database driver not loaded.";
        }
        if(db.databaseName().isEmpty() || db.userName().isEmpty()){
            QSqlDatabase::removeDatabase(section);
        }
        selectedConnection = section;
    }

    emit setStatus("Not connected", DB_STATUS_OK);
}


QSqlDatabase DaqDataBase::getConnection()
{
    // Check previous connection
    if( !selectedConnection.isEmpty() )
    {
        QSqlDatabase db = QSqlDatabase::database(selectedConnection);

        if( db.isOpen() )
            return db;
    }
    // Search other connections
    const QString preferedSection = getPreferedSection();
    for (QString connection : QSqlDatabase::connectionNames())
    {
        if( connection == selectedConnection )
            continue;
        if( connection != preferedSection )
            continue;

        QSqlDatabase db = QSqlDatabase::database( connection );
        if( db.isOpen() )
        {
            selectedConnection = connection;
            return db;
        }
    }
    // No database available
    selectedConnection.clear();
    return QSqlDatabase::database();
}


bool DaqDataBase::getDataBaseStatus()
{
    return getConnection().isOpen();
}

QSqlError DaqDataBase::getLastDbError()
{
    return QSqlDatabase::database(selectedConnection, false).lastError();
}

QString DaqDataBase::toString()
{
    QSqlDatabase db = QSqlDatabase::database(selectedConnection, false);
    return QString("%1: %2@%3").arg(selectedConnection, db.databaseName(), db.hostName());
}

QVector<int> DaqDataBase::selectQuery(QSqlDatabase &db, QString table, QMap<QString, QVariant> vals) const
{
    QVector <int> ids;
    QSqlQuery query(db);
    QStringList conditions;
    for (QString col : vals.keys())
        conditions << QString("%1=:%1").arg(col);

    query.prepare(QString("SELECT id FROM %1 WHERE %2 ORDER BY id DESC").arg(table).arg(conditions.join(" AND ")));
    for (QString col : vals.keys())
        query.bindValue(QString(":%1").arg(col), vals[col]);

    if (query.exec()) {
        while (query.next()) {
            bool ok;
            int id = query.value(0).toInt(&ok);
            if(ok)
                ids.push_back(id);
        }
    } else {
        qWarning() << QString("Failed to make select from %1 table: %2")
                      .arg(table)
                      .arg( query.lastError().text() );
    }
    return ids;
}

bool DaqDataBase::insertQuery(QSqlDatabase &db, QString table, QMap<QString, QVariant> vals, int *id) const
{
    QSqlQuery query(db);
    QStringList columns;
    QStringList params;

    for (QString col : vals.keys()) {
        columns << col;
        params << QString(":%1").arg(col);
    }

    query.prepare(QString("INSERT INTO %1 (%2) VALUES (%3)")
                  .arg(table)
                  .arg(columns.join(", "))
                  .arg(params.join(", ")));
    for (QString col : vals.keys())
        query.bindValue(QString(":%1").arg(col), vals[col]);

    if (query.exec()) {
        if(id)
            *id = query.lastInsertId().toInt();
//        qInfo() << QString("DB record were saved to %1").arg(table);
        return true;
    } else {
        qWarning() << QString("Failed to make record into %1 table: %2")
                      .arg(table)
                      .arg( query.lastError().text() );
        qWarning() << db.lastError().text();
        return false;
    }
}

QString DaqDataBase::getPreferedSection() const
{
    return "test" + QString::number(reinterpret_cast<quint64>(QThread::currentThread()), 10);
}
