#include "RootConfig.h"
#include <QJsonDocument>
#include <QJsonObject>
#include <QDebug>

RootConfig::RootConfig(QString item_name_str):
    id(QUuid::createUuid()),
    item_name(item_name_str)
{
    #ifdef MONGO_DRIVERS
        oid = bsoncxx::oid();
    #endif
}

void RootConfig::append_child(const RootConfig &item)
{
    child_items[item.item_name]=item;
}

RootConfig RootConfig::child(const QString &child_name) const
{
    return child_items.value(child_name);
}

RootConfig &RootConfig::child(const QString &child_name)
{
    return child_items[child_name];
}

QMap<QString, RootConfig> RootConfig::children() const
{
    return child_items;
}

const QMap<QString, RootConfig> &RootConfig::children_const() const
{
    return child_items;
}

int RootConfig::children_count() const
{
     return child_items.count();
}

QMap<QString, QVariant> RootConfig::data() const
{
    return item_data;
}

const QMap<QString, QVariant> &RootConfig::data_const() const
{
    return item_data;
}

void RootConfig::replace_data_from(const QMap<QString, QVariant> &source_data)
{
    item_data = source_data;
}

void RootConfig::remove_data(const QString &key)
{
    item_data.remove(key);
}

void RootConfig::remove_only_data()
{
    item_data.clear();
}

void RootConfig::remove_child(const QString &key)
{
    child_items.remove(key);
}

void RootConfig::remove_all_data()
{
    child_items.clear();
    item_data.clear();
}

QJsonDocument RootConfig::get_json() const
{
    QJsonObject j_object;
    for(QString key : item_data.keys()){
        j_object[key] = QJsonValue::fromVariant(item_data[key]);
    }

    for(QString key : child_items.keys()){
        j_object[key] = jobject_from_root(child_items[key]);
    }

    QJsonDocument j_document(j_object);
    return  j_document;
}

QByteArray RootConfig::get_json_data() const
{
    return get_json().toBinaryData();
}

QJsonObject RootConfig::jobject_from_root(const RootConfig &root_config) const
{
    QJsonObject j_object;
    for(QString key : root_config.item_data.keys()){
        j_object[key] = QJsonValue::fromVariant(root_config.item_data[key]);
    }

    for(QString key : root_config.child_items.keys()){ 
        j_object[key] = jobject_from_root(root_config.child_items[key]);
    }
    return j_object;
}

void RootConfig::set_root_from_json(const QJsonDocument &j_document)
{
    remove_all_data();
    QJsonObject j_object = j_document.object();
    for(QString key : j_object.keys()){
        if(!j_object.value(key).isObject()){
            item_data[key] = j_object.value(key).toVariant();
        }
        else{
            child_items[key] = root_from_jobject(j_object.value(key).toObject());
            child_items[key].item_name = key;
        }
    }
}

void RootConfig::refresh_oid()
{
    #ifdef MONGO_DRIVERS
        oid = bsoncxx::oid();
        id = QUuid::createUuid();
#endif
}

void RootConfig::refresh_oid_all()
{
    refresh_oid();
    for(auto key : child_items.keys()) {
        child_items[key].refresh_oid_all();
    }
}

void RootConfig::print_json()
{
    qInfo().noquote().nospace() << QString(get_json().toJson(QJsonDocument::Indented));
}

RootConfig RootConfig::root_from_jobject(const QJsonObject &j_object)
{
    RootConfig root;
    for(QString key : j_object.keys()){
        if(!j_object.value(key).isObject()){
            root.item_data[key] = j_object.value(key).toVariant();
        }
        else{
            RootConfig child_root;
            child_root = root_from_jobject(j_object.value(key).toObject());
            child_root.item_name = key;
            root.append_child(child_root);
        }
    }
    return root;
}

