#ifndef ROOTCONFIG_H
#define ROOTCONFIG_H
#include <QDateTime>
#include <QString>
#include <QVariant>
#include <QUuid>
#include <QAbstractItemModel>

#ifdef MONGO_DRIVERS
#ifndef Q_MOC_RUN
#include <bsoncxx/types.hpp>
#endif
#endif

class RootConfig
{
public:
    RootConfig(QString item_name_str = "");
    void append_child(const RootConfig &child);
    RootConfig child(const QString &child_name) const;
    RootConfig& child(const QString &child_name);
    QMap<QString, RootConfig> children() const;
    const QMap<QString, RootConfig> &children_const() const;
    int children_count() const;
    QMap<QString, QVariant> data() const;
    const QMap<QString, QVariant> &data_const() const;
    void replace_data_from(const QMap<QString, QVariant> &source_data);
    void remove_data(const QString &key);
    void remove_only_data();
    void remove_child(const QString &key);
    void remove_all_data();
    QJsonDocument get_json() const;
    QByteArray get_json_data() const;
    void set_root_from_json(const QJsonDocument &j_document);
    bool isEmpty(){return child_items.isEmpty() && item_data.isEmpty();}
    bool is_phantom(){return !get_delete_time().isNull();}
    void refresh_oid();
    void refresh_oid_all();
    void print_json();

    QDateTime get_modification_time() { return get_data<QDateTime>("date_of_modification"); }
    QDateTime get_delete_time() { return get_data<QDateTime>("delete_datetime"); }

    template< typename T >
    void set_data( const QString& key, const T& val ) {
        item_data[key] = QVariant::fromValue( val );
    }

    template< typename T >
    T get_data( const QString& key, const T& defVal = T() ) const {
        return unpack< T >( item_data.value(key), defVal );
    }

    friend bool operator != (const RootConfig &root, const RootConfig &root2)
    { return !(root == root2); }
    friend bool operator == (const RootConfig &root, const RootConfig &root2)
    {
        return root.item_data == root2.item_data
                && root.child_items == root2.child_items
                && root.item_name == root2.item_name;
    }

    QUuid id;
    #ifdef MONGO_DRIVERS
        bsoncxx::oid oid;
    #endif
    QString item_name;
private:
    template< typename T >
    static T unpack( const QVariant& var, const T& defVal = T() ) {
        if( var.isValid() && var.canConvert< T >() ) {
            return var.value< T >();
        }
        return defVal;
    }
    QJsonObject jobject_from_root(const RootConfig &root_config) const;
    RootConfig root_from_jobject(const QJsonObject &j_object);

    QMap<QString, RootConfig> child_items;
    QMap<QString, QVariant> item_data;
};
Q_DECLARE_METATYPE(RootConfig)

#endif // ROOTCONFIG_H

//=== Garbage ===
//    qInfo().noquote().nospace() << QString(root_config.get_json().toJson(QJsonDocument::Indented));
