#ifndef XMLDISCOVER_H
#define XMLDISCOVER_H

#include "PNPMessage.h"

#include <QObject>

class MultiCastListener;
class QTimer;
class QUdpSocket;

namespace {
const quint16 PNP_DISCOVER_PORT = 33304;
const QString PNP_DISCOVER_IP_ADDR = "239.192.1.2";
}

class PNPServer : public QObject
{
    Q_OBJECT
public:
    explicit PNPServer(QObject *parent = nullptr);
    virtual ~PNPServer();
    QString getSearchStr();
    static QVector<QHostAddress> getLocalHostAddresses();

public slots:
    void startSearch(QString target, bool single = true);
    void startSearch(QStringList searchTargets = QStringList(), bool single = true);
    void stopSearch();
    void changeProgramDescription(const ProgramDescription &pd);
    void updateProgramDescription(const ProgramDescription &pd);
    void closeProgram(const ProgramDescription &pd);

signals:
    void programDiscovered(const ProgramDescription &, int packetSize) const;
    void programClosed(const ProgramDescription &) const;

protected slots:
    virtual bool decode(const QByteArray &data, const QHostAddress &host);

private slots:
    void sendSearchRequest();

private:
    void sendProgDescr(ProgramDescription pd);
    void sendMulticastMsg(const QString &msg);
    void print(QString s) const;

    QList<ProgramDescription> progDesc;
    bool searchEnabled;
    QStringList searchTargets; // ProgramDescription::type
    QTimer * const searchTimer;
    MultiCastListener * const multiCastListener;
    QUdpSocket * const multiCastSender;
    QMap<QUuid, quint32> seqMap;
};

#endif // XMLDISCOVER_H
