#ifndef REMOTECONTROLSERVER_H
#define REMOTECONTROLSERVER_H

#include <QObject>
#include <QMetaType>
#include <QHostAddress>

#include "remote-control-server/types.h"

class QTcpServer;
class QTcpSocket;
class QTimer;
class ProgramDescription;

struct RemoteControlServerStatus
{
    RemoteControlServerStatus(quint16 p=0):
        preferedTcpPort(p),
        tcpPort(0),
        listening(false),
        portInUse(false),
        peerPort(0)
    {}

    bool operator ==(const RemoteControlServerStatus &st) const
    {
        return tcpPort == st.tcpPort && listening == st.listening && portInUse == st.portInUse
                && peerHost == st.peerHost && peerPort == st.peerPort;
    }

    bool operator !=(const RemoteControlServerStatus &st) const
    {
        return !(this->operator ==(st));
    }

    quint16 preferedTcpPort;
    quint16 tcpPort;
    bool listening;
    bool portInUse;
    QHostAddress peerHost;
    quint16 peerPort;
};
Q_DECLARE_METATYPE(RemoteControlServerStatus)

class RemoteControlServer : public QObject
{
    Q_OBJECT
public:
    explicit RemoteControlServer(quint16 port, QString _descr="");
    virtual ~RemoteControlServer();
    int getTcpPort() const { return status.tcpPort; }
    bool isFree() const { return !status.portInUse; }
    bool isListening() const { return status.listening; }

public slots:
    void completedCmd(bool success, QString cmd, int arg=0) const;
    void completedStart(bool success, QString runIndex, int runNumber) {
        completedStartStop(true, success, runIndex, runNumber);
    }
    void completedStop(bool success, QString runIndex, int runNumber) {
        completedStartStop(false, success, runIndex, runNumber);
    }
    void sendState(QString state, bool read4Run=false);
    void onRunError(QString runIndex, int runNumber, QString runError=QString());
    void onRunWarning(QString runIndex, int runNumber, QString warnStr);
    void onBackPressure(QString runIndex, int runNumber, bool backPressure);
    void onVmeFlag(bool hasVme);
    void startListen(bool state);
    void config_transfer(QByteArray data) const;
    void changeProgramDescription(const ProgramDescription &pd);
    void evNumCheckResponse(int checkIndex, const EvNumSet &evNums) const;

signals:
    void startRequest(QString globIndex, int runNum, quint64 startEvNum=0);
    void stopRequest(QString globIndex, int runNum, quint64 stopEvNum=0);
    void timeToStart(QString timeToStart);
    void stateRequest();
    void evNumCheckRequest(int checkIndex);
    void connectionStatusUpdated(const RemoteControlServerStatus &) const;

private slots:
    void updateConnections();
    void readCommandRequest();
    void onDisconnected();

private:
    void listen();
    void close();
    void sendDescription() const;
    bool isConnectionAlive() const;
    void completedStartStop(bool start, bool success, QString runIndex, int runNumber) const;
    void send_command_message(QString mess) const;
    void send_byte_array(quint32 type, QByteArray mess) const;
    void updateStatus() const;
    void command_line_parse(QByteArray data);

    QTcpServer * const qTcpServer;
    QTcpSocket *tcpSocket;
    QString descr;
    RemoteControlServerStatus status;

    QByteArray data;
};

#endif // REMOTECONTROLSERVER_H
