#include "ZmqDialog.h"
#include "ui_ZmqDialog.h"

#include "ZmqPublisher.h"

ZmqDialog::ZmqDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ZmqDialog)
{
    ui->setupUi(this);
}

ZmqDialog::~ZmqDialog()
{
    delete ui;
}

void ZmqDialog::setConfig(const ZmqConfig &config)
{
    cfg = config;
    block_change_signal = true;
    ui->checkBoxEnable->setChecked(cfg.enabled);
    ui->checkBoxTest->setChecked(cfg.test_mode);
    ui->spinBox->setValue(cfg.port);
    ui->checkBoxPrint->setChecked(cfg.print_messages);
    block_change_signal = false;
}

void ZmqDialog::on_checkBoxEnable_toggled(bool en)
{
    if(block_change_signal) return;
    cfg.enabled = en;
    emit zmq_cfg_change(cfg);
}

void ZmqDialog::on_checkBoxTest_toggled(bool en)
{

    if(block_change_signal) return;
    cfg.test_mode = en;
    emit zmq_cfg_change(cfg);
}

void ZmqDialog::on_spinBox_valueChanged(int p)
{
    if(block_change_signal) return;
    cfg.port = p;
    emit zmq_cfg_change(cfg);
}

void ZmqDialog::on_checkBoxPrint_toggled(bool print)
{
    if(block_change_signal) return;
    cfg.print_messages = print;
    emit zmq_cfg_change(cfg);
}

void ZmqDialog::statusUpdated(const ZmqStatus &s)
{
    QString str = s.working ? "Connected" : "Disconnected";
    QPalette palette = ui->labelConnected->palette();
    palette.setColor(ui->labelConnected->foregroundRole(), s.working ? Qt::darkGreen : Qt::red);
    ui->labelConnected->setPalette(palette);
    ui->labelConnected->setText(str);

    str.clear();
    if(s.working)
        str = QString("packages: %1").arg(s.packages);
    ui->labelStatus->setText(str);
}
